@@ -12,9 +12,10 @@ from sqlglot.optimizer.normalize_identifiers import normalize_identifiers
 
 from sqlmesh.cicd.config import CICDBotConfig
 from sqlmesh.core import constants as c
+from sqlmesh.core.console import get_console
 from sqlmesh.core.config import EnvironmentSuffixTarget
 from sqlmesh.core.config.base import BaseConfig, UpdateStrategy
-from sqlmesh.core.config.common import variables_validator
+from sqlmesh.core.config.common import variables_validator, compile_regex_mapping
 from sqlmesh.core.config.connection import (
     ConnectionConfig,
     DuckDBConnectionConfig,
@@ -29,17 +30,20 @@ from sqlmesh.core.config.model import ModelDefaultsConfig
 from sqlmesh.core.config.naming import NameInferenceConfig as NameInferenceConfig
 from sqlmesh.core.config.plan import PlanConfig
 from sqlmesh.core.config.run import RunConfig
-from sqlmesh.core.config.scheduler import BuiltInSchedulerConfig, SchedulerConfig
+from sqlmesh.core.config.scheduler import (
+    BuiltInSchedulerConfig,
+    SchedulerConfig,
+    scheduler_config_validator,
+)
 from sqlmesh.core.config.ui import UIConfig
 from sqlmesh.core.loader import Loader, SqlMeshLoader
 from sqlmesh.core.notification_target import NotificationTarget
 from sqlmesh.core.user import User
 from sqlmesh.utils.errors import ConfigError
-from sqlmesh.utils.pydantic import (
-    field_validator,
-    model_validator,
-    model_validator_v1_args,
-)
+from sqlmesh.utils.pydantic import field_validator, model_validator
+
+if t.TYPE_CHECKING:
+    from sqlmesh.core._typing import Self
 
 
 class Config(BaseConfig):
@@ -65,8 +69,9 @@ class Config(BaseConfig):
         loader_kwargs: Key-value arguments to pass to the loader instance.
         env_vars: A dictionary of environmental variable names and values.
         model_defaults: Default values for model definitions.
-        physical_schema_override: A mapping from model schema names to names of schemas in which physical tables for corresponding models will be placed.
+        physical_schema_mapping: A mapping from regular expressions to names of schemas in which physical tables for corresponding models will be placed.
         environment_suffix_target: Indicates whether to append the environment name to the schema or table name.
+        environment_catalog_mapping: A mapping from regular expressions to catalog names. The catalog name is used to determine the target catalog for a given environment.
         default_target_environment: The name of the environment that will be the default target for the `sqlmesh plan` and `sqlmesh run` commands.
         log_limit: The default number of logs to keep.
         format: The formatting options for SQL code.
@@ -98,7 +103,7 @@ class Config(BaseConfig):
     loader_kwargs: t.Dict[str, t.Any] = {}
     env_vars: t.Dict[str, str] = {}
     username: str = ""
-    physical_schema_override: t.Dict[str, str] = {}
+    physical_schema_mapping: t.Dict[re.Pattern, str] = {}
     environment_suffix_target: EnvironmentSuffixTarget = Field(
         default=EnvironmentSuffixTarget.default
     )
@@ -117,7 +122,7 @@ class Config(BaseConfig):
     disable_anonymized_analytics: bool = False
 
     _FIELD_UPDATE_STRATEGY: t.ClassVar[t.Dict[str, UpdateStrategy]] = {
-        "gateways": UpdateStrategy.KEY_UPDATE,
+        "gateways": UpdateStrategy.NESTED_UPDATE,
         "notification_targets": UpdateStrategy.EXTEND,
         "ignore_patterns": UpdateStrategy.EXTEND,
         "users": UpdateStrategy.EXTEND,
@@ -133,9 +138,10 @@ class Config(BaseConfig):
     }
 
     _connection_config_validator = connection_config_validator
+    _scheduler_config_validator = scheduler_config_validator
     _variables_validator = variables_validator
 
-    @field_validator("gateways", mode="before", always=True)
+    @field_validator("gateways", mode="before")
     @classmethod
     def _gateways_ensure_dict(cls, value: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
         try:
@@ -145,42 +151,65 @@ class Config(BaseConfig):
         except Exception:
             return value
 
-    @field_validator("environment_catalog_mapping", mode="before")
+    @field_validator("environment_catalog_mapping", "physical_schema_mapping", mode="before")
     @classmethod
     def _validate_regex_keys(
         cls, value: t.Dict[str | re.Pattern, t.Any]
     ) -> t.Dict[re.Pattern, t.Any]:
-        compiled_regexes = {}
-        for k, v in value.items():
-            try:
-                compiled_regexes[re.compile(k)] = v
-            except re.error:
-                raise ConfigError(f"`{k}` is not a valid regular expression.")
-        return compiled_regexes
+        return compile_regex_mapping(value)
 
     @model_validator(mode="before")
-    @model_validator_v1_args
-    def _normalize_and_validate_fields(cls, values: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
-        if "gateways" not in values and "gateway" in values:
-            values["gateways"] = values.pop("gateway")
+    def _normalize_and_validate_fields(cls, data: t.Any) -> t.Any:
+        if not isinstance(data, dict):
+            return data
+
+        if "gateways" not in data and "gateway" in data:
+            data["gateways"] = data.pop("gateway")
 
         for plan_deprecated in ("auto_categorize_changes", "include_unmodified"):
-            if plan_deprecated in values:
+            if plan_deprecated in data:
                 raise ConfigError(
                     f"The `{plan_deprecated}` config is deprecated. Please use the `plan.{plan_deprecated}` config instead."
                 )
 
-        return values
+        if "physical_schema_override" in data:
+            get_console().log_warning(
+                "`physical_schema_override` is deprecated. Please use `physical_schema_mapping` instead."
+            )
+
+            if "physical_schema_mapping" in data:
+                raise ConfigError(
+                    "Only one of `physical_schema_override` and `physical_schema_mapping` can be specified."
+                )
+
+            physical_schema_override: t.Dict[str, str] = data.pop("physical_schema_override")
+            # translate physical_schema_override to physical_schema_mapping
+            data["physical_schema_mapping"] = {
+                f"^{k}$": v for k, v in physical_schema_override.items()
+            }
+
+        return data
 
     @model_validator(mode="after")
-    @model_validator_v1_args
-    def _normalize_fields_after(cls, values: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
-        dialect = values["model_defaults"].dialect
-        values["environment_catalog_mapping"] = {
-            k: normalize_identifiers(v, dialect=dialect).name
-            for k, v in values.get("environment_catalog_mapping", {}).items()
-        }
-        return values
+    def _normalize_fields_after(self) -> Self:
+        dialect = self.model_defaults.dialect
+
+        def _normalize_identifiers(key: str) -> None:
+            setattr(
+                self,
+                key,
+                {
+                    k: normalize_identifiers(v, dialect=dialect).name
+                    for k, v in getattr(self, key, {}).items()
+                },
+            )
+
+        if self.environment_catalog_mapping:
+            _normalize_identifiers("environment_catalog_mapping")
+        if self.physical_schema_mapping:
+            _normalize_identifiers("physical_schema_mapping")
+
+        return self
 
     def get_default_test_connection(
         self,
