@@ -1,12 +1,14 @@
 from __future__ import annotations
 
 import functools
+import logging
 import re
 import sys
 import typing as t
 from contextlib import contextmanager
 from difflib import unified_diff
 from enum import Enum, auto
+from functools import lru_cache
 
 import pandas as pd
 from sqlglot import Dialect, Generator, ParseError, Parser, Tokenizer, TokenType, exp
@@ -21,7 +23,7 @@ from sqlglot.schema import MappingSchema
 from sqlglot.tokens import Token
 
 from sqlmesh.core.constants import MAX_MODEL_DEFINITION_SIZE
-from sqlmesh.utils.errors import SQLMeshError
+from sqlmesh.utils.errors import SQLMeshError, ConfigError
 from sqlmesh.utils.pandas import columns_to_types_from_df
 
 if t.TYPE_CHECKING:
@@ -32,6 +34,8 @@ SQLMESH_MACRO_PREFIX = "@"
 
 TABLES_META = "sqlmesh.tables"
 
+logger = logging.getLogger(__name__)
+
 
 class Model(exp.Expression):
     arg_types = {"expressions": True}
@@ -57,6 +61,10 @@ class JinjaStatement(Jinja):
     pass
 
 
+class VirtualUpdateStatement(exp.Expression):
+    arg_types = {"expressions": True}
+
+
 class ModelKind(exp.Expression):
     arg_types = {"this": True, "expressions": False}
 
@@ -101,15 +109,18 @@ class MetricAgg(exp.AggFunc):
         return self.this.name
 
 
-class StagedFilePath(exp.Table):
+class StagedFilePath(exp.Expression):
     """Represents paths to "staged files" in Snowflake."""
 
+    arg_types = exp.Table.arg_types.copy()
+
 
 def _parse_statement(self: Parser) -> t.Optional[exp.Expression]:
     if self._curr is None:
         return None
 
     parser = PARSERS.get(self._curr.text.upper())
+    error_msg = None
 
     if parser:
         # Capture any available description in the form of a comment
@@ -119,7 +130,8 @@ def _parse_statement(self: Parser) -> t.Optional[exp.Expression]:
         try:
             self._advance()
             meta = self._parse_wrapped(lambda: t.cast(t.Callable, parser)(self))
-        except ParseError:
+        except ParseError as parse_error:
+            error_msg = parse_error.args[0]
             self._retreat(index)
 
         # Only return the DDL expression if we actually managed to parse one. This is
@@ -129,7 +141,12 @@ def _parse_statement(self: Parser) -> t.Optional[exp.Expression]:
             meta.comments = comments
             return meta
 
-    return self.__parse_statement()  # type: ignore
+    try:
+        return self.__parse_statement()  # type: ignore
+    except ParseError:
+        if error_msg:
+            raise ParseError(error_msg)
+        raise
 
 
 def _parse_lambda(self: Parser, alias: bool = False) -> t.Optional[exp.Expression]:
@@ -241,7 +258,7 @@ def _parse_macro(self: Parser, keyword_macro: str = "") -> t.Optional[exp.Expres
             return field
         return self.expression(MacroVar, this=field.this, comments=comments)
 
-    if isinstance(field, exp.Window):
+    if isinstance(field, (exp.Window, exp.IgnoreNulls, exp.RespectNulls)):
         field.set("this", _build_macro(field.this))
     else:
         field = _build_macro(field)
@@ -311,6 +328,42 @@ def _parse_join(
     return macro
 
 
+def _warn_unsupported(self: Parser) -> None:
+    from sqlmesh.core.console import get_console
+
+    sql = self._find_sql(self._tokens[0], self._tokens[-1])[: self.error_message_context]
+
+    get_console().log_warning(
+        f"'{sql}' could not be semantically understood as it contains unsupported syntax, SQLMesh will treat the command as is. Note that any references to the model's "
+        "underlying physical table can't be resolved in this case, consider using Jinja as explained here https://sqlmesh.readthedocs.io/en/stable/concepts/macros/macro_variables/#audit-only-variables"
+    )
+
+
+def _parse_select(
+    self: Parser,
+    nested: bool = False,
+    table: bool = False,
+    parse_subquery_alias: bool = True,
+    parse_set_operation: bool = True,
+) -> t.Optional[exp.Expression]:
+    select = self.__parse_select(  # type: ignore
+        nested=nested,
+        table=table,
+        parse_subquery_alias=parse_subquery_alias,
+        parse_set_operation=parse_set_operation,
+    )
+
+    if (
+        not select
+        and not parse_set_operation
+        and self._match_pair(TokenType.PARAMETER, TokenType.VAR, advance=False)
+    ):
+        self._advance()
+        return _parse_macro(self)
+
+    return select
+
+
 def _parse_where(self: Parser, skip_where_token: bool = False) -> t.Optional[exp.Expression]:
     macro = _parse_matching_macro(self, "WHERE")
     if not macro:
@@ -363,14 +416,27 @@ def _parse_limit(
     return macro
 
 
+def _parse_macro_or_clause(self: Parser, parser: t.Callable) -> t.Optional[exp.Expression]:
+    return _parse_macro(self) if self._match(TokenType.PARAMETER) else parser()
+
+
 def _parse_props(self: Parser) -> t.Optional[exp.Expression]:
     key = self._parse_id_var(any_token=True)
     if not key:
         return None
 
     name = key.name.lower()
-    if name == "when_matched":
-        value: t.Optional[exp.Expression] = self._parse_when_matched()[0]
+    if name == "time_data_type":
+        # TODO: if we make *_data_type a convention to parse things into exp.DataType, we could make this more generic
+        value = self._parse_types(schema=True)
+    elif name == "when_matched":
+        # Parentheses around the WHEN clauses can be used to disambiguate them from other properties
+        value = self._parse_wrapped(
+            lambda: _parse_macro_or_clause(self, self._parse_when_matched),
+            optional=True,
+        )
+    elif name == "merge_filter":
+        value = self._parse_conjunction()
     elif self._match(TokenType.L_PAREN):
         value = self.expression(exp.Tuple, expressions=self._parse_csv(self._parse_equality))
         self._match_r_paren()
@@ -379,7 +445,7 @@ def _parse_props(self: Parser) -> t.Optional[exp.Expression]:
 
     if name == "path" and value:
         # Make sure if we get a windows path that it is converted to posix
-        value = exp.Literal.string(value.this.replace("\\", "/"))
+        value = exp.Literal.string(value.this.replace("\\", "/"))  # type: ignore
 
     return self.expression(exp.Property, this=name, value=value)
 
@@ -408,19 +474,38 @@ def _parse_types(
 #
 # See: https://docs.snowflake.com/en/user-guide/querying-stage
 def _parse_table_parts(
-    self: Parser, schema: bool = False, is_db_reference: bool = False
-) -> exp.Table:
+    self: Parser, schema: bool = False, is_db_reference: bool = False, wildcard: bool = False
+) -> exp.Table | StagedFilePath:
     index = self._index
-    table = self.__parse_table_parts(schema=schema, is_db_reference=is_db_reference)  # type: ignore
-    table_arg = table.this
+    table = self.__parse_table_parts(  # type: ignore
+        schema=schema, is_db_reference=is_db_reference, wildcard=wildcard
+    )
 
-    if isinstance(table_arg, exp.Var) and table_arg.name.startswith(SQLMESH_MACRO_PREFIX):
-        # Macro functions do not clash with the staged file syntax, so we can safely parse them
-        if self._prev.token_type == TokenType.STRING or "(" in table_arg.name:
+    table_arg = table.this
+    name = table_arg.name
+
+    if isinstance(table_arg, exp.Var) and name.startswith(SQLMESH_MACRO_PREFIX):
+        # In these cases, we don't want to produce a `StagedFilePath` node:
+        #
+        # - @'...' needs to parsed as a string template
+        # - @{foo}.bar needs to be parsed as a table with a macro var part
+        # - @name(arg1 [, arg2 ...]) needs to be parsed as a macro function call
+        #
+        # These cases can unambiguously be parsed using the base `_parse_table_parts`, as there
+        # is no overlap with staged files https://docs.snowflake.com/en/user-guide/querying-stage
+        if (
+            self._prev.token_type == TokenType.STRING
+            or "{" in name
+            or (
+                self._curr
+                and self._prev.token_type in (TokenType.L_PAREN, TokenType.R_PAREN)
+                and self._curr.text.upper() not in ("FILE_FORMAT", "PATTERN")
+            )
+        ):
             self._retreat(index)
             return Parser._parse_table_parts(self, schema=schema, is_db_reference=is_db_reference)
 
-        table_arg.replace(MacroVar(this=table_arg.name[1:]))
+        table_arg.replace(MacroVar(this=name[1:]))
         return StagedFilePath(**table.args)
 
     return table
@@ -449,10 +534,16 @@ def _parse_if(self: Parser) -> t.Optional[exp.Expression]:
         else:
             self.raise_error("Expecting )")
 
-        return exp.Anonymous(this="IF", expressions=[cond, self._parse_statement()])
+        index = self._index
+        stmt = self._parse_statement()
+        if self._curr:
+            self._retreat(index)
+            stmt = self._parse_as_command(self._tokens[index])
+
+        return exp.Anonymous(this="IF", expressions=[cond, stmt])
 
 
-def _create_parser(parser_type: t.Type[exp.Expression], table_keys: t.List[str]) -> t.Callable:
+def _create_parser(expression_type: t.Type[exp.Expression], table_keys: t.List[str]) -> t.Callable:
     def parse(self: Parser) -> t.Optional[exp.Expression]:
         from sqlmesh.core.model.kind import ModelKindName
 
@@ -479,14 +570,25 @@ def _create_parser(parser_type: t.Type[exp.Expression], table_keys: t.List[str])
 
             if key in table_keys:
                 value = self._parse_table_parts()
+                if value and self._prev.token_type == TokenType.STRING:
+                    self.raise_error(
+                        f"'{key}' property cannot be a string value: {value}. "
+                        "Please use the identifier syntax instead, e.g. foo.bar instead of 'foo.bar'"
+                    )
             elif key == "columns":
                 value = self._parse_schema()
             elif key == "kind":
-                id_var = self._parse_id_var(any_token=True)
-                if not id_var:
-                    value = None
+                field = _parse_macro_or_clause(self, lambda: self._parse_id_var(any_token=True))
+
+                if not field or isinstance(field, (MacroVar, MacroFunc)):
+                    value = field
                 else:
-                    kind = ModelKindName[id_var.name.upper()]
+                    try:
+                        kind = ModelKindName[field.name.upper()]
+                    except KeyError:
+                        raise SQLMeshError(
+                            f"Model kind specified as '{field.name}', but that is not a valid model kind.\n\nPlease specify one of {', '.join(ModelKindName)}."
+                        )
 
                     if kind in (
                         ModelKindName.INCREMENTAL_BY_TIME_RANGE,
@@ -497,16 +599,13 @@ def _create_parser(parser_type: t.Type[exp.Expression], table_keys: t.List[str])
                         ModelKindName.SCD_TYPE_2,
                         ModelKindName.SCD_TYPE_2_BY_TIME,
                         ModelKindName.SCD_TYPE_2_BY_COLUMN,
+                        ModelKindName.CUSTOM,
                     ) and self._match(TokenType.L_PAREN, advance=False):
                         props = self._parse_wrapped_csv(functools.partial(_parse_props, self))
                     else:
                         props = None
 
-                    value = self.expression(
-                        ModelKind,
-                        this=kind.value,
-                        expressions=props,
-                    )
+                    value = self.expression(ModelKind, this=kind.value, expressions=props)
             elif key == "expression":
                 value = self._parse_conjunction()
             else:
@@ -517,7 +616,7 @@ def _create_parser(parser_type: t.Type[exp.Expression], table_keys: t.List[str])
 
             expressions.append(self.expression(exp.Property, this=key, value=value))
 
-        return self.expression(parser_type, expressions=expressions)
+        return self.expression(expression_type, expressions=expressions)
 
     return parse
 
@@ -538,6 +637,7 @@ def _props_sql(self: Generator, expressions: t.List[exp.Expression]) -> str:
 
         if i < size - 1:
             sql += ","
+
         props.append(self.maybe_comment(sql, expression=prop))
 
     return "\n".join(props)
@@ -566,8 +666,19 @@ def _macro_func_sql(self: Generator, expression: MacroFunc) -> str:
     expression = expression.this
     name = expression.name
     if name in KEYWORD_MACROS:
-        return _macro_keyword_func_sql(self, expression)
-    return f"@{name}({self.format_args(*expression.expressions)})"
+        sql = _macro_keyword_func_sql(self, expression)
+    else:
+        sql = f"@{name}({self.format_args(*expression.expressions)})"
+    return self.maybe_comment(sql, expression)
+
+
+def _whens_sql(self: Generator, expression: exp.Whens) -> str:
+    if isinstance(expression.parent, exp.Merge):
+        return self.whens_sql(expression)
+
+    # If the `WHEN` clauses aren't part of a MERGE statement (e.g. they
+    # appear in the `MODEL` DDL), then we will wrap them with parentheses.
+    return self.wrap(self.expressions(expression, sep=" ", indent=False))
 
 
 def _override(klass: t.Type[Tokenizer | Parser], func: t.Callable) -> None:
@@ -577,49 +688,55 @@ def _override(klass: t.Type[Tokenizer | Parser], func: t.Callable) -> None:
 
 
 def format_model_expressions(
-    expressions: t.List[exp.Expression], dialect: t.Optional[str] = None, **kwargs: t.Any
+    expressions: t.List[exp.Expression],
+    dialect: t.Optional[str] = None,
+    rewrite_casts: bool = True,
+    **kwargs: t.Any,
 ) -> str:
     """Format a model's expressions into a standardized format.
 
     Args:
         expressions: The model's expressions, must be at least model def + query.
         dialect: The dialect to render the expressions as.
+        rewrite_casts: Whether to rewrite all casts to use the :: syntax.
         **kwargs: Additional keyword arguments to pass to the sql generator.
 
     Returns:
         A string representing the formatted model.
     """
-    if len(expressions) == 1:
+    if len(expressions) == 1 and is_meta_expression(expressions[0]):
         return expressions[0].sql(pretty=True, dialect=dialect)
 
-    *statements, query = expressions
+    if rewrite_casts:
 
-    def cast_to_colon(node: exp.Expression) -> exp.Expression:
-        if isinstance(node, exp.Cast) and not any(
-            # Only convert CAST into :: if it doesn't have additional args set, otherwise this
-            # conversion could alter the semantics (eg. changing SAFE_CAST in BigQuery to CAST)
-            arg
-            for name, arg in node.args.items()
-            if name not in ("this", "to")
-        ):
-            this = node.this
+        def cast_to_colon(node: exp.Expression) -> exp.Expression:
+            if isinstance(node, exp.Cast) and not any(
+                # Only convert CAST into :: if it doesn't have additional args set, otherwise this
+                # conversion could alter the semantics (eg. changing SAFE_CAST in BigQuery to CAST)
+                arg
+                for name, arg in node.args.items()
+                if name not in ("this", "to")
+            ):
+                this = node.this
 
-            if not isinstance(this, (exp.Binary, exp.Unary)) or isinstance(this, exp.Paren):
-                cast = DColonCast(this=this, to=node.to)
-                cast.comments = node.comments
-                node = cast
+                if not isinstance(this, (exp.Binary, exp.Unary)) or isinstance(this, exp.Paren):
+                    cast = DColonCast(this=this, to=node.to)
+                    cast.comments = node.comments
+                    node = cast
 
-        exp.replace_children(node, cast_to_colon)
-        return node
+            exp.replace_children(node, cast_to_colon)
+            return node
 
-    query = query.copy()
-    exp.replace_children(query, cast_to_colon)
+        new_expressions = []
+        for expression in expressions:
+            expression = expression.copy()
+            exp.replace_children(expression, cast_to_colon)
+            new_expressions.append(expression)
+
+        expressions = new_expressions
 
     return ";\n\n".join(
-        [
-            *(statement.sql(pretty=True, dialect=dialect, **kwargs) for statement in statements),
-            query.sql(pretty=True, dialect=dialect, **kwargs),
-        ]
+        expression.sql(pretty=True, dialect=dialect, **kwargs) for expression in expressions
     ).strip()
 
 
@@ -661,6 +778,8 @@ def _is_command_statement(command: str, tokens: t.List[Token], pos: int) -> bool
 JINJA_QUERY_BEGIN = "JINJA_QUERY_BEGIN"
 JINJA_STATEMENT_BEGIN = "JINJA_STATEMENT_BEGIN"
 JINJA_END = "JINJA_END"
+ON_VIRTUAL_UPDATE_BEGIN = "ON_VIRTUAL_UPDATE_BEGIN"
+ON_VIRTUAL_UPDATE_END = "ON_VIRTUAL_UPDATE_END"
 
 
 def _is_jinja_statement_begin(tokens: t.List[Token], pos: int) -> bool:
@@ -683,10 +802,24 @@ def jinja_statement(statement: str) -> JinjaStatement:
     return JinjaStatement(this=exp.Literal.string(statement.strip()))
 
 
+def _is_virtual_statement_begin(tokens: t.List[Token], pos: int) -> bool:
+    return _is_command_statement(ON_VIRTUAL_UPDATE_BEGIN, tokens, pos)
+
+
+def _is_virtual_statement_end(tokens: t.List[Token], pos: int) -> bool:
+    return _is_command_statement(ON_VIRTUAL_UPDATE_END, tokens, pos)
+
+
+def virtual_statement(statements: t.List[exp.Expression]) -> VirtualUpdateStatement:
+    return VirtualUpdateStatement(expressions=statements)
+
+
 class ChunkType(Enum):
     JINJA_QUERY = auto()
     JINJA_STATEMENT = auto()
     SQL = auto()
+    VIRTUAL_STATEMENT = auto()
+    VIRTUAL_JINJA_STATEMENT = auto()
 
 
 def parse_one(
@@ -726,9 +859,15 @@ def parse(
     total = len(tokens)
 
     pos = 0
+    virtual = False
     while pos < total:
         token = tokens[pos]
-        if _is_jinja_end(tokens, pos) or (
+        if _is_virtual_statement_end(tokens, pos):
+            chunks[-1][0].append(token)
+            virtual = False
+            chunks.append(([], ChunkType.SQL))
+            pos += 2
+        elif _is_jinja_end(tokens, pos) or (
             chunks[-1][1] == ChunkType.SQL
             and token.token_type == TokenType.SEMICOLON
             and pos < total - 1
@@ -739,13 +878,24 @@ def parse(
                 # Jinja end statement
                 chunks[-1][0].append(token)
                 pos += 2
-            chunks.append(([], ChunkType.SQL))
+            chunks.append(
+                (
+                    [],
+                    ChunkType.VIRTUAL_STATEMENT
+                    if virtual and tokens[pos] != ON_VIRTUAL_UPDATE_END
+                    else ChunkType.SQL,
+                )
+            )
         elif _is_jinja_query_begin(tokens, pos):
             chunks.append(([token], ChunkType.JINJA_QUERY))
             pos += 2
         elif _is_jinja_statement_begin(tokens, pos):
             chunks.append(([token], ChunkType.JINJA_STATEMENT))
             pos += 2
+        elif _is_virtual_statement_begin(tokens, pos):
+            chunks.append(([token], ChunkType.VIRTUAL_STATEMENT))
+            pos += 2
+            virtual = True
         else:
             chunks[-1][0].append(token)
             pos += 1
@@ -753,22 +903,68 @@ def parse(
     parser = dialect.parser()
     expressions: t.List[exp.Expression] = []
 
-    for chunk, chunk_type in chunks:
-        if chunk_type == ChunkType.SQL:
-            parsed_expressions: t.List[t.Optional[exp.Expression]] = (
-                parser.parse(chunk, sql) if into is None else parser.parse_into(into, chunk, sql)
-            )
-            for expression in parsed_expressions:
-                if expression:
+    def parse_sql_chunk(chunk: t.List[Token], meta_sql: bool = True) -> t.List[exp.Expression]:
+        parsed_expressions: t.List[t.Optional[exp.Expression]] = (
+            parser.parse(chunk, sql) if into is None else parser.parse_into(into, chunk, sql)
+        )
+        expressions = []
+        for expression in parsed_expressions:
+            if expression:
+                if meta_sql:
                     expression.meta["sql"] = parser._find_sql(chunk[0], chunk[-1])
-                    expressions.append(expression)
-        else:
-            start, *_, end = chunk
-            segment = sql[start.end + 2 : end.start - 1]
-            factory = jinja_query if chunk_type == ChunkType.JINJA_QUERY else jinja_statement
-            expression = factory(segment.strip())
+                expressions.append(expression)
+        return expressions
+
+    def parse_jinja_chunk(chunk: t.List[Token], meta_sql: bool = True) -> exp.Expression:
+        start, *_, end = chunk
+        segment = sql[start.end + 2 : end.start - 1]
+        factory = jinja_query if chunk_type == ChunkType.JINJA_QUERY else jinja_statement
+        expression = factory(segment.strip())
+        if meta_sql:
             expression.meta["sql"] = sql[start.start : end.end + 1]
-            expressions.append(expression)
+        return expression
+
+    def parse_virtual_statement(
+        chunks: t.List[t.Tuple[t.List[Token], ChunkType]], pos: int
+    ) -> t.Tuple[t.List[exp.Expression], int]:
+        # For virtual statements we need to handle both SQL and Jinja nested blocks within the chunk
+        virtual_update_statements = []
+        start = chunks[pos][0][0].start
+
+        while (
+            chunks[pos - 1][0] == [] or chunks[pos - 1][0][-1].text.upper() != ON_VIRTUAL_UPDATE_END
+        ):
+            chunk, chunk_type = chunks[pos]
+            if chunk_type == ChunkType.JINJA_STATEMENT:
+                virtual_update_statements.append(parse_jinja_chunk(chunk, False))
+            else:
+                virtual_update_statements.extend(
+                    parse_sql_chunk(
+                        chunk[int(chunk[0].text.upper() == ON_VIRTUAL_UPDATE_BEGIN) : -1], False
+                    ),
+                )
+            pos += 1
+
+        if virtual_update_statements:
+            statements = virtual_statement(virtual_update_statements)
+            end = chunk[-1].end + 1
+            statements.meta["sql"] = sql[start:end]
+            return [statements], pos
+
+        return [], pos
+
+    pos = 0
+    total_chunks = len(chunks)
+    while pos < total_chunks:
+        chunk, chunk_type = chunks[pos]
+        if chunk_type == ChunkType.VIRTUAL_STATEMENT:
+            virtual_expression, pos = parse_virtual_statement(chunks, pos)
+            expressions.extend(virtual_expression)
+        elif chunk_type == ChunkType.SQL:
+            expressions.extend(parse_sql_chunk(chunk))
+        else:
+            expressions.append(parse_jinja_chunk(chunk))
+        pos += 1
 
     return expressions
 
@@ -801,7 +997,7 @@ def extend_sqlglot() -> None:
         if MacroFunc not in generator.TRANSFORMS:
             generator.TRANSFORMS.update(
                 {
-                    Audit: lambda self, e: _sqlmesh_ddl_sql(self, e, "Audit"),
+                    Audit: lambda self, e: _sqlmesh_ddl_sql(self, e, "AUDIT"),
                     DColonCast: lambda self, e: f"{self.sql(e, 'this')}::{self.sql(e, 'to')}",
                     Jinja: lambda self, e: e.name,
                     JinjaQuery: lambda self, e: f"{JINJA_QUERY_BEGIN};\n{e.name}\n{JINJA_END};",
@@ -817,15 +1013,17 @@ def extend_sqlglot() -> None:
                     ModelKind: _model_kind_sql,
                     PythonCode: lambda self, e: self.expressions(e, sep="\n", indent=False),
                     StagedFilePath: lambda self, e: self.table_sql(e),
+                    exp.Whens: _whens_sql,
                 }
             )
-
+        if MacroDef not in generator.WITH_SEPARATED_COMMENTS:
             generator.WITH_SEPARATED_COMMENTS = (
                 *generator.WITH_SEPARATED_COMMENTS,
                 Model,
                 MacroDef,
             )
 
+    _override(Parser, _parse_select)
     _override(Parser, _parse_statement)
     _override(Parser, _parse_join)
     _override(Parser, _parse_order)
@@ -838,6 +1036,7 @@ def extend_sqlglot() -> None:
     _override(Parser, _parse_types)
     _override(Parser, _parse_if)
     _override(Parser, _parse_id_var)
+    _override(Parser, _warn_unsupported)
     _override(Snowflake.Parser, _parse_table_parts)
 
 
@@ -894,6 +1093,14 @@ def select_from_values_for_batch_range(
         ]
 
     values_exp = exp.values(expressions, alias=alias, columns=columns_to_types)
+    if values:
+        # BigQuery crashes on `SELECT CAST(x AS TIMESTAMP) FROM UNNEST([NULL]) AS x`, but not
+        # on `SELECT CAST(x AS TIMESTAMP) FROM UNNEST([CAST(NULL AS TIMESTAMP)]) AS x`. This
+        # ensures nulls under the `Values` expression are cast to avoid similar issues.
+        for value, kind in zip(values_exp.expressions[0].expressions, columns_to_types.values()):
+            if isinstance(value, exp.Null):
+                value.replace(exp.cast(value, to=kind))
+
     return exp.select(*casted_columns).from_(values_exp, copy=False).where(where, copy=False)
 
 
@@ -934,6 +1141,7 @@ def set_default_catalog(
     return table
 
 
+@lru_cache(maxsize=None)
 def normalize_model_name(
     table: str | exp.Table | exp.Column,
     default_catalog: t.Optional[str],
@@ -998,19 +1206,52 @@ def transform_values(
     values: t.Tuple[t.Any, ...], columns_to_types: t.Dict[str, exp.DataType]
 ) -> t.Iterator[t.Any]:
     """Perform transformations on values given columns_to_types."""
-    for value, col_type in zip(values, columns_to_types.values()):
-        if col_type.is_type(exp.DataType.Type.JSON):
-            yield exp.func("PARSE_JSON", f"'{value}'")
-        elif isinstance(value, dict) and col_type.is_type(*exp.DataType.STRUCT_TYPES):
-            yield _dict_to_struct(value)
-        else:
-            yield value
+
+    def _transform_value(value: t.Any, dtype: exp.DataType) -> t.Any:
+        if (
+            isinstance(value, list)
+            and dtype.is_type(*exp.DataType.ARRAY_TYPES)
+            and len(dtype.expressions) == 1
+        ):
+            element_type = dtype.expressions[0]
+            return exp.convert([_transform_value(v, element_type) for v in value])
+
+        if (
+            isinstance(value, dict)
+            and dtype.is_type(*exp.DataType.STRUCT_TYPES)
+            and len(value) == len(dtype.expressions)
+        ):
+            expressions = []
+            for (field_name, field_value), field_type in zip(value.items(), dtype.expressions):
+                if isinstance(field_type, exp.ColumnDef):
+                    field_type = field_type.kind
+                else:
+                    field_type = exp.DataType.build(exp.DataType.Type.UNKNOWN)
+
+                expressions.append(
+                    exp.PropertyEQ(
+                        this=exp.to_identifier(field_name),
+                        expression=_transform_value(field_value, field_type),
+                    )
+                )
+
+            return exp.Struct(expressions=expressions)
+
+        if dtype.is_type(exp.DataType.Type.JSON):
+            return exp.func("PARSE_JSON", f"'{value}'")
+
+        return exp.convert(value)
+
+    for col_value, col_type in zip(values, columns_to_types.values()):
+        yield _transform_value(col_value, col_type)
 
 
-def to_schema(sql_path: str | exp.Table) -> exp.Table:
+def to_schema(sql_path: str | exp.Table, dialect: DialectType = None) -> exp.Table:
     if isinstance(sql_path, exp.Table) and sql_path.this is None:
         return sql_path
-    table = exp.to_table(sql_path.copy() if isinstance(sql_path, exp.Table) else sql_path)
+    table = exp.to_table(
+        sql_path.copy() if isinstance(sql_path, exp.Table) else sql_path, dialect=dialect
+    )
     table.set("catalog", table.args.get("db"))
     table.set("db", table.args.get("this"))
     table.set("this", None)
@@ -1050,22 +1291,80 @@ def _unquote_schema(schema: t.Dict) -> t.Dict:
     }
 
 
-def _dict_to_struct(values: t.Dict) -> exp.Struct:
-    expressions = []
-    for key, value in values.items():
-        key = exp.to_identifier(key)
-        value = _dict_to_struct(value) if isinstance(value, dict) else exp.convert(value)
-        expressions.append(exp.PropertyEQ(this=key, expression=value))
-
-    return exp.Struct(expressions=expressions)
-
-
 @contextmanager
 def normalize_and_quote(
-    query: E, dialect: str, default_catalog: t.Optional[str], quote: bool = True
+    query: E, dialect: DialectType, default_catalog: t.Optional[str], quote: bool = True
 ) -> t.Iterator[E]:
     qualify_tables(query, catalog=default_catalog, dialect=dialect)
     normalize_identifiers(query, dialect=dialect)
     yield query
     if quote:
         quote_identifiers(query, dialect=dialect)
+
+
+def interpret_expression(e: exp.Expression) -> exp.Expression | str | int | float | bool:
+    if e.is_int:
+        return int(e.this)
+    if e.is_number:
+        return float(e.this)
+    if isinstance(e, (exp.Literal, exp.Boolean)):
+        return e.this
+    return e
+
+
+def interpret_key_value_pairs(
+    e: exp.Tuple,
+) -> t.Dict[str, exp.Expression | str | int | float | bool]:
+    return {i.this.name: interpret_expression(i.expression) for i in e.expressions}
+
+
+def extract_func_call(
+    v: exp.Expression, allow_tuples: bool = False
+) -> t.Tuple[str, t.Dict[str, exp.Expression]]:
+    kwargs = {}
+
+    if isinstance(v, exp.Anonymous):
+        func = v.name
+        args = v.expressions
+    elif isinstance(v, exp.Func):
+        func = v.sql_name()
+        args = list(v.args.values())
+    elif isinstance(v, exp.Paren):
+        func = ""
+        args = [v.this]
+    elif isinstance(v, exp.Tuple):  # airflow only
+        if not allow_tuples:
+            raise ConfigError("Audit name is missing (eg. MY_AUDIT())")
+
+        func = ""
+        args = v.expressions
+    else:
+        return v.name.lower(), {}
+
+    for arg in args:
+        if not isinstance(arg, (exp.PropertyEQ, exp.EQ)):
+            raise ConfigError(
+                f"Function '{func}' must be called with key-value arguments like {func}(arg := value)."
+            )
+        kwargs[arg.left.name.lower()] = arg.right
+    return func.lower(), kwargs
+
+
+def is_meta_expression(v: t.Any) -> bool:
+    return isinstance(v, (Audit, Metric, Model))
+
+
+def replace_merge_table_aliases(expression: exp.Expression) -> exp.Expression:
+    """
+    Resolves references from the "source" and "target" tables (or their DBT equivalents)
+    with the corresponding SQLMesh merge aliases (MERGE_SOURCE_ALIAS and MERGE_TARGET_ALIAS)
+    """
+    from sqlmesh.core.engine_adapter.base import MERGE_SOURCE_ALIAS, MERGE_TARGET_ALIAS
+
+    if isinstance(expression, exp.Column) and (first_part := expression.parts[0]):
+        if first_part.this.lower() in ("target", "dbt_internal_dest", "__merge_target__"):
+            first_part.replace(exp.to_identifier(MERGE_TARGET_ALIAS))
+        elif first_part.this.lower() in ("source", "dbt_internal_source", "__merge_source__"):
+            first_part.replace(exp.to_identifier(MERGE_SOURCE_ALIAS))
+
+    return expression
