@@ -2,13 +2,18 @@ from __future__ import annotations
 
 import logging
 import typing as t
+from collections import defaultdict
 
 import pandas as pd
-from sqlglot import exp
+from sqlglot import exp, parse_one
 from sqlglot.transforms import remove_precision_parameterized_types
 
 from sqlmesh.core.dialect import to_schema
-from sqlmesh.core.engine_adapter.mixins import InsertOverwriteWithMergeMixin
+from sqlmesh.core.engine_adapter.mixins import (
+    InsertOverwriteWithMergeMixin,
+    ClusteredByMixin,
+    RowDiffMixin,
+)
 from sqlmesh.core.engine_adapter.shared import (
     CatalogSupport,
     DataObject,
@@ -18,6 +23,7 @@ from sqlmesh.core.engine_adapter.shared import (
 )
 from sqlmesh.core.node import IntervalUnit
 from sqlmesh.core.schema_diff import SchemaDiffer
+from sqlmesh.utils import optional_import
 from sqlmesh.utils.date import to_datetime
 from sqlmesh.utils.errors import SQLMeshError
 
@@ -26,19 +32,29 @@ if t.TYPE_CHECKING:
     from google.cloud import bigquery
     from google.cloud.bigquery import StandardSqlDataType
     from google.cloud.bigquery.client import Client as BigQueryClient
-    from google.cloud.bigquery.client import Connection as BigQueryConnection
     from google.cloud.bigquery.job.base import _AsyncJob as BigQueryQueryResult
     from google.cloud.bigquery.table import Table as BigQueryTable
 
     from sqlmesh.core._typing import SchemaName, SessionProperties, TableName
-    from sqlmesh.core.engine_adapter._typing import DF, Query
+    from sqlmesh.core.engine_adapter._typing import BigframeSession, DF, Query
     from sqlmesh.core.engine_adapter.base import QueryOrDF
 
+
 logger = logging.getLogger(__name__)
 
+bigframes = optional_import("bigframes")
+bigframes_pd = optional_import("bigframes.pandas")
+
+
+NestedField = t.Tuple[str, str, t.List[str]]
+NestedFieldsDict = t.Dict[str, t.List[NestedField]]
+
+# used to tag AST nodes to be specially handled in alter_table()
+_CLUSTERING_META_KEY = "__sqlmesh_update_table_clustering"
+
 
 @set_catalog()
-class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
+class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin, ClusteredByMixin, RowDiffMixin):
     """
     BigQuery Engine Adapter using the `google-cloud-bigquery` library's DB API.
     """
@@ -48,12 +64,9 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
     SUPPORTS_TRANSACTIONS = False
     SUPPORTS_MATERIALIZED_VIEWS = True
     SUPPORTS_CLONING = True
-    CATALOG_SUPPORT = CatalogSupport.FULL_SUPPORT
     MAX_TABLE_COMMENT_LENGTH = 1024
     MAX_COLUMN_COMMENT_LENGTH = 1024
 
-    # SQL is not supported for adding columns to structs: https://cloud.google.com/bigquery/docs/managing-table-schemas#api_1
-    # Can explore doing this with the API in the future
     SCHEMA_DIFFER = SchemaDiffer(
         compatible_types={
             exp.DataType.build("INT64", dialect=DIALECT): {
@@ -65,14 +78,32 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
                 exp.DataType.build("FLOAT64", dialect=DIALECT),
                 exp.DataType.build("BIGNUMERIC", dialect=DIALECT),
             },
-            exp.DataType.build("BIGNUMERIC", dialect=DIALECT): {
-                exp.DataType.build("FLOAT64", dialect=DIALECT),
-            },
             exp.DataType.build("DATE", dialect=DIALECT): {
                 exp.DataType.build("DATETIME", dialect=DIALECT),
             },
         },
+        coerceable_types={
+            exp.DataType.build("FLOAT64", dialect=DIALECT): {
+                exp.DataType.build("BIGNUMERIC", dialect=DIALECT),
+            },
+        },
         support_coercing_compatible_types=True,
+        parameterized_type_defaults={
+            exp.DataType.build("DECIMAL", dialect=DIALECT).this: [(38, 9), (0,)],
+            exp.DataType.build("BIGDECIMAL", dialect=DIALECT).this: [(76.76, 38), (0,)],
+        },
+        types_with_unlimited_length={
+            # parameterized `STRING(n)` can ALTER to unparameterized `STRING`
+            exp.DataType.build("STRING", dialect=DIALECT).this: {
+                exp.DataType.build("STRING", dialect=DIALECT).this,
+            },
+            # parameterized `BYTES(n)` can ALTER to unparameterized `BYTES`
+            exp.DataType.build("BYTES", dialect=DIALECT).this: {
+                exp.DataType.build("BYTES", dialect=DIALECT).this,
+            },
+        },
+        support_nested_operations=True,
+        support_nested_drop=False,
     )
 
     @property
@@ -80,8 +111,15 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         return self.connection._client
 
     @property
-    def connection(self) -> BigQueryConnection:
-        return self.cursor.connection
+    def bigframe(self) -> t.Optional[BigframeSession]:
+        if bigframes:
+            options = bigframes.BigQueryOptions(
+                credentials=self.client._credentials,
+                project=self.client.project,
+                location=self.client.location,
+            )
+            return bigframes.connect(context=options)
+        return None
 
     @property
     def _job_params(self) -> t.Dict[str, t.Any]:
@@ -97,6 +135,10 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
             params["maximum_bytes_billed"] = self._extra_config.get("maximum_bytes_billed")
         return params
 
+    @property
+    def catalog_support(self) -> CatalogSupport:
+        return CatalogSupport.FULL_SUPPORT
+
     def _df_to_source_queries(
         self,
         df: DF,
@@ -114,7 +156,12 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         )
 
         def query_factory() -> Query:
-            if not self.table_exists(temp_table):
+            if bigframes_pd and isinstance(df, bigframes_pd.DataFrame):
+                df.to_gbq(
+                    f"{temp_bq_table.project}.{temp_bq_table.dataset_id}.{temp_bq_table.table_id}",
+                    if_exists="replace",
+                )
+            elif not self.table_exists(temp_table):
                 # Make mypy happy
                 assert isinstance(df, pd.DataFrame)
                 self._db_call(self.client.create_table, table=temp_bq_table, exists_ok=False)
@@ -160,6 +207,7 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         schema_name: SchemaName,
         ignore_if_exists: bool = True,
         warn_on_error: bool = True,
+        properties: t.List[exp.Expression] = [],
     ) -> None:
         """Create a schema from a name or qualified table name."""
         from google.api_core.exceptions import Conflict
@@ -204,25 +252,72 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
                 return "JSON"
             return kind.name
 
-        table = self._get_table(table_name)
-        columns = {
-            field.name: exp.DataType.build(
-                dtype_to_sql(field.to_standard_sql().type), dialect=self.dialect
-            )
-            for field in table.schema
-        }
-        if include_pseudo_columns and table.time_partitioning and not table.time_partitioning.field:
-            columns["_PARTITIONTIME"] = exp.DataType.build("TIMESTAMP")
-            if table.time_partitioning.type_ == "DAY":
-                columns["_PARTITIONDATE"] = exp.DataType.build("DATE")
+        def create_mapping_schema(
+            schema: t.Sequence[bigquery.SchemaField],
+        ) -> t.Dict[str, exp.DataType]:
+            return {
+                field.name: exp.DataType.build(
+                    dtype_to_sql(field.to_standard_sql().type), dialect=self.dialect
+                )
+                for field in schema
+            }
+
+        table = exp.to_table(table_name)
+        if len(table.parts) == 3 and "." in table.name:
+            # The client's `get_table` method can't handle paths with >3 identifiers
+            self.execute(exp.select("*").from_(table).limit(1))
+            query_results = self._query_job._query_results
+            columns = create_mapping_schema(query_results.schema)
+        else:
+            bq_table = self._get_table(table)
+            columns = create_mapping_schema(bq_table.schema)
+
+            if (
+                include_pseudo_columns
+                and bq_table.time_partitioning
+                and not bq_table.time_partitioning.field
+            ):
+                columns["_PARTITIONTIME"] = exp.DataType.build("TIMESTAMP")
+                if bq_table.time_partitioning.type_ == "DAY":
+                    columns["_PARTITIONDATE"] = exp.DataType.build("DATE")
+
         return columns
 
+    def alter_table(
+        self,
+        alter_expressions: t.List[exp.Alter],
+    ) -> None:
+        """
+        Performs the alter statements to change the current table into the structure of the target table,
+        and uses the API to add columns to structs, where SQL is not supported.
+        """
+
+        nested_fields, non_nested_expressions = self._split_alter_expressions(alter_expressions)
+
+        if nested_fields:
+            self._update_table_schema_nested_fields(nested_fields, alter_expressions[0].this)
+
+        # this is easier than trying to detect exp.Cluster nodes
+        # or exp.Command nodes that contain the string "DROP CLUSTERING KEY"
+        clustering_change_operations = [
+            e for e in non_nested_expressions if _CLUSTERING_META_KEY in e.meta
+        ]
+        for op in clustering_change_operations:
+            non_nested_expressions.remove(op)
+            table, cluster_by = op.meta[_CLUSTERING_META_KEY]
+            assert isinstance(table, str) or isinstance(table, exp.Table)
+
+            self._update_clustering_key(table, cluster_by)
+
+        if non_nested_expressions:
+            super().alter_table(non_nested_expressions)
+
     def fetchone(
         self,
         query: t.Union[exp.Expression, str],
         ignore_unsupported_errors: bool = False,
         quote_identifiers: bool = False,
-    ) -> t.Tuple:
+    ) -> t.Optional[t.Tuple]:
         """
         BigQuery's `fetchone` method doesn't call execute and therefore would not benefit from the execute
         configuration we have in place. Therefore this implementation calls execute instead.
@@ -235,7 +330,7 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         try:
             return next(self._query_data)
         except StopIteration:
-            return ()
+            return None
 
     def fetchall(
         self,
@@ -254,6 +349,104 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         )
         return list(self._query_data)
 
+    def _split_alter_expressions(
+        self,
+        alter_expressions: t.List[exp.Alter],
+    ) -> t.Tuple[NestedFieldsDict, t.List[exp.Alter]]:
+        """
+        Returns a dictionary of the nested fields to add and a list of the non-nested alter expressions.
+        """
+        nested_fields_to_add: NestedFieldsDict = defaultdict(list)
+        non_nested_expressions = []
+
+        for alter_expression in alter_expressions:
+            action = alter_expression.args["actions"][0]
+            if (
+                isinstance(action, exp.ColumnDef)
+                and isinstance(action.this, exp.Dot)
+                and isinstance(action.kind, exp.DataType)
+            ):
+                root_field, *leaf_fields = action.this.this.sql(dialect=self.dialect).split(".")
+                new_field = action.this.expression.sql(dialect=self.dialect)
+                data_type = action.kind.sql(dialect=self.dialect)
+                nested_fields_to_add[root_field].append((new_field, data_type, leaf_fields))
+            else:
+                non_nested_expressions.append(alter_expression)
+
+        return nested_fields_to_add, non_nested_expressions
+
+    def _build_nested_fields(
+        self,
+        current_fields: t.List[bigquery.SchemaField],
+        fields_to_add: t.List[NestedField],
+    ) -> t.List[bigquery.SchemaField]:
+        """
+        Recursively builds and updates the schema fields with the new nested fields.
+        """
+        from google.cloud import bigquery
+
+        new_fields = []
+        root: t.List[t.Tuple[str, str]] = []
+        leaves: NestedFieldsDict = defaultdict(list)
+        for new_field, data_type, leaf_fields in fields_to_add:
+            if leaf_fields:
+                leaves[leaf_fields[0]].append((new_field, data_type, leaf_fields[1:]))
+            else:
+                root.append((new_field, data_type))
+
+        for field in current_fields:
+            # If the new fields are nested, we need to recursively build them
+            if field.name in leaves:
+                subfields = list(field.fields)
+                subfields = self._build_nested_fields(subfields, leaves[field.name])
+                new_fields.append(
+                    bigquery.SchemaField(
+                        field.name, "RECORD", mode=field.mode, fields=tuple(subfields)
+                    )
+                )
+            else:
+                new_fields.append(field)
+
+        # Build and append the new root-level fields
+        new_fields.extend(
+            self.__get_bq_schemafield(
+                new_field[0], exp.DataType.build(new_field[1], dialect=self.dialect)
+            )
+            for new_field in root
+        )
+        return new_fields
+
+    def _update_table_schema_nested_fields(
+        self, nested_fields_to_add: NestedFieldsDict, table_name: str
+    ) -> None:
+        """
+        Updates a BigQuery table schema by adding the new nested fields provided.
+        """
+        from google.cloud import bigquery
+
+        table = self._get_table(table_name)
+        original_schema = table.schema
+        new_schema = []
+        for field in original_schema:
+            if field.name in nested_fields_to_add:
+                fields = self._build_nested_fields(
+                    list(field.fields), nested_fields_to_add[field.name]
+                )
+                new_schema.append(
+                    bigquery.SchemaField(
+                        field.name,
+                        "RECORD",
+                        mode=field.mode,
+                        fields=tuple(fields),
+                    )
+                )
+            else:
+                new_schema.append(field)
+
+        if new_schema != original_schema:
+            table.schema = new_schema
+            self.client.update_table(table, ["schema"])
+
     def __load_pandas_to_table(
         self,
         table: bigquery.Table,
@@ -287,20 +480,50 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         )
         return self._db_call(job.result)
 
+    def __get_bq_schemafield(self, name: str, tpe: exp.DataType) -> bigquery.SchemaField:
+        from google.cloud import bigquery
+
+        mode = "NULLABLE"
+        if tpe.is_type(exp.DataType.Type.ARRAY):
+            mode = "REPEATED"
+            tpe = tpe.expressions[0]
+
+        field_type = tpe.sql(dialect=self.dialect)
+        fields = []
+        if tpe.is_type(*exp.DataType.NESTED_TYPES):
+            field_type = "RECORD"
+            for inner_field in tpe.expressions:
+                if isinstance(inner_field, exp.ColumnDef):
+                    inner_name = inner_field.this.sql(dialect=self.dialect)
+                    inner_type = inner_field.kind
+                    if inner_type is None:
+                        raise ValueError(
+                            f"cannot convert unknown type to BQ schema field {inner_field}"
+                        )
+                    fields.append(self.__get_bq_schemafield(name=inner_name, tpe=inner_type))
+                else:
+                    raise ValueError(f"unexpected nested expression {inner_field}")
+
+        return bigquery.SchemaField(
+            name=name,
+            field_type=field_type,
+            mode=mode,
+            fields=fields,
+        )
+
     def __get_bq_schema(
         self, columns_to_types: t.Dict[str, exp.DataType]
     ) -> t.List[bigquery.SchemaField]:
         """
         Returns a bigquery schema object from a dictionary of column names to types.
         """
-        from google.cloud import bigquery
 
         precisionless_col_to_types = {
             col_name: remove_precision_parameterized_types(col_type)
             for col_name, col_type in columns_to_types.items()
         }
         return [
-            bigquery.SchemaField(col_name, col_type.sql(dialect=self.dialect))
+            self.__get_bq_schemafield(name=col_name, tpe=t.cast(exp.DataType, col_type))
             for col_name, col_type in precisionless_col_to_types.items()
         ]
 
@@ -325,7 +548,7 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         table_ = exp.to_table(table).copy()
 
         if not table_.catalog:
-            table_.set("catalog", exp.to_identifier(self.client.project))
+            table_.set("catalog", exp.to_identifier(self.default_catalog))
 
         return bigquery.Table(
             table_ref=self._table_name(table_),
@@ -383,6 +606,7 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
                 partition_type_sql,
                 granularity=granularity,
                 agg_func="ARRAY_AGG",
+                catalog=temp_table_name.catalog or self.default_catalog,
             )
 
             self.execute(
@@ -433,31 +657,41 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         table_name: TableName,
         column_comments: t.Dict[str, str],
         table_kind: str = "TABLE",
+        materialized_view: bool = False,
     ) -> None:
-        table = self._get_table(table_name)
+        if not (table_kind == "VIEW" and materialized_view):
+            table = self._get_table(table_name)
 
-        # convert Table object to dict
-        table_def = table.to_api_repr()
+            # convert Table object to dict
+            table_def = table.to_api_repr()
 
-        # set the column descriptions
-        for i in range(len(table_def["schema"]["fields"])):
-            comment = column_comments.get(table_def["schema"]["fields"][i]["name"], None)
-            if comment:
-                table_def["schema"]["fields"][i]["description"] = self._truncate_comment(
-                    comment, self.MAX_COLUMN_COMMENT_LENGTH
-                )
+            # Set column descriptions, supporting nested fields (e.g. record.field.nested_field)
+            for column, comment in column_comments.items():
+                fields = table_def["schema"]["fields"]
+                field_names = column.split(".")
+                last_index = len(field_names) - 1
 
-        # An "etag" is BQ versioning metadata that changes when an object is updated/modified. `update_table`
-        # compares the etags of the table object passed to it and the remote table, erroring if the etags
-        # don't match. We set the local etag to None to avoid this check.
-        table_def["etag"] = None
+                # Traverse the fields with nested fields down to leaf level
+                for idx, name in enumerate(field_names):
+                    if field := next((field for field in fields if field["name"] == name), None):
+                        if idx == last_index:
+                            field["description"] = self._truncate_comment(
+                                comment, self.MAX_COLUMN_COMMENT_LENGTH
+                            )
+                        else:
+                            fields = field.get("fields") or []
 
-        # convert dict back to a Table object
-        table = table.from_api_repr(table_def)
+            # An "etag" is BQ versioning metadata that changes when an object is updated/modified. `update_table`
+            # compares the etags of the table object passed to it and the remote table, erroring if the etags
+            # don't match. We set the local etag to None to avoid this check.
+            table_def["etag"] = None
 
-        # update table schema
-        logger.info(f"Registering column comments for table {table_name}")
-        self._db_call(self.client.update_table, table=table, fields=["schema"])
+            # convert dict back to a Table object
+            table = table.from_api_repr(table_def)
+
+            # update table schema
+            logger.info(f"Registering column comments for table {table_name}")
+            self._db_call(self.client.update_table, table=table, fields=["schema"])
 
     def _build_description_property_exp(
         self,
@@ -469,58 +703,76 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
             value=exp.Literal.string(trunc_method(description)),
         )
 
+    def _build_partitioned_by_exp(
+        self,
+        partitioned_by: t.List[exp.Expression],
+        *,
+        partition_interval_unit: t.Optional[IntervalUnit] = None,
+        columns_to_types: t.Optional[t.Dict[str, exp.DataType]] = None,
+        **kwargs: t.Any,
+    ) -> t.Optional[exp.PartitionedByProperty]:
+        if len(partitioned_by) > 1:
+            raise SQLMeshError("BigQuery only supports partitioning by a single column")
+
+        this = partitioned_by[0]
+        if (
+            isinstance(this, exp.Column)
+            and partition_interval_unit is not None
+            and not partition_interval_unit.is_minute
+        ):
+            column_type: t.Optional[exp.DataType] = (columns_to_types or {}).get(this.name)
+
+            if column_type == exp.DataType.build(
+                "date", dialect=self.dialect
+            ) and partition_interval_unit in (
+                IntervalUnit.MONTH,
+                IntervalUnit.YEAR,
+            ):
+                trunc_func = "DATE_TRUNC"
+            elif column_type == exp.DataType.build("timestamp", dialect=self.dialect):
+                trunc_func = "TIMESTAMP_TRUNC"
+            elif column_type == exp.DataType.build("datetime", dialect=self.dialect):
+                trunc_func = "DATETIME_TRUNC"
+            else:
+                trunc_func = ""
+
+            if trunc_func:
+                this = exp.func(
+                    trunc_func,
+                    this,
+                    exp.var(partition_interval_unit.value.upper()),
+                    dialect=self.dialect,
+                )
+
+        return exp.PartitionedByProperty(this=this)
+
     def _build_table_properties_exp(
         self,
         catalog_name: t.Optional[str] = None,
+        table_format: t.Optional[str] = None,
         storage_format: t.Optional[str] = None,
         partitioned_by: t.Optional[t.List[exp.Expression]] = None,
         partition_interval_unit: t.Optional[IntervalUnit] = None,
-        clustered_by: t.Optional[t.List[str]] = None,
+        clustered_by: t.Optional[t.List[exp.Expression]] = None,
         table_properties: t.Optional[t.Dict[str, exp.Expression]] = None,
         columns_to_types: t.Optional[t.Dict[str, exp.DataType]] = None,
         table_description: t.Optional[str] = None,
+        table_kind: t.Optional[str] = None,
+        **kwargs: t.Any,
     ) -> t.Optional[exp.Properties]:
         properties: t.List[exp.Expression] = []
 
-        if partitioned_by:
-            if len(partitioned_by) > 1:
-                raise SQLMeshError("BigQuery only supports partitioning by a single column")
+        if partitioned_by and (
+            partitioned_by_prop := self._build_partitioned_by_exp(
+                partitioned_by,
+                partition_interval_unit=partition_interval_unit,
+                columns_to_types=columns_to_types,
+            )
+        ):
+            properties.append(partitioned_by_prop)
 
-            this = partitioned_by[0]
-
-            if (
-                isinstance(this, exp.Column)
-                and partition_interval_unit is not None
-                and not partition_interval_unit.is_minute
-            ):
-                column_type: t.Optional[exp.DataType] = (columns_to_types or {}).get(this.name)
-
-                if column_type == exp.DataType.build(
-                    "date", dialect=self.dialect
-                ) and partition_interval_unit in (
-                    IntervalUnit.MONTH,
-                    IntervalUnit.YEAR,
-                ):
-                    trunc_func = "DATE_TRUNC"
-                elif column_type == exp.DataType.build("timestamp", dialect=self.dialect):
-                    trunc_func = "TIMESTAMP_TRUNC"
-                elif column_type == exp.DataType.build("datetime", dialect=self.dialect):
-                    trunc_func = "DATETIME_TRUNC"
-                else:
-                    trunc_func = ""
-
-                if trunc_func:
-                    this = exp.func(
-                        trunc_func,
-                        this,
-                        exp.var(partition_interval_unit.value.upper()),
-                        dialect=self.dialect,
-                    )
-
-            properties.append(exp.PartitionedByProperty(this=this))
-
-        if clustered_by:
-            properties.append(exp.Cluster(expressions=[exp.column(col) for col in clustered_by]))
+        if clustered_by and (clustered_by_exp := self._build_clustered_by_exp(clustered_by)):
+            properties.append(clustered_by_exp)
 
         if table_description:
             properties.append(
@@ -535,6 +787,65 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
             return exp.Properties(expressions=properties)
         return None
 
+    def _build_column_def(
+        self,
+        col_name: str,
+        column_descriptions: t.Optional[t.Dict[str, str]] = None,
+        engine_supports_schema_comments: bool = False,
+        col_type: t.Optional[exp.DATA_TYPE] = None,
+        nested_names: t.List[str] = [],
+    ) -> exp.ColumnDef:
+        # Helper function to build column definitions with column descriptions
+        def _build_struct_with_descriptions(
+            col_type: exp.DataType,
+            nested_names: t.List[str],
+        ) -> exp.DataType:
+            column_expressions = []
+            for column_def in col_type.expressions:
+                # This is expected to  be true, but this check is included as a
+                # precautionary measure in case of an unexpected edge case
+                if isinstance(column_def, exp.ColumnDef):
+                    column = self._build_column_def(
+                        col_name=column_def.name,
+                        column_descriptions=column_descriptions,
+                        engine_supports_schema_comments=engine_supports_schema_comments,
+                        col_type=column_def.kind,
+                        nested_names=nested_names,
+                    )
+                else:
+                    column = column_def
+                column_expressions.append(column)
+            return exp.DataType(this=col_type.this, expressions=column_expressions, nested=True)
+
+        # Recursively build column definitions for BigQuery's RECORDs (struct) and REPEATED RECORDs (array of struct)
+        if isinstance(col_type, exp.DataType) and (expressions := col_type.expressions):
+            if col_type.is_type(exp.DataType.Type.STRUCT):
+                col_type = _build_struct_with_descriptions(col_type, nested_names + [col_name])
+            elif col_type.is_type(exp.DataType.Type.ARRAY) and expressions[0].is_type(
+                exp.DataType.Type.STRUCT
+            ):
+                col_type = exp.DataType(
+                    this=exp.DataType.Type.ARRAY,
+                    expressions=[
+                        _build_struct_with_descriptions(
+                            col_type.expressions[0], nested_names + [col_name]
+                        )
+                    ],
+                    nested=True,
+                )
+
+        return exp.ColumnDef(
+            this=exp.to_identifier(col_name),
+            kind=col_type,
+            constraints=(
+                self._build_col_comment_exp(
+                    ".".join(nested_names + [col_name]), column_descriptions
+                )
+                if engine_supports_schema_comments and self.comments_enabled and column_descriptions
+                else None
+            ),
+        )
+
     def _build_col_comment_exp(
         self, col_name: str, column_descriptions: t.Dict[str, str]
     ) -> t.List[exp.ColumnConstraint]:
@@ -557,6 +868,7 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         self,
         view_properties: t.Optional[t.Dict[str, exp.Expression]] = None,
         table_description: t.Optional[str] = None,
+        **kwargs: t.Any,
     ) -> t.Optional[exp.Properties]:
         """Creates a SQLGlot table properties expression for view"""
         properties: t.List[exp.Expression] = []
@@ -668,26 +980,56 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
         # The BigQuery Client's list_tables method does not support filtering by table name, so we have to
         # resort to using SQL instead.
         schema = to_schema(schema_name)
-        catalog = schema.catalog or self.get_current_catalog()
-        query = exp.select(
-            exp.column("table_catalog").as_("catalog"),
-            exp.column("table_name").as_("name"),
-            exp.column("table_schema").as_("schema_name"),
-            exp.case()
-            .when(exp.column("table_type").eq("BASE TABLE"), exp.Literal.string("TABLE"))
-            .when(exp.column("table_type").eq("CLONE"), exp.Literal.string("TABLE"))
-            .when(exp.column("table_type").eq("EXTERNAL"), exp.Literal.string("TABLE"))
-            .when(exp.column("table_type").eq("SNAPSHOT"), exp.Literal.string("TABLE"))
-            .when(exp.column("table_type").eq("VIEW"), exp.Literal.string("VIEW"))
-            .when(
-                exp.column("table_type").eq("MATERIALIZED VIEW"),
-                exp.Literal.string("MATERIALIZED_VIEW"),
+        catalog = schema.catalog or self.default_catalog
+        query = (
+            exp.select(
+                exp.column("table_catalog").as_("catalog"),
+                exp.column("table_name").as_("name"),
+                exp.column("table_schema").as_("schema_name"),
+                exp.case()
+                .when(exp.column("table_type").eq("BASE TABLE"), exp.Literal.string("TABLE"))
+                .when(exp.column("table_type").eq("CLONE"), exp.Literal.string("TABLE"))
+                .when(exp.column("table_type").eq("EXTERNAL"), exp.Literal.string("TABLE"))
+                .when(exp.column("table_type").eq("SNAPSHOT"), exp.Literal.string("TABLE"))
+                .when(exp.column("table_type").eq("VIEW"), exp.Literal.string("VIEW"))
+                .when(
+                    exp.column("table_type").eq("MATERIALIZED VIEW"),
+                    exp.Literal.string("MATERIALIZED_VIEW"),
+                )
+                .else_(exp.column("table_type"))
+                .as_("type"),
+                exp.column("clustering_key", "ci").as_("clustering_key"),
             )
-            .else_(exp.column("table_type"))
-            .as_("type"),
-        ).from_(
-            exp.to_table(
-                f"`{catalog}`.`{schema.db}`.INFORMATION_SCHEMA.TABLES", dialect=self.dialect
+            .with_(
+                "clustering_info",
+                as_=exp.select(
+                    exp.column("table_catalog"),
+                    exp.column("table_schema"),
+                    exp.column("table_name"),
+                    parse_one(
+                        "string_agg(column_name order by clustering_ordinal_position)",
+                        dialect=self.dialect,
+                    ).as_("clustering_key"),
+                )
+                .from_(
+                    exp.to_table(
+                        f"`{catalog}`.`{schema.db}`.INFORMATION_SCHEMA.COLUMNS",
+                        dialect=self.dialect,
+                    )
+                )
+                .where(exp.column("clustering_ordinal_position").is_(exp.not_(exp.null())))
+                .group_by("1", "2", "3"),
+            )
+            .from_(
+                exp.to_table(
+                    f"`{catalog}`.`{schema.db}`.INFORMATION_SCHEMA.TABLES", dialect=self.dialect
+                )
+            )
+            .join(
+                "clustering_info",
+                using=["table_catalog", "table_schema", "table_name"],
+                join_type="left",
+                join_alias="ci",
             )
         )
         if object_names:
@@ -708,10 +1050,47 @@ class BigQueryEngineAdapter(InsertOverwriteWithMergeMixin):
                 schema=row.schema_name,  # type: ignore
                 name=row.name,  # type: ignore
                 type=DataObjectType.from_str(row.type),  # type: ignore
+                clustering_key=f"({row.clustering_key})" if row.clustering_key else None,  # type: ignore
             )
             for row in df.itertuples()
         ]
 
+    def _change_clustering_key_expr(
+        self, table: exp.Table, cluster_by: t.List[exp.Expression]
+    ) -> exp.Alter:
+        expr = super()._change_clustering_key_expr(table=table, cluster_by=cluster_by)
+        expr.meta[_CLUSTERING_META_KEY] = (table, cluster_by)
+        return expr
+
+    def _drop_clustering_key_expr(self, table: exp.Table) -> exp.Alter:
+        expr = super()._drop_clustering_key_expr(table=table)
+        expr.meta[_CLUSTERING_META_KEY] = (table, None)
+        return expr
+
+    def _update_clustering_key(
+        self, table_name: TableName, cluster_by: t.Optional[t.List[exp.Expression]]
+    ) -> None:
+        cluster_by = cluster_by or []
+        bq_table = self._get_table(table_name)
+
+        rendered_columns = [c.sql(dialect=self.dialect) for c in cluster_by]
+        bq_table.clustering_fields = (
+            rendered_columns or None
+        )  # causes a drop of the key if cluster_by is empty or None
+
+        self._db_call(self.client.update_table, table=bq_table, fields=["clustering_fields"])
+
+        if cluster_by:
+            # BigQuery only applies new clustering going forward, so this rewrites the columns to apply the new clustering to historical data
+            # ref: https://cloud.google.com/bigquery/docs/creating-clustered-tables#modifying-cluster-spec
+            self.execute(exp.update(table_name, {c: c for c in cluster_by}, where=exp.true()))
+
+    def _normalize_decimal_value(self, col: exp.Expression, precision: int) -> exp.Expression:
+        return exp.func("FORMAT", exp.Literal.string(f"%.{precision}f"), col)
+
+    def _normalize_nested_value(self, col: exp.Expression) -> exp.Expression:
+        return exp.func("TO_JSON_STRING", col, dialect=self.dialect)
+
     @property
     def _query_data(self) -> t.Any:
         return self._connection_pool.get_attribute("query_data")
@@ -788,24 +1167,24 @@ def select_partitions_expr(
     data_type: t.Union[str, exp.DataType],
     granularity: t.Optional[str] = None,
     agg_func: str = "MAX",
-    database: t.Optional[str] = None,
+    catalog: t.Optional[str] = None,
 ) -> str:
     """Generates a SQL expression that aggregates partition values for a table.
 
     Args:
-        schema: The schema (BigQueyr dataset) of the table.
+        schema: The schema (BigQuery dataset) of the table.
         table_name: The name of the table.
         data_type: The data type of the partition column.
         granularity: The granularity of the partition. Supported values are: 'day', 'month', 'year' and 'hour'.
         agg_func: The aggregation function to use.
-        database: The database (BigQuery project ID) of the table.
+        catalog: The catalog (BigQuery project ID) of the table.
 
     Returns:
         A SELECT statement that aggregates partition values for a table.
     """
     partitions_table_name = f"`{schema}`.INFORMATION_SCHEMA.PARTITIONS"
-    if database:
-        partitions_table_name = f"`{database}`.{partitions_table_name}"
+    if catalog:
+        partitions_table_name = f"`{catalog}`.{partitions_table_name}"
 
     if isinstance(data_type, exp.DataType):
         data_type = data_type.sql(dialect="bigquery")
