@@ -1,22 +1,29 @@
 from __future__ import annotations
 
+import inspect
+import sys
+import types
 import typing as t
 from enum import Enum
 from functools import reduce
 from itertools import chain
+from pathlib import Path
 from string import Template
+from datetime import datetime, date
 
 import sqlglot
 from jinja2 import Environment
-from sqlglot import Generator, exp
+from sqlglot import Generator, exp, parse_one
 from sqlglot.executor.env import ENV
 from sqlglot.executor.python import Python
 from sqlglot.helper import csv, ensure_collection
+from sqlglot.optimizer.normalize_identifiers import normalize_identifiers
 from sqlglot.schema import MappingSchema
 
 from sqlmesh.core import constants as c
 from sqlmesh.core.dialect import (
     SQLMESH_MACRO_PREFIX,
+    Dialect,
     MacroDef,
     MacroFunc,
     MacroSQL,
@@ -25,21 +32,37 @@ from sqlmesh.core.dialect import (
     StagedFilePath,
     normalize_model_name,
 )
-from sqlmesh.utils import DECORATOR_RETURN_TYPE, UniqueKeyDict, registry_decorator
+from sqlmesh.utils import (
+    DECORATOR_RETURN_TYPE,
+    UniqueKeyDict,
+    columns_to_types_all_known,
+    registry_decorator,
+)
+from sqlmesh.utils.date import DatetimeRanges, to_datetime, to_date
 from sqlmesh.utils.errors import MacroEvalError, SQLMeshError
 from sqlmesh.utils.jinja import JinjaMacroRegistry, has_jinja
 from sqlmesh.utils.metaprogramming import Executable, prepare_env, print_exception
 
 if t.TYPE_CHECKING:
+    from sqlglot.dialects.dialect import DialectType
     from sqlmesh.core._typing import TableName
     from sqlmesh.core.engine_adapter import EngineAdapter
     from sqlmesh.core.snapshot import Snapshot
 
 
+if sys.version_info >= (3, 10):
+    UNION_TYPES = (t.Union, types.UnionType)
+else:
+    UNION_TYPES = (t.Union,)
+
+
 class RuntimeStage(Enum):
     LOADING = "loading"
     CREATING = "creating"
     EVALUATING = "evaluating"
+    PROMOTING = "promoting"
+    AUDITING = "auditing"
+    TESTING = "testing"
 
 
 class MacroStrTemplate(Template):
@@ -47,6 +70,16 @@ class MacroStrTemplate(Template):
 
 
 EXPRESSIONS_NAME_MAP = {}
+SQL = t.NewType("SQL", str)
+
+SUPPORTED_TYPES = {
+    "t": t,
+    "typing": t,
+    "List": t.List,
+    "Tuple": t.Tuple,
+    "Union": t.Union,
+    "DatetimeRanges": DatetimeRanges,
+}
 
 for klass in sqlglot.Parser.EXPRESSION_PARSERS:
     name = klass if isinstance(klass, str) else klass.__name__  # type: ignore
@@ -112,14 +145,16 @@ class MacroEvaluator:
 
     def __init__(
         self,
-        dialect: str = "",
+        dialect: DialectType = "",
         python_env: t.Optional[t.Dict[str, Executable]] = None,
         jinja_env: t.Optional[Environment] = None,
         schema: t.Optional[MappingSchema] = None,
         runtime_stage: RuntimeStage = RuntimeStage.LOADING,
+        resolve_table: t.Optional[t.Callable[[str | exp.Expression], str]] = None,
         resolve_tables: t.Optional[t.Callable[[exp.Expression], exp.Expression]] = None,
         snapshots: t.Optional[t.Dict[str, Snapshot]] = None,
         default_catalog: t.Optional[str] = None,
+        path: Path = Path(),
     ):
         self.dialect = dialect
         self.generator = MacroDialect().generator()
@@ -127,27 +162,34 @@ class MacroEvaluator:
             "runtime_stage": runtime_stage.value,
             "default_catalog": default_catalog,
         }
-        self.env = {**ENV, "self": self}
+        self.env = {
+            **ENV,
+            "self": self,
+            "SQL": SQL,
+            "MacroEvaluator": MacroEvaluator,
+        }
         self.python_env = python_env or {}
         self._jinja_env: t.Optional[Environment] = jinja_env
         self.macros = {normalize_macro_name(k): v.func for k, v in macro.get_registry().items()}
         self._schema = schema
+        self._resolve_table = resolve_table
         self._resolve_tables = resolve_tables
         self.columns_to_types_called = False
         self._snapshots = snapshots if snapshots is not None else {}
         self.default_catalog = default_catalog
+        self._path = path
 
         prepare_env(self.python_env, self.env)
         for k, v in self.python_env.items():
             if v.is_definition:
                 self.macros[normalize_macro_name(k)] = self.env[v.name or k]
-            elif v.is_import and getattr(self.env.get(k), "__sqlmesh_macro__", None):
+            elif v.is_import and getattr(self.env.get(k), c.SQLMESH_MACRO, None):
                 self.macros[normalize_macro_name(k)] = self.env[k]
             elif v.is_value:
                 self.locals[k] = self.env[k]
 
     def send(
-        self, name: str, *args: t.Any
+        self, name: str, *args: t.Any, **kwargs: t.Any
     ) -> t.Union[None, exp.Expression, t.List[exp.Expression]]:
         func = self.macros.get(normalize_macro_name(name))
 
@@ -155,7 +197,7 @@ class MacroEvaluator:
             raise SQLMeshError(f"Macro '{name}' does not exist.")
 
         try:
-            return func(self, *args)
+            return call_macro(func, self.dialect, self._path, self, *args, **kwargs)  # type: ignore
         except Exception as e:
             print_exception(e, self.python_env)
             raise MacroEvalError("Error trying to eval macro.") from e
@@ -272,7 +314,22 @@ class MacroEvaluator:
             )
         else:
             func = t.cast(exp.Anonymous, node.this)
-            result = self.send(func.name, *func.expressions)
+
+            args = []
+            kwargs = {}
+            for e in func.expressions:
+                if isinstance(e, exp.PropertyEQ):
+                    kwargs[e.this.name] = e.expression
+                else:
+                    if kwargs:
+                        raise MacroEvalError(
+                            "Positional argument cannot follow keyword argument.\n  "
+                            f"{func.sql(dialect=self.dialect)} at '{self._path}'"
+                        )
+
+                    args.append(e)
+
+            result = self.send(func.name, *args, **kwargs)
 
         if result is None:
             return None
@@ -329,18 +386,29 @@ class MacroEvaluator:
 
     def columns_to_types(self, model_name: TableName | exp.Column) -> t.Dict[str, exp.DataType]:
         """Returns the columns-to-types mapping corresponding to the specified model."""
-        if self._schema is None or self._schema.empty:
+
+        # We only return this dummy schema at load time, because if we don't actually know the
+        # target model's schema at creation/evaluation time, returning a dummy schema could lead
+        # to unintelligible errors when the query is executed
+        if (self._schema is None or self._schema.empty) and self.runtime_stage == "loading":
             self.columns_to_types_called = True
             return {"__schema_unavailable_at_load__": exp.DataType.build("unknown")}
 
-        if isinstance(model_name, exp.Column):
-            model_name = exp.table_(
-                model_name.this,
-                db=model_name.args.get("table"),
-                catalog=model_name.args.get("db"),
-            )
+        normalized_model_name = normalize_model_name(
+            model_name,
+            default_catalog=self.default_catalog,
+            dialect=self.dialect,
+        )
+        model_name = exp.to_table(normalized_model_name)
+
+        columns_to_types = (
+            self._schema.find(model_name, ensure_data_types=True) if self._schema else None
+        )
+        if columns_to_types is None:
+            snapshot = self.get_snapshot(model_name)
+            if snapshot and snapshot.node.is_model:
+                columns_to_types = snapshot.node.columns_to_types  # type: ignore
 
-        columns_to_types = self._schema.find(exp.to_table(model_name))
         if columns_to_types is None:
             raise SQLMeshError(f"Schema for model '{model_name}' can't be statically determined.")
 
@@ -356,6 +424,14 @@ class MacroEvaluator:
             )
         )
 
+    def resolve_table(self, table: str | exp.Expression) -> str:
+        """Gets the physical table name for a given model."""
+        if not self._resolve_table:
+            raise SQLMeshError(
+                "Macro evaluator not properly initialized with resolve_table lambda."
+            )
+        return self._resolve_table(table)
+
     def resolve_tables(self, query: exp.Expression) -> exp.Expression:
         """Resolves queries with references to SQLMesh model names to their physical tables."""
         if not self._resolve_tables:
@@ -369,6 +445,14 @@ class MacroEvaluator:
         """Returns the current runtime stage of the macro evaluation."""
         return self.locals["runtime_stage"]
 
+    @property
+    def this_model(self) -> str:
+        """Returns the resolved name of the surrounding model."""
+        this_model = self.locals.get("this_model")
+        if not this_model:
+            raise SQLMeshError("Model name is not available in the macro evaluator.")
+        return this_model.sql(dialect=self.dialect, identify=True, comments=False)
+
     @property
     def engine_adapter(self) -> EngineAdapter:
         engine_adapter = self.locals.get("engine_adapter")
@@ -388,6 +472,10 @@ class MacroEvaluator:
         """Returns the value of the specified variable, or the default value if it doesn't exist."""
         return (self.locals.get(c.SQLMESH_VARS) or {}).get(var_name.lower(), default)
 
+    def _coerce(self, expr: exp.Expression, typ: t.Any, strict: bool = False) -> t.Any:
+        """Coerces the given expression to the specified type on a best-effort basis."""
+        return _coerce(expr, typ, self.dialect, self._path, strict)
+
 
 class macro(registry_decorator):
     """Specifies a function is a macro and registers it the global MACROS registry.
@@ -408,13 +496,19 @@ class macro(registry_decorator):
 
     registry_name = "macros"
 
+    def __init__(self, *args: t.Any, metadata_only: bool = False, **kwargs: t.Any) -> None:
+        super().__init__(*args, **kwargs)
+        self.metadata_only = metadata_only
+
     def __call__(
         self, func: t.Callable[..., DECORATOR_RETURN_TYPE]
     ) -> t.Callable[..., DECORATOR_RETURN_TYPE]:
+        if self.metadata_only:
+            setattr(func, c.SQLMESH_METADATA, self.metadata_only)
         wrapper = super().__call__(func)
 
-        # This is useful to identify macros at runtime
-        setattr(wrapper, "__sqlmesh_macro__", True)
+        # This is used to identify macros at runtime to unwrap during serialization.
+        setattr(wrapper, c.SQLMESH_MACRO, True)
         return wrapper
 
 
@@ -637,15 +731,17 @@ def eval_(evaluator: MacroEvaluator, condition: exp.Condition) -> t.Any:
     return evaluator.eval_expression(condition)
 
 
+# macros with union types need to use t.Union since | isn't available until 3.9
 @macro()
 def star(
     evaluator: MacroEvaluator,
     relation: exp.Table,
     alias: exp.Column = t.cast(exp.Column, exp.column("")),
-    except_: exp.Array | exp.Tuple = exp.Tuple(this=[]),
+    exclude: t.Union[exp.Array, exp.Tuple] = exp.Tuple(expressions=[]),
     prefix: exp.Literal = exp.Literal.string(""),
     suffix: exp.Literal = exp.Literal.string(""),
     quote_identifiers: exp.Boolean = exp.true(),
+    except_: t.Union[exp.Array, exp.Tuple] = exp.Tuple(expressions=[]),
 ) -> t.List[exp.Alias]:
     """Returns a list of projections for the given relation.
 
@@ -653,26 +749,35 @@ def star(
         evaluator: MacroEvaluator that invoked the macro
         relation: The relation to select star from
         alias: The alias of the relation
-        except_: Columns to exclude
+        exclude: Columns to exclude
         prefix: A prefix to use for all selections
         suffix: A suffix to use for all selections
         quote_identifiers: Whether or not quote the resulting aliases, defaults to true
+        except_: Alias for exclude (TODO: deprecate this, update docs)
 
     Returns:
         An array of columns.
 
     Example:
-        >>> from sqlglot import parse_one
+        >>> from sqlglot import parse_one, exp
         >>> from sqlglot.schema import MappingSchema
         >>> from sqlmesh.core.macros import MacroEvaluator
-        >>> sql = "SELECT @STAR(foo, bar, [c], 'baz_') FROM foo AS bar"
-        >>> MacroEvaluator(schema=MappingSchema({"foo": {"a": "string", "b": "string", "c": "string", "d": "int"}})).transform(parse_one(sql)).sql()
+        >>> sql = "SELECT @STAR(foo, bar, exclude := [c], prefix := 'baz_') FROM foo AS bar"
+        >>> MacroEvaluator(schema=MappingSchema({"foo": {"a": exp.DataType.build("string"), "b": exp.DataType.build("string"), "c": exp.DataType.build("string"), "d": exp.DataType.build("int")}})).transform(parse_one(sql)).sql()
         'SELECT CAST("bar"."a" AS TEXT) AS "baz_a", CAST("bar"."b" AS TEXT) AS "baz_b", CAST("bar"."d" AS INT) AS "baz_d" FROM foo AS bar'
     """
     if alias and not isinstance(alias, (exp.Identifier, exp.Column)):
         raise SQLMeshError(f"Invalid alias '{alias}'. Expected an identifier.")
-    if except_ and not isinstance(except_, (exp.Array, exp.Tuple)):
-        raise SQLMeshError(f"Invalid except '{except_}'. Expected an array.")
+    if exclude and not isinstance(exclude, (exp.Array, exp.Tuple)):
+        raise SQLMeshError(f"Invalid exclude '{exclude}'. Expected an array.")
+    if except_ != exp.tuple_():
+        from sqlmesh.core.console import get_console
+
+        get_console().log_warning(
+            "The 'except_' argument in @STAR will soon be deprecated. Use 'exclude' instead."
+        )
+        if not isinstance(exclude, (exp.Array, exp.Tuple)):
+            raise SQLMeshError(f"Invalid exclude_ '{exclude}'. Expected an array.")
     if prefix and not isinstance(prefix, exp.Literal):
         raise SQLMeshError(f"Invalid prefix '{prefix}'. Expected a literal.")
     if suffix and not isinstance(suffix, exp.Literal):
@@ -680,31 +785,54 @@ def star(
     if not isinstance(quote_identifiers, exp.Boolean):
         raise SQLMeshError(f"Invalid quote_identifiers '{quote_identifiers}'. Expected a boolean.")
 
-    exclude = {e.name for e in except_.expressions}
+    excluded_names = {
+        normalize_identifiers(excluded, dialect=evaluator.dialect).name
+        for excluded in exclude.expressions or except_.expressions
+    }
     quoted = quote_identifiers.this
-    table_identifier = alias.name or relation.name
+    table_identifier = normalize_identifiers(
+        alias if alias.name else relation, dialect=evaluator.dialect
+    ).name
 
+    columns_to_types = {
+        k: v for k, v in evaluator.columns_to_types(relation).items() if k not in excluded_names
+    }
+    if columns_to_types_all_known(columns_to_types):
+        return [
+            exp.cast(
+                exp.column(column, table=table_identifier, quoted=quoted),
+                dtype,
+                dialect=evaluator.dialect,
+            ).as_(f"{prefix.this}{column}{suffix.this}", quoted=quoted)
+            for column, dtype in columns_to_types.items()
+        ]
     return [
-        exp.cast(
-            exp.column(column, table=table_identifier, quoted=quoted),
-            type_,
-            dialect=evaluator.dialect,
-        ).as_(f"{prefix.this}{column}{suffix.this}", quoted=quoted)
-        for column, type_ in evaluator.columns_to_types(relation).items()
-        if column not in exclude
+        exp.column(column, table=table_identifier, quoted=quoted).as_(
+            f"{prefix.this}{column}{suffix.this}", quoted=quoted
+        )
+        for column, type_ in columns_to_types.items()
     ]
 
 
 @macro()
-def generate_surrogate_key(_: MacroEvaluator, *fields: exp.Column) -> exp.Func:
-    """Generates a surrogate key for the given fields.
+def generate_surrogate_key(
+    evaluator: MacroEvaluator,
+    *fields: exp.Expression,
+    hash_function: exp.Literal = exp.Literal.string("MD5"),
+) -> exp.Func:
+    """Generates a surrogate key (string) for the given fields.
 
     Example:
         >>> from sqlglot import parse_one
         >>> from sqlmesh.core.macros import MacroEvaluator
+        >>>
         >>> sql = "SELECT @GENERATE_SURROGATE_KEY(a, b, c) FROM foo"
-        >>> MacroEvaluator().transform(parse_one(sql)).sql()
-        "SELECT MD5(CONCAT(COALESCE(CAST(a AS TEXT), '_sqlmesh_surrogate_key_null_'), '|', COALESCE(CAST(b AS TEXT), '_sqlmesh_surrogate_key_null_'), '|', COALESCE(CAST(c AS TEXT), '_sqlmesh_surrogate_key_null_'))) FROM foo"
+        >>> MacroEvaluator(dialect="bigquery").transform(parse_one(sql, dialect="bigquery")).sql("bigquery")
+        "SELECT TO_HEX(MD5(CONCAT(COALESCE(CAST(a AS STRING), '_sqlmesh_surrogate_key_null_'), '|', COALESCE(CAST(b AS STRING), '_sqlmesh_surrogate_key_null_'), '|', COALESCE(CAST(c AS STRING), '_sqlmesh_surrogate_key_null_')))) FROM foo"
+        >>>
+        >>> sql = "SELECT @GENERATE_SURROGATE_KEY(a, b, c, hash_function := 'SHA256') FROM foo"
+        >>> MacroEvaluator(dialect="bigquery").transform(parse_one(sql, dialect="bigquery")).sql("bigquery")
+        "SELECT SHA256(CONCAT(COALESCE(CAST(a AS STRING), '_sqlmesh_surrogate_key_null_'), '|', COALESCE(CAST(b AS STRING), '_sqlmesh_surrogate_key_null_'), '|', COALESCE(CAST(c AS STRING), '_sqlmesh_surrogate_key_null_'))) FROM foo"
     """
     string_fields: t.List[exp.Expression] = []
     for i, field in enumerate(fields):
@@ -717,11 +845,20 @@ def generate_surrogate_key(_: MacroEvaluator, *fields: exp.Column) -> exp.Func:
                 exp.Literal.string("_sqlmesh_surrogate_key_null_"),
             )
         )
-    return exp.func("MD5", exp.func("CONCAT", *string_fields))
+
+    func = exp.func(
+        hash_function.name,
+        exp.func("CONCAT", *string_fields),
+        dialect=evaluator.dialect,
+    )
+    if isinstance(func, exp.MD5Digest):
+        func = exp.MD5(this=func.this)
+
+    return func
 
 
 @macro()
-def safe_add(_: MacroEvaluator, *fields: exp.Column) -> exp.Case:
+def safe_add(_: MacroEvaluator, *fields: exp.Expression) -> exp.Case:
     """Adds numbers together, substitutes nulls for 0s and only returns null if all fields are null.
 
     Example:
@@ -774,7 +911,7 @@ def safe_div(_: MacroEvaluator, numerator: exp.Expression, denominator: exp.Expr
 def union(
     evaluator: MacroEvaluator,
     type_: exp.Literal = exp.Literal.string("ALL"),
-    *tables: exp.Column,  # These represent tables but the ast node will be columns
+    *tables: exp.Table,
 ) -> exp.Query:
     """Returns a UNION of the given tables. Only choosing columns that have the same name and type.
 
@@ -856,7 +993,7 @@ def haversine_distance(
 def pivot(
     evaluator: MacroEvaluator,
     column: exp.Column,
-    values: exp.Array | exp.Tuple,
+    values: t.Union[exp.Array, exp.Tuple],
     alias: exp.Boolean = exp.true(),
     agg: exp.Literal = exp.Literal.string("SUM"),
     cmp: exp.Literal = exp.Literal.string("="),
@@ -922,6 +1059,317 @@ def var(
     return exp.convert(evaluator.var(var_name.this, default))
 
 
+@macro()
+def deduplicate(
+    evaluator: MacroEvaluator,
+    relation: exp.Expression,
+    partition_by: t.List[exp.Expression],
+    order_by: t.List[str],
+) -> exp.Query:
+    """Returns a QUERY to deduplicate rows within a table
+
+    Args:
+        relation: table or CTE name to deduplicate
+        partition_by: column names, or expressions to use to identify a window of rows out of which to select one as the deduplicated row
+        order_by: A list of strings representing the ORDER BY clause
+
+    Example:
+        >>> from sqlglot import parse_one
+        >>> from sqlglot.schema import MappingSchema
+        >>> from sqlmesh.core.macros import MacroEvaluator
+        >>> sql = "@deduplicate(demo.table, [user_id, cast(timestamp as date)], ['timestamp desc', 'status asc'])"
+        >>> MacroEvaluator().transform(parse_one(sql)).sql()
+        'SELECT * FROM demo.table QUALIFY ROW_NUMBER() OVER (PARTITION BY user_id, CAST(timestamp AS DATE) ORDER BY timestamp DESC, status ASC) = 1'
+    """
+    if not isinstance(partition_by, list):
+        raise SQLMeshError(
+            "partition_by must be a list of columns: [<column>, cast(<column> as <type>)]"
+        )
+
+    if not isinstance(order_by, list):
+        raise SQLMeshError(
+            "order_by must be a list of strings, optional - nulls ordering: ['<column> <asc|desc> nulls <first|last>']"
+        )
+
+    partition_clause = exp.tuple_(*partition_by)
+
+    order_expressions = [
+        evaluator.transform(parse_one(order_item, into=exp.Ordered, dialect=evaluator.dialect))
+        for order_item in order_by
+    ]
+
+    if not order_expressions:
+        raise SQLMeshError(
+            "order_by must be a list of strings, optional - nulls ordering: ['<column> <asc|desc> nulls <first|last>']"
+        )
+
+    order_clause = exp.Order(expressions=order_expressions)
+
+    window_function = exp.Window(
+        this=exp.RowNumber(), partition_by=partition_clause, order=order_clause
+    )
+
+    first_unique_row = window_function.eq(1)
+
+    query = exp.select("*").from_(relation).qualify(first_unique_row)
+
+    return query
+
+
+@macro()
+def date_spine(
+    evaluator: MacroEvaluator,
+    datepart: exp.Expression,
+    start_date: exp.Expression,
+    end_date: exp.Expression,
+) -> exp.Select:
+    """Returns a query that produces a date spine with the given datepart, and range of start_date and end_date. Useful for joining as a date lookup table.
+
+    Args:
+        datepart: The datepart to use for the date spine - day, week, month, quarter, year
+        start_date: The start date for the date spine in format YYYY-MM-DD
+        end_date: The end date for the date spine in format YYYY-MM-DD
+
+    Example:
+        >>> from sqlglot import parse_one
+        >>> from sqlglot.schema import MappingSchema
+        >>> from sqlmesh.core.macros import MacroEvaluator
+        >>> sql = "@date_spine('week', '2022-01-20', '2024-12-16')"
+        >>> MacroEvaluator().transform(parse_one(sql)).sql()
+        "SELECT date_week FROM UNNEST(GENERATE_DATE_ARRAY(CAST(\'2022-01-20\' AS DATE), CAST(\'2024-12-16\' AS DATE), INTERVAL \'1\' WEEK)) AS _exploded(date_week)"
+    """
+    datepart_name = datepart.name.lower()
+    if datepart_name not in ("day", "week", "month", "quarter", "year"):
+        raise SQLMeshError(
+            f"Invalid datepart '{datepart_name}'. Expected: 'day', 'week', 'month', 'quarter', or 'year'"
+        )
+
+    start_date_name = start_date.name
+    end_date_name = end_date.name
+
+    try:
+        if start_date.is_string and end_date.is_string:
+            start_date_obj = datetime.strptime(start_date_name, "%Y-%m-%d").date()
+            end_date_obj = datetime.strptime(end_date_name, "%Y-%m-%d").date()
+        else:
+            start_date_obj = None
+            end_date_obj = None
+    except Exception as e:
+        raise SQLMeshError(
+            f"Invalid date format - start_date and end_date must be in format: YYYY-MM-DD. Error: {e}"
+        )
+
+    if start_date_obj and end_date_obj:
+        if start_date_obj > end_date_obj:
+            raise SQLMeshError(
+                f"Invalid date range - start_date '{start_date_name}' is after end_date '{end_date_name}'."
+            )
+
+        start_date = exp.cast(start_date, "DATE")
+        end_date = exp.cast(end_date, "DATE")
+
+    if datepart_name == "quarter" and evaluator.dialect in (
+        "spark",
+        "spark2",
+        "databricks",
+        "postgres",
+    ):
+        date_interval = exp.Interval(this=exp.Literal.number(3), unit=exp.var("month"))
+    else:
+        date_interval = exp.Interval(this=exp.Literal.number(1), unit=exp.var(datepart_name))
+
+    generate_date_array = exp.func(
+        "GENERATE_DATE_ARRAY",
+        start_date,
+        end_date,
+        date_interval,
+    )
+
+    alias_name = f"date_{datepart_name}"
+    exploded = exp.alias_(exp.func("unnest", generate_date_array), "_exploded", table=[alias_name])
+
+    return exp.select(alias_name).from_(exploded)
+
+
+@macro()
+def resolve_template(
+    evaluator: MacroEvaluator,
+    template: exp.Literal,
+    mode: str = "literal",
+) -> t.Union[exp.Literal, exp.Table]:
+    """
+    Generates either a String literal or an exp.Table representing a physical table location, based on rendering the provided template String literal.
+
+    Note: It relies on the @this_model variable being available in the evaluation context (@this_model resolves to an exp.Table object
+    representing the current physical table).
+    Therefore, the @resolve_template macro must be used at creation or evaluation time and not at load time.
+
+    Args:
+        template: Template string literal. Can contain the following placeholders:
+            @{catalog_name} -> replaced with the catalog of the exp.Table returned from @this_model
+            @{schema_name} -> replaced with the schema of the exp.Table returned from @this_model
+            @{table_name} -> replaced with the name of the exp.Table returned from @this_model
+        mode: What to return.
+            'literal' -> return an exp.Literal string
+            'table' -> return an exp.Table
+
+    Example:
+        >>> from sqlglot import parse_one, exp
+        >>> from sqlmesh.core.macros import MacroEvaluator, RuntimeStage
+        >>> sql = "@resolve_template('s3://data-bucket/prod/@{catalog_name}/@{schema_name}/@{table_name}')"
+        >>> evaluator = MacroEvaluator(runtime_stage=RuntimeStage.CREATING)
+        >>> evaluator.locals.update({"this_model": exp.to_table("test_catalog.sqlmesh__test.test__test_model__2517971505")})
+        >>> evaluator.transform(parse_one(sql)).sql()
+        "'s3://data-bucket/prod/test_catalog/sqlmesh__test/test__test_model__2517971505'"
+    """
+    if "this_model" in evaluator.locals:
+        this_model = exp.to_table(evaluator.locals["this_model"], dialect=evaluator.dialect)
+        template_str: str = template.this
+        result = (
+            template_str.replace("@{catalog_name}", this_model.catalog)
+            .replace("@{schema_name}", this_model.db)
+            .replace("@{table_name}", this_model.name)
+        )
+
+        if mode.lower() == "table":
+            return exp.to_table(result, dialect=evaluator.dialect)
+        return exp.Literal.string(result)
+    elif evaluator.runtime_stage != RuntimeStage.LOADING.value:
+        # only error if we are CREATING, EVALUATING or TESTING and @this_model is not present; this could indicate a bug
+        # otherwise, for LOADING, it's a no-op
+        raise SQLMeshError(
+            "@this_model must be present in the macro evaluation context in order to use @resolve_template"
+        )
+
+    return template
+
+
 def normalize_macro_name(name: str) -> str:
     """Prefix macro name with @ and upcase"""
     return f"@{name.upper()}"
+
+
+for m in macro.get_registry().values():
+    setattr(m, c.SQLMESH_BUILTIN, True)
+
+
+def call_macro(
+    func: t.Callable,
+    dialect: DialectType,
+    path: Path,
+    *args: t.Any,
+    **kwargs: t.Any,
+) -> t.Any:
+    # Bind the macro's actual parameters to its formal parameters
+    sig = inspect.signature(func)
+    bound = sig.bind(*args, **kwargs)
+    bound.apply_defaults()
+
+    try:
+        annotations = t.get_type_hints(func, localns=SUPPORTED_TYPES)
+    except (NameError, TypeError):  # forward references aren't handled
+        annotations = {}
+
+    # If the macro is annotated, we try coerce the actual parameters to the corresponding types
+    if annotations:
+        for arg, value in bound.arguments.items():
+            typ = annotations.get(arg)
+            if not typ:
+                continue
+
+            # Changes to bound.arguments will reflect in bound.args and bound.kwargs
+            # https://docs.python.org/3/library/inspect.html#inspect.BoundArguments.arguments
+            param = sig.parameters[arg]
+            if param.kind is inspect.Parameter.VAR_POSITIONAL:
+                bound.arguments[arg] = tuple(_coerce(v, typ, dialect, path) for v in value)
+            elif param.kind is inspect.Parameter.VAR_KEYWORD:
+                bound.arguments[arg] = {k: _coerce(v, typ, dialect, path) for k, v in value.items()}
+            else:
+                bound.arguments[arg] = _coerce(value, typ, dialect, path)
+
+    return func(*bound.args, **bound.kwargs)
+
+
+def _coerce(
+    expr: t.Any,
+    typ: t.Any,
+    dialect: DialectType,
+    path: Path,
+    strict: bool = False,
+) -> t.Any:
+    """Coerces the given expression to the specified type on a best-effort basis."""
+    base_err_msg = f"Failed to coerce expression '{expr}' to type '{typ}'."
+    try:
+        if typ is None or typ is t.Any or not isinstance(expr, exp.Expression):
+            return expr
+        base = t.get_origin(typ) or typ
+
+        # We need to handle Union and TypeVars first since we cannot use isinstance with it
+        if base in UNION_TYPES:
+            for branch in t.get_args(typ):
+                try:
+                    return _coerce(expr, branch, dialect, path, strict=True)
+                except Exception:
+                    pass
+            raise SQLMeshError(base_err_msg)
+        if base is SQL and isinstance(expr, exp.Expression):
+            return expr.sql(dialect)
+
+        if isinstance(expr, base):
+            return expr
+        if issubclass(base, exp.Expression):
+            d = Dialect.get_or_raise(dialect)
+            into = base if base in d.parser_class.EXPRESSION_PARSERS else None
+            if into is None:
+                if isinstance(expr, exp.Literal):
+                    coerced = parse_one(expr.this)
+                else:
+                    raise SQLMeshError(
+                        f"{base_err_msg} Coercion to {base} requires a literal expression."
+                    )
+            else:
+                coerced = parse_one(
+                    expr.this if isinstance(expr, exp.Literal) else expr.sql(), into=into
+                )
+            if isinstance(coerced, base):
+                return coerced
+            raise SQLMeshError(base_err_msg)
+
+        if base in (int, float, str) and isinstance(expr, exp.Literal):
+            return base(expr.this)
+        if base is str and isinstance(expr, exp.Column) and not expr.table:
+            return expr.name
+        if base is bool and isinstance(expr, exp.Boolean):
+            return expr.this
+        if base is datetime and isinstance(expr, exp.Literal):
+            return to_datetime(expr.this)
+        if base is date and isinstance(expr, exp.Literal):
+            return to_date(expr.this)
+        if base is tuple and isinstance(expr, (exp.Tuple, exp.Array)):
+            generic = t.get_args(typ)
+            if not generic:
+                return tuple(expr.expressions)
+            if generic[-1] is ...:
+                return tuple(_coerce(expr, generic[0], dialect, path) for expr in expr.expressions)
+            elif len(generic) == len(expr.expressions):
+                return tuple(
+                    _coerce(expr, generic[i], dialect, path)
+                    for i, expr in enumerate(expr.expressions)
+                )
+            raise SQLMeshError(f"{base_err_msg} Expected {len(generic)} items.")
+        if base is list and isinstance(expr, (exp.Array, exp.Tuple)):
+            generic = t.get_args(typ)
+            if not generic:
+                return expr.expressions
+            return [_coerce(expr, generic[0], dialect, path) for expr in expr.expressions]
+        raise SQLMeshError(base_err_msg)
+    except Exception:
+        if strict:
+            raise
+
+        from sqlmesh.core.console import get_console
+
+        get_console().log_error(
+            f"Coercion of expression '{expr}' to type '{typ}' failed. Using non coerced expression at '{path}'",
+        )
+        return expr
