@@ -2,10 +2,8 @@ from __future__ import annotations
 
 import logging
 import re
-import sys
 import typing as t
 from collections import defaultdict
-from datetime import datetime
 from functools import cached_property
 
 
@@ -18,17 +16,23 @@ from sqlmesh.core.config import (
 from sqlmesh.core.context_diff import ContextDiff
 from sqlmesh.core.environment import EnvironmentNamingInfo
 from sqlmesh.core.plan.definition import Plan, SnapshotMapping, earliest_interval_start
-from sqlmesh.core.schema_diff import SchemaDiffer, has_drop_alteration
+from sqlmesh.core.schema_diff import SchemaDiffer, has_drop_alteration, get_dropped_column_names
 from sqlmesh.core.snapshot import (
     DeployabilityIndex,
     Snapshot,
     SnapshotChangeCategory,
-    categorize_change,
 )
-from sqlmesh.core.snapshot.definition import Interval, SnapshotId, start_date
+from sqlmesh.core.snapshot.categorizer import categorize_change
+from sqlmesh.core.snapshot.definition import Interval, SnapshotId
 from sqlmesh.utils import columns_to_types_all_known, random_id
 from sqlmesh.utils.dag import DAG
-from sqlmesh.utils.date import TimeLike, now, to_datetime, yesterday_ds
+from sqlmesh.utils.date import (
+    TimeLike,
+    now,
+    to_datetime,
+    yesterday_ds,
+    to_timestamp,
+)
 from sqlmesh.utils.errors import NoChangesPlanError, PlanError, SQLMeshError
 
 logger = logging.getLogger(__name__)
@@ -51,6 +55,7 @@ class PlanBuilder:
             part of the target environment have no data gaps when compared against previous
             snapshots for same nodes.
         skip_backfill: Whether to skip the backfill step.
+        empty_backfill: Like skip_backfill, but also records processed intervals.
         is_dev: Whether this plan is for development purposes.
         forward_only: Whether the purpose of the plan is to make forward only changes.
         allow_destructive_models: A list of fully qualified model names whose forward-only changes are allowed to be destructive.
@@ -69,6 +74,7 @@ class PlanBuilder:
             environment state, or to use whatever snapshots are in the current environment state even if
             the environment is not finalized.
         engine_schema_differ: Schema differ from the context engine adapter.
+        interval_end_per_model: The mapping from model FQNs to target end dates.
     """
 
     def __init__(
@@ -83,6 +89,7 @@ class PlanBuilder:
         backfill_models: t.Optional[t.Iterable[str]] = None,
         no_gaps: bool = False,
         skip_backfill: bool = False,
+        empty_backfill: bool = False,
         is_dev: bool = False,
         forward_only: bool = False,
         allow_destructive_models: t.Optional[t.Iterable[str]] = None,
@@ -98,11 +105,13 @@ class PlanBuilder:
         enable_preview: bool = False,
         end_bounded: bool = False,
         ensure_finalized_snapshots: bool = False,
+        interval_end_per_model: t.Optional[t.Dict[str, int]] = None,
         console: t.Optional[Console] = None,
     ):
         self._context_diff = context_diff
         self._no_gaps = no_gaps
         self._skip_backfill = skip_backfill
+        self._empty_backfill = empty_backfill
         self._is_dev = is_dev
         self._forward_only = forward_only
         self._allow_destructive_models = set(
@@ -111,6 +120,7 @@ class PlanBuilder:
         self._enable_preview = enable_preview
         self._end_bounded = end_bounded
         self._ensure_finalized_snapshots = ensure_finalized_snapshots
+        self._interval_end_per_model = interval_end_per_model
         self._environment_ttl = environment_ttl
         self._categorizer_config = categorizer_config or CategorizerConfig()
         self._auto_categorization_enabled = auto_categorization_enabled
@@ -123,9 +133,12 @@ class PlanBuilder:
         self._apply = apply
         self._engine_schema_differ = engine_schema_differ
         self._console = console or get_console()
+        self._choices: t.Dict[SnapshotId, SnapshotChangeCategory] = {}
 
         self._start = start
-        if not self._start and self._forward_only_preview_needed:
+        if not self._start and (
+            self._forward_only_preview_needed or self._auto_restatement_preview_needed
+        ):
             self._start = default_start or yesterday_ds()
 
         self._plan_id: str = random_id()
@@ -137,6 +150,7 @@ class PlanBuilder:
             environment_catalog_mapping or {},
             name=self._context_diff.environment,
             suffix_target=environment_suffix_target,
+            normalize_name=self._context_diff.normalize_environment_name,
         )
 
         self._latest_plan: t.Optional[Plan] = None
@@ -179,9 +193,22 @@ class PlanBuilder:
             snapshot: The target snapshot.
             choice: The user decision on how to version the target snapshot and its children.
         """
-        plan = self.build()
-        self._set_choice(snapshot, choice, plan.directly_modified, plan.indirectly_modified)
-        self._adjust_new_snapshot_intervals()
+        if self._forward_only:
+            raise PlanError("Choice setting is not supported by a forward-only plan.")
+        if not self._is_new_snapshot(snapshot):
+            raise SQLMeshError(
+                f"A choice can't be changed for the existing version of '{snapshot.name}'."
+            )
+        if (
+            not self._context_diff.directly_modified(snapshot.name)
+            and snapshot.snapshot_id not in self._context_diff.added
+        ):
+            raise SQLMeshError(
+                f"Only directly modified models can be categorized ({snapshot.name})."
+            )
+
+        self._choices[snapshot.snapshot_id] = choice
+        self._latest_plan = None
         return self
 
     def apply(self) -> None:
@@ -207,37 +234,25 @@ class PlanBuilder:
         directly_modified, indirectly_modified = self._build_directly_and_indirectly_modified(dag)
 
         self._check_destructive_changes(directly_modified)
-        self._categorize_snapshots(dag, directly_modified, indirectly_modified)
+        self._categorize_snapshots(dag, indirectly_modified)
         self._adjust_new_snapshot_intervals()
 
         deployability_index = (
-            DeployabilityIndex.create(self._context_diff.snapshots.values())
+            DeployabilityIndex.create(self._context_diff.snapshots.values(), start=self._start)
             if self._is_dev
             else DeployabilityIndex.all_deployable()
         )
 
-        filtered_dag, ignored = self._build_filtered_dag(dag, deployability_index)
-
-        # Exclude ignored snapshots from the modified sets.
-        directly_modified = {s_id for s_id in directly_modified if s_id not in ignored}
-        for s_id in list(indirectly_modified):
-            if s_id in ignored:
-                indirectly_modified.pop(s_id, None)
-            else:
-                indirectly_modified[s_id] = {
-                    s_id for s_id in indirectly_modified[s_id] if s_id not in ignored
-                }
-
-        filtered_snapshots = {
-            s.snapshot_id: s
-            for s in self._context_diff.snapshots.values()
-            if s.snapshot_id not in ignored
-        }
-
-        models_to_backfill = self._build_models_to_backfill(filtered_dag)
         restatements = self._build_restatements(
-            dag, earliest_interval_start(filtered_snapshots.values())
+            dag, earliest_interval_start(self._context_diff.snapshots.values())
         )
+        models_to_backfill = self._build_models_to_backfill(dag, restatements)
+
+        interval_end_per_model = self._interval_end_per_model
+        if interval_end_per_model and self.override_end:
+            # If the end date was provided explicitly by a user, then interval end for each individual
+            # model should be ignored.
+            interval_end_per_model = None
 
         plan = Plan(
             context_diff=self._context_diff,
@@ -246,6 +261,7 @@ class PlanBuilder:
             provided_end=self._end,
             is_dev=self._is_dev,
             skip_backfill=self._skip_backfill,
+            empty_backfill=self._empty_backfill,
             no_gaps=self._no_gaps,
             forward_only=self._forward_only,
             allow_destructive_models=t.cast(t.Set, self._allow_destructive_models),
@@ -254,9 +270,10 @@ class PlanBuilder:
             environment_naming_info=self.environment_naming_info,
             directly_modified=directly_modified,
             indirectly_modified=indirectly_modified,
-            ignored=ignored,
             deployability_index=deployability_index,
             restatements=restatements,
+            interval_end_per_model=interval_end_per_model,
+            selected_models_to_backfill=self._backfill_models,
             models_to_backfill=models_to_backfill,
             effective_from=self._effective_from,
             execution_time=self._execution_time,
@@ -272,38 +289,9 @@ class PlanBuilder:
             dag.add(s_id, context_snapshot.parents)
         return dag
 
-    def _build_filtered_dag(
-        self, full_dag: DAG[SnapshotId], deployability_index: DeployabilityIndex
-    ) -> t.Tuple[DAG[SnapshotId], t.Set[SnapshotId]]:
-        ignored_snapshot_ids: t.Set[SnapshotId] = set()
-        filtered_dag: DAG[SnapshotId] = DAG()
-        cache: t.Optional[t.Dict[str, datetime]] = {}
-        for s_id in full_dag:
-            snapshot = self._context_diff.snapshots.get(s_id)
-            # If the snapshot doesn't exist then it must be an external model
-            if not snapshot:
-                continue
-
-            is_deployable = deployability_index.is_deployable(s_id)
-            is_valid_start = snapshot.is_valid_start(
-                self._start, start_date(snapshot, self._context_diff.snapshots.values(), cache)
-            )
-            if not is_deployable or (
-                is_valid_start and set(snapshot.parents).isdisjoint(ignored_snapshot_ids)
-            ):
-                filtered_dag.add(s_id, snapshot.parents)
-            else:
-                ignored_snapshot_ids.add(s_id)
-        return filtered_dag, ignored_snapshot_ids
-
     def _build_restatements(
         self, dag: DAG[SnapshotId], earliest_interval_start: TimeLike
     ) -> t.Dict[SnapshotId, Interval]:
-        def is_restateable_snapshot(snapshot: Snapshot) -> bool:
-            if not self._is_dev and snapshot.disable_restatement:
-                return False
-            return not snapshot.is_symbolic and not snapshot.is_seed
-
         restate_models = self._restate_models
         if restate_models == set():
             # This is a warning but we print this as error since the Console is lacking API for warnings.
@@ -321,56 +309,87 @@ class PlanBuilder:
             restate_models = {
                 s.name
                 for s in self._context_diff.new_snapshots.values()
-                if s.is_materialized and (self._forward_only or s.model.forward_only)
+                if s.is_materialized
+                and (self._forward_only or s.model.forward_only)
+                and (
+                    # Metadata changes should not be previewed.
+                    self._context_diff.directly_modified(s.name)
+                    or self._context_diff.indirectly_modified(s.name)
+                )
             }
             is_preview = True
 
         if not restate_models:
             return {}
 
+        start = self._start or earliest_interval_start
+        end = self._end or now()
+
         # Add restate snapshots and their downstream snapshots
-        dummy_interval = (sys.maxsize, -sys.maxsize)
         for model_fqn in restate_models:
-            snapshot = self._model_fqn_to_snapshot.get(model_fqn)
-            if not snapshot:
+            if model_fqn not in self._model_fqn_to_snapshot:
                 raise PlanError(f"Cannot restate model '{model_fqn}'. Model does not exist.")
+
+        # Get restatement intervals for all restated snapshots and make sure that if an incremental snapshot expands it's
+        # restatement range that it's downstream dependencies all expand their restatement ranges as well.
+        for s_id in dag:
+            snapshot = self._context_diff.snapshots[s_id]
+
             if not forward_only_preview_needed:
-                if not self._is_dev and snapshot.disable_restatement:
-                    # This is a warning but we print this as error since the Console is lacking API for warnings.
-                    self._console.log_error(
-                        f"Cannot restate model '{model_fqn}'. Restatement is disabled for this model."
+                if self._is_dev and not snapshot.is_paused:
+                    self._console.log_warning(
+                        f"Cannot restate model '{snapshot.name}' because the current version is used in production. "
+                        "Run the restatement against the production environment instead to restate this model."
+                    )
+                    continue
+                elif (not self._is_dev or not snapshot.is_paused) and snapshot.disable_restatement:
+                    self._console.log_warning(
+                        f"Cannot restate model '{snapshot.name}'. "
+                        "Restatement is disabled for this model to prevent possible data loss."
+                        "If you want to restate this model, change the model's `disable_restatement` setting to `false`."
                     )
                     continue
                 elif snapshot.is_symbolic or snapshot.is_seed:
-                    logger.info("Skipping restatement for model '%s'", model_fqn)
+                    logger.info("Skipping restatement for model '%s'", snapshot.name)
                     continue
 
-            restatements[snapshot.snapshot_id] = dummy_interval
-            for downstream_s_id in dag.downstream(snapshot.snapshot_id):
-                if is_restateable_snapshot(self._context_diff.snapshots[downstream_s_id]):
-                    restatements[downstream_s_id] = dummy_interval
-        # Get restatement intervals for all restated snapshots and make sure that if a snapshot expands it's
-        # restatement range that it's downstream dependencies all expand their restatement ranges as well.
-        for s_id in dag:
-            if s_id not in restatements:
-                continue
-            snapshot = self._context_diff.snapshots[s_id]
-            interval = snapshot.get_removal_interval(
-                self._start or earliest_interval_start,
-                self._end or now(),
-                self._execution_time,
-                strict=False,
-                is_preview=is_preview,
-            )
             # Since we are traversing the graph in topological order and the largest interval range is pushed down
             # the graph we just have to check our immediate parents in the graph and not the whole upstream graph.
-            snapshot_dependencies = snapshot.parents
-            possible_intervals = [
-                restatements.get(s, dummy_interval) for s in snapshot_dependencies
-            ] + [interval]
+            restating_parents = [
+                self._context_diff.snapshots[s] for s in snapshot.parents if s in restatements
+            ]
+
+            if not restating_parents and snapshot.name not in restate_models:
+                continue
+
+            possible_intervals = {
+                restatements[p.snapshot_id] for p in restating_parents if p.is_incremental
+            }
+            possible_intervals.add(
+                snapshot.get_removal_interval(
+                    start,
+                    end,
+                    self._execution_time,
+                    strict=False,
+                    is_preview=is_preview,
+                )
+            )
             snapshot_start = min(i[0] for i in possible_intervals)
             snapshot_end = max(i[1] for i in possible_intervals)
+
+            # We may be tasked with restating a time range smaller than the target snapshot interval unit
+            # For example, restating an hour of Hourly Model A, which has a downstream dependency of Daily Model B
+            # we need to ensure the whole affected day in Model B is restated
+            floored_snapshot_start = snapshot.node.interval_unit.cron_floor(snapshot_start)
+            floored_snapshot_end = snapshot.node.interval_unit.cron_floor(snapshot_end)
+            if to_timestamp(floored_snapshot_end) < snapshot_end:
+                snapshot_start = to_timestamp(floored_snapshot_start)
+                snapshot_end = to_timestamp(
+                    snapshot.node.interval_unit.cron_next(floored_snapshot_end)
+                )
+
             restatements[s_id] = (snapshot_start, snapshot_end)
+
         return restatements
 
     def _build_directly_and_indirectly_modified(
@@ -405,19 +424,25 @@ class PlanBuilder:
             indirectly_modified,
         )
 
-    def _build_models_to_backfill(self, dag: DAG[SnapshotId]) -> t.Optional[t.Set[str]]:
-        if self._backfill_models is None:
+    def _build_models_to_backfill(
+        self, dag: DAG[SnapshotId], restatements: t.Collection[SnapshotId]
+    ) -> t.Optional[t.Set[str]]:
+        backfill_models = (
+            self._backfill_models
+            if self._backfill_models is not None
+            else [r.name for r in restatements]
+            # Only backfill models explicitly marked for restatement.
+            if self._restate_models
+            else None
+        )
+        if backfill_models is None:
             return None
-        if not self._is_dev:
-            raise PlanError(
-                "Selecting models to backfill is only supported for development environments."
-            )
         return {
             self._context_diff.snapshots[s_id].name
             for s_id in dag.subdag(
                 *[
                     self._model_fqn_to_snapshot[m].snapshot_id
-                    for m in self._backfill_models
+                    for m in backfill_models
                     if m in self._model_fqn_to_snapshot
                 ]
             ).sorted
@@ -466,19 +491,24 @@ class PlanBuilder:
                 )
 
                 if has_drop_alteration(schema_diff):
-                    warning_msg = f"Plan results in a destructive change to forward-only model '{snapshot.name}'s schema"
-                    if snapshot.model.on_destructive_change.is_warn:
-                        logger.warning(warning_msg)
-                    else:
+                    snapshot_name = snapshot.name
+                    dropped_column_names = get_dropped_column_names(schema_diff)
+                    model_dialect = snapshot.model.dialect
+
+                    get_console().log_destructive_change(
+                        snapshot_name,
+                        dropped_column_names,
+                        schema_diff,
+                        model_dialect,
+                        error=not snapshot.model.on_destructive_change.is_warn,
+                    )
+                    if snapshot.model.on_destructive_change.is_error:
                         raise PlanError(
-                            f"{warning_msg}. To allow this, change the model's `on_destructive_change` setting to `warn` or `allow` or include it in the plan's `--allow-destructive-model` option."
+                            "Plan requires a destructive change to a forward-only model."
                         )
 
     def _categorize_snapshots(
-        self,
-        dag: DAG[SnapshotId],
-        directly_modified: t.Set[SnapshotId],
-        indirectly_modified: SnapshotMapping,
+        self, dag: DAG[SnapshotId], indirectly_modified: SnapshotMapping
     ) -> None:
         """Automatically categorizes snapshots that can be automatically categorized and
         returns a list of added and directly modified snapshots as well as the mapping of
@@ -489,152 +519,86 @@ class PlanBuilder:
         # assigned to its upstream dependencies.
         for s_id in dag:
             snapshot = self._context_diff.snapshots.get(s_id)
-            if not snapshot or snapshot.change_category:
+
+            if not snapshot or not self._is_new_snapshot(snapshot):
                 continue
 
-            if s_id.name in self._context_diff.modified_snapshots:
-                is_directly_modified = self._context_diff.directly_modified(s_id.name)
-                if self._is_new_snapshot(snapshot):
-                    if self._forward_only:
-                        # In case of the forward only plan any modifications result in reuse of the
-                        # previous version for non-seed models.
-                        # New snapshots of seed models are considered non-breaking ones.
-                        if not snapshot.is_seed:
-                            snapshot.categorize_as(SnapshotChangeCategory.FORWARD_ONLY)
-                        else:
-                            snapshot.categorize_as(SnapshotChangeCategory.NON_BREAKING)
-                    elif self._is_forward_only_change(s_id) and is_directly_modified:
-                        self._set_choice(
-                            snapshot,
-                            SnapshotChangeCategory.FORWARD_ONLY,
-                            directly_modified,
-                            indirectly_modified,
-                        )
-                    elif self._auto_categorization_enabled and is_directly_modified:
-                        s_id_with_missing_columns: t.Optional[SnapshotId] = None
-                        this_sid_with_downstream = indirectly_modified.get(s_id, set()) | {s_id}
-                        for downstream_s_id in this_sid_with_downstream:
-                            downstream_snapshot = self._context_diff.snapshots[downstream_s_id]
-                            if (
-                                downstream_snapshot.is_model
-                                and downstream_snapshot.model.columns_to_types is None
-                            ):
-                                s_id_with_missing_columns = downstream_s_id
-                                break
+            if s_id in self._choices:
+                snapshot.categorize_as(self._choices[s_id])
+                continue
 
-                        new, old = self._context_diff.modified_snapshots[s_id.name]
-                        if s_id_with_missing_columns is None:
-                            change_category = categorize_change(
-                                new, old, config=self._categorizer_config
-                            )
-                            if change_category is not None:
-                                self._set_choice(
-                                    new, change_category, directly_modified, indirectly_modified
-                                )
-                        else:
-                            mode = self._categorizer_config.dict().get(
-                                new.model.source_type, AutoCategorizationMode.OFF
-                            )
-                            if mode == AutoCategorizationMode.FULL:
-                                self._set_choice(
-                                    new,
-                                    SnapshotChangeCategory.BREAKING,
-                                    directly_modified,
-                                    indirectly_modified,
-                                )
-
-                if (
-                    not is_directly_modified
-                    and not snapshot.version
-                    and not any(
-                        self._context_diff.directly_modified(upstream.name)
-                        and not self._context_diff.snapshots[upstream].version
-                        for upstream in dag.upstream(s_id)
-                    )
-                ):
-                    if self._context_diff.indirectly_modified(snapshot.name):
-                        # Set to breaking if an indirect child has no directly modified parents
-                        # that need a decision. this can happen when a revert to a parent causes
-                        # an indirectly modified snapshot to be created because of a new parent
-                        snapshot.categorize_as(
-                            SnapshotChangeCategory.FORWARD_ONLY
-                            if self._is_forward_only_change(s_id)
-                            else SnapshotChangeCategory.INDIRECT_BREAKING
-                        )
-                    else:
-                        # Metadata updated.
-                        snapshot.categorize_as(SnapshotChangeCategory.FORWARD_ONLY)
-
-            elif s_id in self._context_diff.added and self._is_new_snapshot(snapshot):
-                snapshot.categorize_as(
-                    SnapshotChangeCategory.FORWARD_ONLY
-                    if self._is_forward_only_change(s_id)
-                    else SnapshotChangeCategory.BREAKING
+            if s_id in self._context_diff.added:
+                snapshot.categorize_as(SnapshotChangeCategory.BREAKING)
+            elif self._is_forward_only_change(s_id) or self._forward_only:
+                # In case of the forward only plan any modifications result in reuse of the
+                # previous version for non-seed models.
+                # New snapshots of seed models are considered non-breaking ones.
+                category = (
+                    SnapshotChangeCategory.NON_BREAKING
+                    if snapshot.is_seed
+                    else SnapshotChangeCategory.FORWARD_ONLY
                 )
+                # If the model kind changes mark as breaking
+                if snapshot.is_model and snapshot.name in self._context_diff.modified_snapshots:
+                    _, old = self._context_diff.modified_snapshots[snapshot.name]
+                    if old.model.kind.name != snapshot.model.kind.name:
+                        category = SnapshotChangeCategory.BREAKING
 
-    def _set_choice(
-        self,
-        snapshot: Snapshot,
-        choice: SnapshotChangeCategory,
-        directly_modified: t.Set[SnapshotId],
-        indirectly_modified: SnapshotMapping,
+                snapshot.categorize_as(category)
+            elif s_id.name in self._context_diff.modified_snapshots:
+                self._categorize_snapshot(snapshot, dag, indirectly_modified)
+
+    def _categorize_snapshot(
+        self, snapshot: Snapshot, dag: DAG[SnapshotId], indirectly_modified: SnapshotMapping
     ) -> None:
-        if self._forward_only:
-            raise PlanError("Choice setting is not supported by a forward-only plan.")
-        if not self._is_new_snapshot(snapshot):
-            raise SQLMeshError(
-                f"A choice can't be changed for the existing version of '{snapshot.name}'."
-            )
+        s_id = snapshot.snapshot_id
 
-        snapshot.categorize_as(choice)
+        if self._context_diff.directly_modified(s_id.name):
+            if self._auto_categorization_enabled:
+                s_id_with_missing_columns: t.Optional[SnapshotId] = None
+                this_sid_with_downstream = indirectly_modified.get(s_id, set()) | {s_id}
+                for downstream_s_id in this_sid_with_downstream:
+                    downstream_snapshot = self._context_diff.snapshots[downstream_s_id]
+                    if (
+                        downstream_snapshot.is_model
+                        and downstream_snapshot.model.columns_to_types is None
+                    ):
+                        s_id_with_missing_columns = downstream_s_id
+                        break
 
-        is_breaking_choice = choice in (
-            SnapshotChangeCategory.BREAKING,
-            SnapshotChangeCategory.INDIRECT_BREAKING,
-        )
-
-        for child_s_id in indirectly_modified.get(snapshot.snapshot_id, set()):
-            child_snapshot = self._context_diff.snapshots[child_s_id]
-            # If the snapshot isn't new then we are reverting to a previously existing snapshot
-            # and therefore we don't want to recategorize it.
-            if not self._is_new_snapshot(child_snapshot):
-                continue
-
-            is_forward_only_child = self._is_forward_only_change(child_s_id)
-
-            if is_breaking_choice:
-                child_snapshot.categorize_as(
-                    SnapshotChangeCategory.FORWARD_ONLY
-                    if is_forward_only_child
-                    else SnapshotChangeCategory.INDIRECT_BREAKING
-                )
-            elif choice == SnapshotChangeCategory.FORWARD_ONLY:
-                child_snapshot.categorize_as(SnapshotChangeCategory.FORWARD_ONLY)
-            else:
-                child_snapshot.categorize_as(SnapshotChangeCategory.INDIRECT_NON_BREAKING)
-
-            for upstream_id in directly_modified:
-                if upstream_id == snapshot.snapshot_id or child_s_id not in indirectly_modified.get(
-                    upstream_id, set()
-                ):
-                    continue
-
-                upstream = self._context_diff.snapshots[upstream_id]
-                if upstream.change_category == SnapshotChangeCategory.BREAKING:
-                    # If any other snapshot specified breaking this child, then that child
-                    # needs to be backfilled as a part of the plan.
-                    child_snapshot.categorize_as(
-                        SnapshotChangeCategory.FORWARD_ONLY
-                        if is_forward_only_child
-                        else SnapshotChangeCategory.INDIRECT_BREAKING
+                new, old = self._context_diff.modified_snapshots[s_id.name]
+                if s_id_with_missing_columns is None:
+                    change_category = categorize_change(new, old, config=self._categorizer_config)
+                    if change_category is not None:
+                        snapshot.categorize_as(change_category)
+                else:
+                    mode = self._categorizer_config.dict().get(
+                        new.model.source_type, AutoCategorizationMode.OFF
                     )
-                    break
-                elif (
-                    upstream.change_category == SnapshotChangeCategory.FORWARD_ONLY
-                    and child_snapshot.is_indirect_non_breaking
-                ):
-                    # FORWARD_ONLY takes precedence over INDIRECT_NON_BREAKING.
-                    child_snapshot.categorize_as(SnapshotChangeCategory.FORWARD_ONLY)
+                    if mode == AutoCategorizationMode.FULL:
+                        snapshot.categorize_as(SnapshotChangeCategory.BREAKING)
+        elif self._context_diff.indirectly_modified(snapshot.name):
+            categories = []
+
+            for p_id in dag.upstream(s_id):
+                parent = self._context_diff.snapshots.get(p_id)
+
+                if parent and self._is_new_snapshot(parent):
+                    categories.append(parent.change_category)
+
+            if not categories or any(
+                category.is_breaking or category.is_indirect_breaking
+                for category in categories
+                if category
+            ):
+                snapshot.categorize_as(SnapshotChangeCategory.INDIRECT_BREAKING)
+            elif any(category.is_forward_only for category in categories if category):
+                snapshot.categorize_as(SnapshotChangeCategory.FORWARD_ONLY)
+            else:
+                snapshot.categorize_as(SnapshotChangeCategory.INDIRECT_NON_BREAKING)
+        else:
+            # Metadata updated.
+            snapshot.categorize_as(SnapshotChangeCategory.METADATA)
 
     def _apply_effective_from(self) -> None:
         if self._effective_from:
@@ -644,7 +608,11 @@ class PlanBuilder:
                 raise PlanError("Effective date cannot be in the future.")
 
         for snapshot in self._context_diff.new_snapshots.values():
-            if not snapshot.disable_restatement:
+            if (
+                snapshot.evaluatable
+                and not snapshot.disable_restatement
+                and (not snapshot.full_history_restatement_only or not snapshot.is_incremental)
+            ):
                 snapshot.effective_from = self._effective_from
 
     def _is_forward_only_change(self, s_id: SnapshotId) -> bool:
@@ -681,10 +649,11 @@ class PlanBuilder:
         for name, (candidate, promoted) in self._context_diff.modified_snapshots.items():
             if (
                 candidate.snapshot_id not in self._context_diff.new_snapshots
+                and candidate.is_model
+                and not candidate.model.forward_only
                 and promoted.is_forward_only
                 and not promoted.is_paused
-                and not candidate.is_forward_only
-                and not candidate.is_indirect_non_breaking
+                and not candidate.reuses_previous_version
                 and promoted.version == candidate.version
             ):
                 raise PlanError(
@@ -693,9 +662,9 @@ class PlanBuilder:
 
     def _ensure_no_broken_references(self) -> None:
         for snapshot in self._context_diff.snapshots.values():
-            broken_references = {x.name for x in self._context_diff.removed_snapshots} & {
-                x for x in snapshot.node.depends_on
-            }
+            broken_references = {
+                x.name for x in self._context_diff.removed_snapshots.values() if not x.is_external
+            } & {x for x in snapshot.node.depends_on}
             if broken_references:
                 broken_references_msg = ", ".join(f"'{x}'" for x in broken_references)
                 raise PlanError(
@@ -717,9 +686,10 @@ class PlanBuilder:
             and not self._include_unmodified
             and self._context_diff.is_new_environment
             and not self._context_diff.has_snapshot_changes
+            and not self._backfill_models
         ):
             raise NoChangesPlanError(
-                "No changes were detected. Make a change or run with --include-unmodified to create a new environment without changes."
+                f"Creating a new environment requires a change, but project files match the `{self._context_diff.create_from}` environment. Make a change or use the --include-unmodified flag to create a new environment without changes."
             )
 
     @cached_property
@@ -731,8 +701,25 @@ class PlanBuilder:
                 self._enable_preview
                 and any(
                     snapshot.model.forward_only
-                    for snapshot in self._context_diff.new_snapshots.values()
+                    for snapshot in self._modified_and_added_snapshots
                     if snapshot.is_model
                 )
             )
         )
+
+    @cached_property
+    def _auto_restatement_preview_needed(self) -> bool:
+        return self._is_dev and any(
+            snapshot.model.auto_restatement_cron is not None
+            for snapshot in self._modified_and_added_snapshots
+            if snapshot.is_model
+        )
+
+    @cached_property
+    def _modified_and_added_snapshots(self) -> t.List[Snapshot]:
+        return [
+            snapshot
+            for snapshot in self._context_diff.snapshots.values()
+            if snapshot.name in self._context_diff.modified_snapshots
+            or snapshot.snapshot_id in self._context_diff.added
+        ]
