@@ -17,7 +17,12 @@ from sqlmesh.core.snapshot import (
     merge_intervals,
     missing_intervals,
 )
-from sqlmesh.core.snapshot.definition import Interval, SnapshotId, format_intervals
+from sqlmesh.core.snapshot.definition import (
+    Interval,
+    SnapshotId,
+    SnapshotTableInfo,
+    format_intervals,
+)
 from sqlmesh.utils.date import TimeLike, now, to_datetime, to_timestamp
 from sqlmesh.utils.pydantic import PydanticModel
 
@@ -32,8 +37,10 @@ class Plan(PydanticModel, frozen=True):
 
     is_dev: bool
     skip_backfill: bool
+    empty_backfill: bool
     no_gaps: bool
     forward_only: bool
+    allow_destructive_models: t.Set[str]
     include_unmodified: bool
     end_bounded: bool
     ensure_finalized_snapshots: bool
@@ -43,12 +50,15 @@ class Plan(PydanticModel, frozen=True):
 
     directly_modified: t.Set[SnapshotId]
     indirectly_modified: t.Dict[SnapshotId, t.Set[SnapshotId]]
-    ignored: t.Set[SnapshotId]
 
     deployability_index: DeployabilityIndex
     restatements: t.Dict[SnapshotId, Interval]
+    interval_end_per_model: t.Optional[t.Dict[str, int]]
 
+    selected_models_to_backfill: t.Optional[t.Set[str]] = None
+    """Models that have been explicitly selected for backfill by a user."""
     models_to_backfill: t.Optional[t.Set[str]] = None
+    """All models that should be backfilled as part of this plan."""
     effective_from: t.Optional[TimeLike] = None
     execution_time: t.Optional[TimeLike] = None
 
@@ -73,20 +83,15 @@ class Plan(PydanticModel, frozen=True):
 
     @property
     def requires_backfill(self) -> bool:
-        return not self.skip_backfill and (bool(self.restatements) or bool(self.missing_intervals))
+        return (
+            not self.skip_backfill
+            and not self.empty_backfill
+            and (bool(self.restatements) or bool(self.missing_intervals))
+        )
 
     @property
     def has_changes(self) -> bool:
-        modified_snapshot_ids = {
-            *self.context_diff.added,
-            *self.context_diff.removed_snapshots,
-            *self.context_diff.current_modified_snapshot_ids,
-        } - self.ignored
-        return (
-            self.context_diff.is_new_environment
-            or self.context_diff.is_unfinalized_environment
-            or bool(modified_snapshot_ids)
-        )
+        return self.context_diff.has_changes
 
     @property
     def has_unmodified_unpromoted(self) -> bool:
@@ -103,7 +108,7 @@ class Plan(PydanticModel, frozen=True):
         """Returns the already categorized snapshots."""
         return [
             self.context_diff.snapshots[s_id]
-            for s_id in sorted(self.directly_modified)
+            for s_id in sorted({*self.directly_modified, *self.metadata_updated})
             if self.context_diff.snapshots[s_id].version
         ]
 
@@ -116,14 +121,12 @@ class Plan(PydanticModel, frozen=True):
             if not self.context_diff.snapshots[s_id].version
         ]
 
-    @cached_property
+    @property
     def snapshots(self) -> t.Dict[SnapshotId, Snapshot]:
-        return {
-            s_id: s for s_id, s in self.context_diff.snapshots.items() if s_id not in self.ignored
-        }
+        return self.context_diff.snapshots
 
     @cached_property
-    def modified_snapshots(self) -> t.Dict[SnapshotId, Snapshot]:
+    def modified_snapshots(self) -> t.Dict[SnapshotId, t.Union[Snapshot, SnapshotTableInfo]]:
         """Returns the modified (either directly or indirectly) snapshots."""
         return {
             **{s_id: self.context_diff.snapshots[s_id] for s_id in sorted(self.directly_modified)},
@@ -132,14 +135,22 @@ class Plan(PydanticModel, frozen=True):
                 for downstream_s_ids in self.indirectly_modified.values()
                 for s_id in sorted(downstream_s_ids)
             },
+            **self.context_diff.removed_snapshots,
+            **{s_id: self.context_diff.snapshots[s_id] for s_id in sorted(self.metadata_updated)},
+        }
+
+    @cached_property
+    def metadata_updated(self) -> t.Set[SnapshotId]:
+        return {
+            snapshot.snapshot_id
+            for snapshot, _ in self.context_diff.modified_snapshots.values()
+            if self.context_diff.metadata_updated(snapshot.name)
         }
 
     @property
     def new_snapshots(self) -> t.List[Snapshot]:
         """Gets only new snapshots in the plan/environment."""
-        return [
-            s for s in self.context_diff.new_snapshots.values() if s.snapshot_id not in self.ignored
-        ]
+        return list(self.context_diff.new_snapshots.values())
 
     @property
     def missing_intervals(self) -> t.List[SnapshotIntervals]:
@@ -155,6 +166,7 @@ class Plan(PydanticModel, frozen=True):
                 execution_time=self.execution_time,
                 restatements=self.restatements,
                 deployability_index=self.deployability_index,
+                interval_end_per_model=self.interval_end_per_model,
                 end_bounded=self.end_bounded,
             ).items()
             if snapshot.is_model and missing
@@ -174,17 +186,19 @@ class Plan(PydanticModel, frozen=True):
         snapshots = [s.table_info for s in self.snapshots.values()]
         promoted_snapshot_ids = None
         if self.is_dev and not self.include_unmodified:
-            promotable_snapshot_ids = self.context_diff.promotable_snapshot_ids.copy()
-            if self.models_to_backfill is not None:
+            if self.selected_models_to_backfill is not None:
                 # Only promote models that have been explicitly selected for backfill.
-                promotable_snapshot_ids &= {
+                promotable_snapshot_ids = {
                     *self.context_diff.previously_promoted_snapshot_ids,
                     *[
                         snapshots_by_name[m].snapshot_id
-                        for m in self.models_to_backfill
+                        for m in self.selected_models_to_backfill
                         if m in snapshots_by_name
                     ],
                 }
+            else:
+                promotable_snapshot_ids = self.context_diff.promotable_snapshot_ids.copy()
+
             promoted_snapshot_ids = [
                 s.snapshot_id for s in snapshots if s.snapshot_id in promotable_snapshot_ids
             ]
@@ -204,23 +218,92 @@ class Plan(PydanticModel, frozen=True):
             expiration_ts=expiration_ts,
             promoted_snapshot_ids=promoted_snapshot_ids,
             previous_finalized_snapshots=previous_finalized_snapshots,
+            requirements=self.context_diff.requirements,
             **self.environment_naming_info.dict(),
         )
 
     def is_new_snapshot(self, snapshot: Snapshot) -> bool:
         """Returns True if the given snapshot is a new snapshot in this plan."""
         snapshot_id = snapshot.snapshot_id
-        return snapshot_id in self.context_diff.new_snapshots and snapshot_id not in self.ignored
+        return snapshot_id in self.context_diff.new_snapshots
 
     def is_selected_for_backfill(self, model_fqn: str) -> bool:
         """Returns True if a model with the given FQN should be backfilled as part of this plan."""
         return self.models_to_backfill is None or model_fqn in self.models_to_backfill
 
+    def to_evaluatable(self) -> EvaluatablePlan:
+        return EvaluatablePlan(
+            start=self.start,
+            end=self.end,
+            new_snapshots=self.new_snapshots,
+            environment=self.environment,
+            no_gaps=self.no_gaps,
+            skip_backfill=self.skip_backfill,
+            empty_backfill=self.empty_backfill,
+            restatements={s.name: i for s, i in self.restatements.items()},
+            is_dev=self.is_dev,
+            allow_destructive_models=self.allow_destructive_models,
+            forward_only=self.forward_only,
+            end_bounded=self.end_bounded,
+            ensure_finalized_snapshots=self.ensure_finalized_snapshots,
+            directly_modified_snapshots=sorted(self.directly_modified),
+            indirectly_modified_snapshots={
+                s.name: sorted(snapshot_ids) for s, snapshot_ids in self.indirectly_modified.items()
+            },
+            removed_snapshots=sorted(self.context_diff.removed_snapshots),
+            requires_backfill=self.requires_backfill,
+            models_to_backfill=self.models_to_backfill,
+            interval_end_per_model=self.interval_end_per_model,
+            execution_time=self.execution_time,
+            disabled_restatement_models={
+                s.name
+                for s in self.snapshots.values()
+                if s.is_model and s.model.disable_restatement
+            },
+        )
+
     @cached_property
     def _earliest_interval_start(self) -> datetime:
         return earliest_interval_start(self.snapshots.values())
 
 
+class EvaluatablePlan(PydanticModel):
+    """A serializable version of a plan that can be evaluated."""
+
+    start: TimeLike
+    end: TimeLike
+    new_snapshots: t.List[Snapshot]
+    environment: Environment
+    no_gaps: bool
+    skip_backfill: bool
+    empty_backfill: bool
+    restatements: t.Dict[str, Interval]
+    is_dev: bool
+    allow_destructive_models: t.Set[str]
+    forward_only: bool
+    end_bounded: bool
+    ensure_finalized_snapshots: bool
+    directly_modified_snapshots: t.List[SnapshotId]
+    indirectly_modified_snapshots: t.Dict[str, t.List[SnapshotId]]
+    removed_snapshots: t.List[SnapshotId]
+    requires_backfill: bool
+    models_to_backfill: t.Optional[t.Set[str]] = None
+    interval_end_per_model: t.Optional[t.Dict[str, int]] = None
+    execution_time: t.Optional[TimeLike] = None
+    disabled_restatement_models: t.Set[str]
+
+    def is_selected_for_backfill(self, model_fqn: str) -> bool:
+        return self.models_to_backfill is None or model_fqn in self.models_to_backfill
+
+    @property
+    def plan_id(self) -> str:
+        return self.environment.plan_id
+
+    @property
+    def is_prod(self) -> bool:
+        return not self.is_dev
+
+
 class PlanStatus(str, Enum):
     STARTED = "started"
     FINISHED = "finished"
