@@ -17,23 +17,35 @@ Refer to `sqlmesh.core.plan`.
 import abc
 import logging
 import typing as t
-
+from sqlmesh.core import analytics
+from sqlmesh.core import constants as c
 from sqlmesh.core.console import Console, get_console
+from sqlmesh.core.environment import EnvironmentNamingInfo
 from sqlmesh.core.notification_target import (
     NotificationTarget,
-    NotificationTargetManager,
 )
-from sqlmesh.core.plan.definition import Plan
+from sqlmesh.core.snapshot.definition import Interval, to_view_mapping
+from sqlmesh.core.plan.definition import EvaluatablePlan
 from sqlmesh.core.scheduler import Scheduler
-from sqlmesh.core.snapshot import DeployabilityIndex, Snapshot, SnapshotEvaluator
+from sqlmesh.core.snapshot import (
+    DeployabilityIndex,
+    Snapshot,
+    SnapshotEvaluator,
+    SnapshotIntervals,
+    SnapshotId,
+    SnapshotInfoLike,
+    SnapshotTableInfo,
+)
 from sqlmesh.core.state_sync import StateSync
 from sqlmesh.core.state_sync.base import PromotionResult
 from sqlmesh.core.user import User
 from sqlmesh.schedulers.airflow import common as airflow_common
 from sqlmesh.schedulers.airflow.client import AirflowClient, BaseAirflowClient
 from sqlmesh.schedulers.airflow.mwaa_client import MWAAClient
-from sqlmesh.utils import random_id
-from sqlmesh.utils.errors import SQLMeshError
+from sqlmesh.utils.concurrency import NodeExecutionFailedError
+from sqlmesh.utils.errors import SQLMeshError, PlanError
+from sqlmesh.utils.dag import DAG
+from sqlmesh.utils.date import now
 
 logger = logging.getLogger(__name__)
 
@@ -41,7 +53,7 @@ logger = logging.getLogger(__name__)
 class PlanEvaluator(abc.ABC):
     @abc.abstractmethod
     def evaluate(
-        self, plan: Plan, circuit_breaker: t.Optional[t.Callable[[], bool]] = None
+        self, plan: EvaluatablePlan, circuit_breaker: t.Optional[t.Callable[[], bool]] = None
     ) -> None:
         """Evaluates a plan by pushing snapshots and backfilling data.
 
@@ -60,31 +72,40 @@ class BuiltInPlanEvaluator(PlanEvaluator):
         self,
         state_sync: StateSync,
         snapshot_evaluator: SnapshotEvaluator,
+        create_scheduler: t.Callable[[t.Iterable[Snapshot]], Scheduler],
         default_catalog: t.Optional[str],
-        backfill_concurrent_tasks: int = 1,
         console: t.Optional[Console] = None,
-        notification_target_manager: t.Optional[NotificationTargetManager] = None,
     ):
         self.state_sync = state_sync
         self.snapshot_evaluator = snapshot_evaluator
+        self.create_scheduler = create_scheduler
         self.default_catalog = default_catalog
-        self.backfill_concurrent_tasks = backfill_concurrent_tasks
         self.console = console or get_console()
-        self.notification_target_manager = notification_target_manager
 
     def evaluate(
         self,
-        plan: Plan,
+        plan: EvaluatablePlan,
         circuit_breaker: t.Optional[t.Callable[[], bool]] = None,
     ) -> None:
         self.console.start_plan_evaluation(plan)
+        analytics.collector.on_plan_apply_start(
+            plan=plan,
+            engine_type=self.snapshot_evaluator.adapter.dialect,
+            state_sync_type=self.state_sync.state_type(),
+            scheduler_type=c.BUILTIN,
+        )
 
         try:
-            snapshots = plan.snapshots
-            all_names = {
-                s.name for s in snapshots.values() if plan.is_selected_for_backfill(s.name)
-            }
-            deployability_index_for_evaluation = DeployabilityIndex.create(snapshots)
+            new_snapshots = {s.snapshot_id: s for s in plan.new_snapshots}
+            stored_snapshots = self.state_sync.get_snapshots(plan.environment.snapshots)
+            snapshots = {**new_snapshots, **stored_snapshots}
+            snapshots_by_name = {s.name: s for s in snapshots.values()}
+
+            all_names = {name for name in snapshots_by_name if plan.is_selected_for_backfill(name)}
+
+            deployability_index_for_evaluation = DeployabilityIndex.create(
+                snapshots, start=plan.start
+            )
             deployability_index_for_creation = deployability_index_for_evaluation
             if plan.is_dev:
                 before_promote_snapshots = all_names
@@ -99,33 +120,42 @@ class BuiltInPlanEvaluator(PlanEvaluator):
                 after_promote_snapshots = all_names - before_promote_snapshots
                 deployability_index_for_evaluation = DeployabilityIndex.all_deployable()
 
+            self._push(plan, snapshots, deployability_index_for_creation)
             update_intervals_for_new_snapshots(plan.new_snapshots, self.state_sync)
-
-            self._push(plan, deployability_index_for_creation)
-            self._restate(plan)
+            self._restate(plan, snapshots_by_name)
             self._backfill(
                 plan,
+                snapshots_by_name,
                 before_promote_snapshots,
                 deployability_index_for_evaluation,
                 circuit_breaker=circuit_breaker,
             )
-            promotion_result = self._promote(plan, before_promote_snapshots)
+            promotion_result = self._promote(plan, snapshots, before_promote_snapshots)
             self._backfill(
                 plan,
+                snapshots_by_name,
                 after_promote_snapshots,
                 deployability_index_for_evaluation,
                 circuit_breaker=circuit_breaker,
             )
-            self._update_views(plan, promotion_result, deployability_index_for_evaluation)
+            self._update_views(
+                plan, snapshots, promotion_result, deployability_index_for_evaluation
+            )
 
             if not plan.requires_backfill:
                 self.console.log_success("Virtual Update executed successfully")
+        except Exception as e:
+            analytics.collector.on_plan_apply_end(plan_id=plan.plan_id, error=e)
+            raise
+        else:
+            analytics.collector.on_plan_apply_end(plan_id=plan.plan_id)
         finally:
             self.console.stop_plan_evaluation()
 
     def _backfill(
         self,
-        plan: Plan,
+        plan: EvaluatablePlan,
+        snapshots_by_name: t.Dict[str, Snapshot],
         selected_snapshots: t.Set[str],
         deployability_index: DeployabilityIndex,
         circuit_breaker: t.Optional[t.Callable[[], bool]] = None,
@@ -136,33 +166,52 @@ class BuiltInPlanEvaluator(PlanEvaluator):
             plan: The plan to source snapshots from.
             selected_snapshots: The snapshots to backfill.
         """
+        if plan.empty_backfill:
+            intervals_to_add = []
+            for snapshot in snapshots_by_name.values():
+                intervals = [snapshot.inclusive_exclusive(plan.start, plan.end, strict=False)]
+                is_deployable = deployability_index.is_deployable(snapshot)
+                intervals_to_add.append(
+                    SnapshotIntervals(
+                        name=snapshot.name,
+                        identifier=snapshot.identifier,
+                        version=snapshot.version,
+                        dev_version=snapshot.dev_version,
+                        intervals=intervals if is_deployable else [],
+                        dev_intervals=intervals if not is_deployable else [],
+                    )
+                )
+            self.state_sync.add_snapshots_intervals(intervals_to_add)
+            return
+
         if not plan.requires_backfill or not selected_snapshots:
             return
 
-        snapshots = plan.snapshots
-        scheduler = Scheduler(
-            snapshots.values(),
-            self.snapshot_evaluator,
-            self.state_sync,
-            default_catalog=self.default_catalog,
-            max_workers=self.backfill_concurrent_tasks,
-            console=self.console,
-            notification_target_manager=self.notification_target_manager,
-        )
-        is_run_successful = scheduler.run(
-            plan.environment_naming_info,
+        scheduler = self.create_scheduler(snapshots_by_name.values())
+        completion_status = scheduler.run(
+            plan.environment,
             plan.start,
             plan.end,
-            restatements=plan.restatements,
+            execution_time=plan.execution_time,
+            restatements={
+                snapshots_by_name[name].snapshot_id: interval
+                for name, interval in plan.restatements.items()
+            },
             selected_snapshots=selected_snapshots,
             deployability_index=deployability_index,
             circuit_breaker=circuit_breaker,
             end_bounded=plan.end_bounded,
+            interval_end_per_model=plan.interval_end_per_model,
         )
-        if not is_run_successful:
-            raise SQLMeshError("Plan application failed.")
+        if completion_status.is_failure:
+            raise PlanError("Plan application failed.")
 
-    def _push(self, plan: Plan, deployability_index: t.Optional[DeployabilityIndex] = None) -> None:
+    def _push(
+        self,
+        plan: EvaluatablePlan,
+        snapshots: t.Dict[SnapshotId, Snapshot],
+        deployability_index: t.Optional[DeployabilityIndex] = None,
+    ) -> None:
         """Push the snapshots to the state sync.
 
         As a part of plan pushing, snapshot tables are created.
@@ -171,34 +220,61 @@ class BuiltInPlanEvaluator(PlanEvaluator):
             plan: The plan to source snapshots from.
             deployability_index: Indicates which snapshots are deployable in the context of this creation.
         """
-        snapshots_to_create = [
-            s
-            for s in plan.snapshots.values()
-            if s.is_model and not s.is_symbolic and plan.is_selected_for_backfill(s.name)
-        ]
-        snapshots_to_create_count = len(snapshots_to_create)
+        promoted_snapshot_ids = (
+            set(plan.environment.promoted_snapshot_ids)
+            if plan.environment.promoted_snapshot_ids is not None
+            else None
+        )
 
-        if snapshots_to_create_count > 0:
-            self.console.start_creation_progress(
-                snapshots_to_create_count, plan.environment_naming_info, self.default_catalog
+        def _should_create(s: Snapshot) -> bool:
+            if not s.is_model or s.is_symbolic:
+                return False
+            # Only create tables for snapshots that we're planning to promote or that were selected for backfill
+            return (
+                plan.is_selected_for_backfill(s.name)
+                or promoted_snapshot_ids is None
+                or s.snapshot_id in promoted_snapshot_ids
             )
 
+        snapshots_to_create = [s for s in snapshots.values() if _should_create(s)]
+
         completed = False
+        progress_stopped = False
         try:
             self.snapshot_evaluator.create(
                 snapshots_to_create,
-                plan.snapshots,
+                snapshots,
+                allow_destructive_snapshots=plan.allow_destructive_models,
                 deployability_index=deployability_index,
+                on_start=lambda x: self.console.start_creation_progress(
+                    x, plan.environment, self.default_catalog
+                ),
                 on_complete=self.console.update_creation_progress,
             )
             completed = True
+        except NodeExecutionFailedError as ex:
+            self.console.stop_creation_progress(success=False)
+            progress_stopped = True
+
+            logger.info(str(ex), exc_info=ex)
+            self.console.log_failed_models([ex])
+
+            raise PlanError("Plan application failed.")
         finally:
-            self.console.stop_creation_progress(success=completed)
+            if not progress_stopped:
+                self.console.stop_creation_progress(success=completed)
 
         self.state_sync.push_snapshots(plan.new_snapshots)
 
+        analytics.collector.on_snapshots_created(
+            new_snapshots=plan.new_snapshots, plan_id=plan.plan_id
+        )
+
     def _promote(
-        self, plan: Plan, no_gaps_snapshot_names: t.Optional[t.Set[str]] = None
+        self,
+        plan: EvaluatablePlan,
+        snapshots: t.Dict[SnapshotId, Snapshot],
+        no_gaps_snapshot_names: t.Optional[t.Set[str]] = None,
     ) -> PromotionResult:
         """Promote a plan.
 
@@ -213,10 +289,15 @@ class BuiltInPlanEvaluator(PlanEvaluator):
         )
 
         if not plan.is_dev:
-            self.snapshot_evaluator.migrate(
-                [s for s in plan.snapshots.values() if s.is_paused],
-                plan.snapshots,
-            )
+            try:
+                self.snapshot_evaluator.migrate(
+                    [s for s in snapshots.values() if s.is_paused],
+                    snapshots,
+                    plan.allow_destructive_models,
+                )
+            except NodeExecutionFailedError as ex:
+                raise PlanError(str(ex.__cause__) if ex.__cause__ else str(ex))
+
             if not plan.ensure_finalized_snapshots:
                 # Only unpause at this point if we don't have to use the finalized snapshots
                 # for subsequent plan applications. Otherwise, unpause right before finalizing
@@ -227,7 +308,8 @@ class BuiltInPlanEvaluator(PlanEvaluator):
 
     def _update_views(
         self,
-        plan: Plan,
+        plan: EvaluatablePlan,
+        snapshots: t.Dict[SnapshotId, Snapshot],
         promotion_result: PromotionResult,
         deployability_index: t.Optional[DeployabilityIndex] = None,
     ) -> None:
@@ -254,36 +336,128 @@ class BuiltInPlanEvaluator(PlanEvaluator):
 
         completed = False
         try:
-            self.snapshot_evaluator.promote(
-                [plan.context_diff.snapshots[s.snapshot_id] for s in promotion_result.added],
+            self._promote_snapshots(
+                plan,
+                [snapshots[s.snapshot_id] for s in promotion_result.added],
                 environment.naming_info,
                 deployability_index=deployability_index,
                 on_complete=lambda s: self.console.update_promotion_progress(s, True),
+                snapshots=snapshots,
             )
             if promotion_result.removed_environment_naming_info:
-                self.snapshot_evaluator.demote(
+                self._demote_snapshots(
+                    plan,
                     promotion_result.removed,
                     promotion_result.removed_environment_naming_info,
                     on_complete=lambda s: self.console.update_promotion_progress(s, False),
                 )
+
             self.state_sync.finalize(environment)
             completed = True
         finally:
             self.console.stop_promotion_progress(success=completed)
 
-    def _restate(self, plan: Plan) -> None:
-        if not plan.restatements:
+    def _promote_snapshots(
+        self,
+        plan: EvaluatablePlan,
+        target_snapshots: t.Iterable[Snapshot],
+        environment_naming_info: EnvironmentNamingInfo,
+        snapshots: t.Dict[SnapshotId, Snapshot],
+        deployability_index: t.Optional[DeployabilityIndex] = None,
+        on_complete: t.Optional[t.Callable[[SnapshotInfoLike], None]] = None,
+    ) -> None:
+        self.snapshot_evaluator.promote(
+            target_snapshots,
+            start=plan.start,
+            end=plan.end,
+            execution_time=plan.execution_time or now(),
+            snapshots=snapshots,
+            table_mapping=to_view_mapping(
+                snapshots.values(),
+                environment_naming_info,
+                default_catalog=self.default_catalog,
+                dialect=self.snapshot_evaluator.adapter.dialect,
+            ),
+            environment_naming_info=environment_naming_info,
+            deployability_index=deployability_index,
+            on_complete=on_complete,
+        )
+
+    def _demote_snapshots(
+        self,
+        plan: EvaluatablePlan,
+        target_snapshots: t.Iterable[SnapshotTableInfo],
+        environment_naming_info: EnvironmentNamingInfo,
+        on_complete: t.Optional[t.Callable[[SnapshotInfoLike], None]] = None,
+    ) -> None:
+        self.snapshot_evaluator.demote(
+            target_snapshots, environment_naming_info, on_complete=on_complete
+        )
+
+    def _restate(self, plan: EvaluatablePlan, snapshots_by_name: t.Dict[str, Snapshot]) -> None:
+        if not plan.restatements or plan.is_dev:
             return
 
-        self.state_sync.remove_interval(
-            [
-                (plan.context_diff.snapshots[s_id], interval)
-                for s_id, interval in plan.restatements.items()
-            ],
-            plan.execution_time,
-            remove_shared_versions=not plan.is_dev,
+        snapshot_intervals_to_restate = {
+            (snapshots_by_name[name].table_info, intervals)
+            for name, intervals in plan.restatements.items()
+        }
+
+        # Restating intervals on prod plans should mean that the intervals are cleared across
+        # all environments, not just the version currently in prod
+        # This ensures that work done in dev environments can still be promoted to prod
+        # by forcing dev environments to re-run intervals that changed in prod
+        #
+        # Without this rule, its possible that promoting a dev table to prod will introduce old data to prod
+        snapshot_intervals_to_restate.update(
+            self._restatement_intervals_across_all_environments(
+                plan.restatements, plan.disabled_restatement_models
+            )
         )
 
+        self.state_sync.remove_intervals(
+            snapshot_intervals=list(snapshot_intervals_to_restate),
+            remove_shared_versions=plan.is_prod,
+        )
+
+    def _restatement_intervals_across_all_environments(
+        self, prod_restatements: t.Dict[str, Interval], disable_restatement_models: t.Set[str]
+    ) -> t.Set[t.Tuple[SnapshotTableInfo, Interval]]:
+        """
+        Given a map of snapshot names + intervals to restate in prod:
+         - Look up matching snapshots across all environments (match based on name - regardless of version)
+         - For each match, also match downstream snapshots while filtering out models that have restatement disabled
+         - Return all matches mapped to the intervals of the prod snapshot being restated
+
+        The goal here is to produce a list of intervals to invalidate across all environments so that a cadence
+        run in those environments causes the intervals to be repopulated
+        """
+        if not prod_restatements:
+            return set()
+
+        snapshots_to_restate: t.Set[t.Tuple[SnapshotTableInfo, Interval]] = set()
+
+        for env in self.state_sync.get_environments():
+            keyed_snapshots = {s.name: s.table_info for s in env.snapshots}
+
+            # We dont just restate matching snapshots, we also have to restate anything downstream of them
+            # so that if A gets restated in prod and dev has A <- B <- C, B and C get restated in dev
+            env_dag = DAG({s.name: {p.name for p in s.parents} for s in env.snapshots})
+
+            for restatement, intervals in prod_restatements.items():
+                if restatement not in keyed_snapshots:
+                    continue
+                affected_snapshot_names = [
+                    x
+                    for x in ([restatement] + env_dag.downstream(restatement))
+                    if x not in disable_restatement_models
+                ]
+                snapshots_to_restate.update(
+                    {(keyed_snapshots[a], intervals) for a in affected_snapshot_names}
+                )
+
+        return snapshots_to_restate
+
 
 class BaseAirflowPlanEvaluator(PlanEvaluator):
     def __init__(
@@ -301,14 +475,21 @@ class BaseAirflowPlanEvaluator(PlanEvaluator):
         self.console = console or get_console()
 
     def evaluate(
-        self, plan: Plan, circuit_breaker: t.Optional[t.Callable[[], bool]] = None
+        self, plan: EvaluatablePlan, circuit_breaker: t.Optional[t.Callable[[], bool]] = None
     ) -> None:
-        plan_request_id = random_id()
+        plan_request_id = plan.plan_id
         self._apply_plan(plan, plan_request_id)
 
+        analytics.collector.on_plan_apply_start(
+            plan=plan,
+            engine_type=None,
+            state_sync_type=None,
+            scheduler_type=c.AIRFLOW,
+        )
+
         if self.blocking:
             plan_application_dag_id = airflow_common.plan_application_dag_id(
-                plan.environment_naming_info.name, plan_request_id
+                plan.environment.name, plan_request_id
             )
 
             self.console.log_status_update(
@@ -332,50 +513,35 @@ class BaseAirflowPlanEvaluator(PlanEvaluator):
                 self.dag_run_poll_interval_secs,
             )
             if not plan_application_succeeded:
-                raise SQLMeshError("Plan application failed.")
+                msg = "Plan application failed."
+                logger.info(msg)
+                raise PlanError(msg)
 
-            self.console.log_success("The plan has been applied successfully")
+            self.console.log_success("Plan applied successfully")
 
     @property
     def client(self) -> BaseAirflowClient:
         raise NotImplementedError
 
-    def _apply_plan(self, plan: Plan, plan_request_id: str) -> None:
+    def _apply_plan(self, plan: EvaluatablePlan, plan_request_id: str) -> None:
         raise NotImplementedError
 
 
 class StateBasedAirflowPlanEvaluator(BaseAirflowPlanEvaluator):
-
     backfill_concurrent_tasks: int
     ddl_concurrent_tasks: int
     notification_targets: t.Optional[t.List[NotificationTarget]]
     users: t.Optional[t.List[User]]
 
-    def _apply_plan(self, plan: Plan, plan_request_id: str) -> None:
+    def _apply_plan(self, plan: EvaluatablePlan, plan_request_id: str) -> None:
         from sqlmesh.schedulers.airflow.plan import PlanDagState, create_plan_dag_spec
 
         plan_application_request = airflow_common.PlanApplicationRequest(
-            new_snapshots=plan.new_snapshots,
-            environment=plan.environment,
-            no_gaps=plan.no_gaps,
-            skip_backfill=plan.skip_backfill,
-            request_id=plan_request_id,
-            restatements=plan.restatements or {},
+            plan=plan,
             notification_targets=self.notification_targets or [],
             backfill_concurrent_tasks=self.backfill_concurrent_tasks,
             ddl_concurrent_tasks=self.ddl_concurrent_tasks,
             users=self.users or [],
-            is_dev=plan.is_dev,
-            forward_only=plan.forward_only,
-            models_to_backfill=plan.models_to_backfill,
-            end_bounded=plan.end_bounded,
-            ensure_finalized_snapshots=plan.ensure_finalized_snapshots,
-            directly_modified_snapshots=list(plan.directly_modified),
-            indirectly_modified_snapshots={
-                change_source.name: list(snapshots)
-                for change_source, snapshots in plan.indirectly_modified.items()
-            },
-            removed_snapshots=list(plan.context_diff.removed_snapshots),
         )
         plan_dag_spec = create_plan_dag_spec(plan_application_request, self.state_sync)
         PlanDagState.from_state_sync(self.state_sync).add_dag_spec(plan_dag_spec)
@@ -425,33 +591,17 @@ class AirflowPlanEvaluator(StateBasedAirflowPlanEvaluator):
             raise SQLMeshError("State Sync is not configured")
         return self._state_sync
 
-    def _apply_plan(self, plan: Plan, plan_request_id: str) -> None:
+    def _apply_plan(self, plan: EvaluatablePlan, plan_request_id: str) -> None:
         if self._state_sync is not None:
             super()._apply_plan(plan, plan_request_id)
             return
 
         self._airflow_client.apply_plan(
-            plan.new_snapshots,
-            plan.environment,
-            plan_request_id,
-            no_gaps=plan.no_gaps,
-            skip_backfill=plan.skip_backfill,
-            restatements=plan.restatements,
+            plan,
             notification_targets=self.notification_targets,
             backfill_concurrent_tasks=self.backfill_concurrent_tasks,
             ddl_concurrent_tasks=self.ddl_concurrent_tasks,
             users=self.users,
-            is_dev=plan.is_dev,
-            forward_only=plan.forward_only,
-            models_to_backfill=plan.models_to_backfill,
-            end_bounded=plan.end_bounded,
-            ensure_finalized_snapshots=plan.ensure_finalized_snapshots,
-            directly_modified_snapshots=list(plan.directly_modified),
-            indirectly_modified_snapshots={
-                change_source.name: list(snapshots)
-                for change_source, snapshots in plan.indirectly_modified.items()
-            },
-            removed_snapshots=list(plan.context_diff.removed_snapshots),
         )
 
 
@@ -496,8 +646,19 @@ class MWAAPlanEvaluator(StateBasedAirflowPlanEvaluator):
 def update_intervals_for_new_snapshots(
     snapshots: t.Collection[Snapshot], state_sync: StateSync
 ) -> None:
+    snapshots_intervals: t.List[SnapshotIntervals] = []
     for snapshot in state_sync.refresh_snapshot_intervals(snapshots):
         if snapshot.is_forward_only:
             snapshot.dev_intervals = snapshot.intervals.copy()
-            for start, end in snapshot.dev_intervals:
-                state_sync.add_interval(snapshot, start, end, is_dev=True)
+            snapshots_intervals.append(
+                SnapshotIntervals(
+                    name=snapshot.name,
+                    identifier=snapshot.identifier,
+                    version=snapshot.version,
+                    dev_version=snapshot.dev_version,
+                    dev_intervals=snapshot.dev_intervals,
+                )
+            )
+
+    if snapshots_intervals:
+        state_sync.add_snapshots_intervals(snapshots_intervals)
