@@ -7,6 +7,7 @@ from pathlib import Path
 
 from sqlglot import exp, parse
 from sqlglot.errors import SqlglotError
+from sqlglot.helper import ensure_list
 from sqlglot.optimizer.annotate_types import annotate_types
 from sqlglot.optimizer.qualify import qualify
 from sqlglot.optimizer.simplify import simplify
@@ -14,10 +15,9 @@ from sqlglot.optimizer.simplify import simplify
 from sqlmesh.core import constants as c
 from sqlmesh.core import dialect as d
 from sqlmesh.core.macros import MacroEvaluator, RuntimeStage
-from sqlmesh.utils.date import TimeLike, date_dict, make_inclusive_end, to_datetime
+from sqlmesh.utils.date import TimeLike, date_dict, make_inclusive, to_datetime
 from sqlmesh.utils.errors import (
     ConfigError,
-    MacroEvalError,
     ParsetimeAdapterCallError,
     SQLMeshError,
     raise_config_error,
@@ -27,6 +27,7 @@ from sqlmesh.utils.metaprogramming import Executable, prepare_env
 
 if t.TYPE_CHECKING:
     from sqlglot._typing import E
+    from sqlglot.dialects.dialect import DialectType
 
     from sqlmesh.core.snapshot import DeployabilityIndex, Snapshot
 
@@ -38,7 +39,7 @@ class BaseExpressionRenderer:
     def __init__(
         self,
         expression: exp.Expression,
-        dialect: str,
+        dialect: DialectType,
         macro_definitions: t.List[d.MacroDef],
         path: Path = Path(),
         jinja_macro_registry: t.Optional[JinjaMacroRegistry] = None,
@@ -49,6 +50,8 @@ class BaseExpressionRenderer:
         quote_identifiers: bool = True,
         model_fqn: t.Optional[str] = None,
         normalize_identifiers: bool = True,
+        optimize_query: t.Optional[bool] = True,
+        validate_query: t.Optional[bool] = False,
     ):
         self._expression = expression
         self._dialect = dialect
@@ -63,6 +66,8 @@ class BaseExpressionRenderer:
         self.update_schema({} if schema is None else schema)
         self._cache: t.List[t.Optional[exp.Expression]] = []
         self._model_fqn = model_fqn
+        self._optimize_query_flag = optimize_query is not False
+        self._validate_query = validate_query
 
     def update_schema(self, schema: t.Dict[str, t.Any]) -> None:
         self.schema = d.normalize_mapping_schema(schema, dialect=self._dialect)
@@ -101,38 +106,56 @@ class BaseExpressionRenderer:
         if should_cache and self._cache:
             return self._cache
 
-        if self._model_fqn and "this_model" not in kwargs:
-            kwargs["this_model"] = exp.to_table(
-                self._to_table_mapping(
-                    (
-                        [snapshots[self._model_fqn]]
-                        if snapshots and self._model_fqn in snapshots
-                        else []
-                    ),
-                    deployability_index,
-                ).get(self._model_fqn, self._model_fqn),
-                dialect=self._dialect,
-            ).sql(dialect=self._dialect, identify=True)
+        this_model = kwargs.pop("this_model", None)
+
+        if not this_model and self._model_fqn:
+            this_snapshot = (snapshots or {}).get(self._model_fqn)
+            this_model = self._resolve_table(
+                self._model_fqn,
+                snapshots={self._model_fqn: this_snapshot} if this_snapshot else None,
+                deployability_index=deployability_index,
+                table_mapping=table_mapping,
+            )
 
         expressions = [self._expression]
 
+        start_time, end_time = (
+            make_inclusive(start or c.EPOCH, end or c.EPOCH, self._dialect)
+            if not self._only_execution_time
+            else (None, None)
+        )
+
         render_kwargs = {
             **date_dict(
                 to_datetime(execution_time or c.EPOCH),
-                to_datetime(start or c.EPOCH) if not self._only_execution_time else None,
-                make_inclusive_end(end or c.EPOCH) if not self._only_execution_time else None,
+                start_time,
+                end_time,
             ),
             **kwargs,
         }
 
-        jinja_env = self._jinja_macro_registry.build_environment(
-            **{**render_kwargs, **prepare_env(self._python_env)},
-            snapshots=(snapshots or {}),
-            table_mapping=table_mapping,
-            deployability_index=deployability_index,
-            default_catalog=self._default_catalog,
-            runtime_stage=runtime_stage.value,
-        )
+        variables = kwargs.pop("variables", {})
+        jinja_env_kwargs = {
+            **{**render_kwargs, **prepare_env(self._python_env), **variables},
+            "snapshots": snapshots or {},
+            "table_mapping": table_mapping,
+            "deployability_index": deployability_index,
+            "default_catalog": self._default_catalog,
+            "runtime_stage": runtime_stage.value,
+            "resolve_table": lambda table: self._resolve_table(
+                d.normalize_model_name(table, self._default_catalog, self._dialect),
+                snapshots=snapshots,
+                table_mapping=table_mapping,
+                deployability_index=deployability_index,
+            ).sql(dialect=self._dialect, identify=True, comments=False),
+        }
+        if this_model:
+            render_kwargs["this_model"] = this_model
+            jinja_env_kwargs["this_model"] = this_model.sql(
+                dialect=self._dialect, identify=True, comments=False
+            )
+
+        jinja_env = self._jinja_macro_registry.build_environment(**jinja_env_kwargs)
 
         if isinstance(self._expression, d.Jinja):
             try:
@@ -156,6 +179,7 @@ class BaseExpressionRenderer:
             jinja_env=jinja_env,
             schema=self.schema,
             runtime_stage=runtime_stage,
+            resolve_table=jinja_env.globals["resolve_table"],  # type: ignore
             resolve_tables=lambda e: self._resolve_tables(
                 e,
                 snapshots=snapshots,
@@ -174,20 +198,26 @@ class BaseExpressionRenderer:
         for definition in self._macro_definitions:
             try:
                 macro_evaluator.evaluate(definition)
-            except MacroEvalError as ex:
+            except Exception as ex:
                 raise_config_error(f"Failed to evaluate macro '{definition}'. {ex}", self._path)
 
         macro_evaluator.locals.update(render_kwargs)
 
+        if variables:
+            macro_evaluator.locals.setdefault(c.SQLMESH_VARS, {}).update(variables)
+
         resolved_expressions: t.List[t.Optional[exp.Expression]] = []
 
         for expression in expressions:
             try:
-                expression = macro_evaluator.transform(expression)  # type: ignore
-            except MacroEvalError as ex:
-                raise_config_error(f"Failed to resolve macro for expression. {ex}", self._path)
+                transformed_expressions = ensure_list(macro_evaluator.transform(expression))
+            except Exception as ex:
+                raise_config_error(
+                    f"Failed to resolve macros for\n{expression.sql(dialect=self._dialect, pretty=True)}\n{ex}",
+                    self._path,
+                )
 
-            if expression:
+            for expression in t.cast(t.List[exp.Expression], transformed_expressions):
                 with self._normalize_and_quote(expression) as expression:
                     if hasattr(expression, "selects"):
                         for select in expression.selects:
@@ -216,6 +246,28 @@ class BaseExpressionRenderer:
     def update_cache(self, expression: t.Optional[exp.Expression]) -> None:
         self._cache = [expression]
 
+    def _resolve_table(
+        self,
+        table_name: str | exp.Expression,
+        snapshots: t.Optional[t.Dict[str, Snapshot]] = None,
+        table_mapping: t.Optional[t.Dict[str, str]] = None,
+        deployability_index: t.Optional[DeployabilityIndex] = None,
+    ) -> exp.Table:
+        table = exp.replace_tables(
+            exp.maybe_parse(table_name, into=exp.Table, dialect=self._dialect),
+            {
+                **self._to_table_mapping((snapshots or {}).values(), deployability_index),
+                **(table_mapping or {}),
+            },
+            dialect=self._dialect,
+            copy=False,
+        )
+        # We quote the table here to mimic the behavior of _resolve_tables, otherwise we may end
+        # up normalizing twice, because _to_table_mapping returns the mapped names unquoted.
+        return (
+            d.quote_identifiers(table, dialect=self._dialect) if self._quote_identifiers else table
+        )
+
     def _resolve_tables(
         self,
         expression: E,
@@ -347,7 +399,7 @@ class ExpressionRenderer(BaseExpressionRenderer):
                 **kwargs,
             )
             for e in expressions
-            if e
+            if e and not isinstance(e, exp.Semicolon)
         ]
 
 
@@ -369,7 +421,7 @@ class QueryRenderer(BaseExpressionRenderer):
         table_mapping: t.Optional[t.Dict[str, str]] = None,
         deployability_index: t.Optional[DeployabilityIndex] = None,
         expand: t.Iterable[str] = tuple(),
-        optimize: bool = True,
+        needs_optimization: bool = True,
         runtime_stage: RuntimeStage = RuntimeStage.LOADING,
         **kwargs: t.Any,
     ) -> t.Optional[exp.Query]:
@@ -386,7 +438,8 @@ class QueryRenderer(BaseExpressionRenderer):
             expand: Expand referenced models as subqueries. This is used to bypass backfills when running queries
                 that depend on materialized tables.  Model definitions are inlined and can thus be run end to
                 end on the fly.
-            optimize: Whether to optimize the query.
+            needs_optimization: Whether or not an optimization should be attempted
+                (if passing False, it still may return a cached optimized query).
             runtime_stage: Indicates the current runtime stage, for example if we're still loading the project, etc.
             kwargs: Additional kwargs to pass to the renderer.
 
@@ -398,7 +451,9 @@ class QueryRenderer(BaseExpressionRenderer):
             runtime_stage, start, end, execution_time, *kwargs.values()
         )
 
-        if should_cache and self._optimized_cache and optimize:
+        needs_optimization = needs_optimization and self._optimize_query_flag
+
+        if should_cache and self._optimized_cache:
             query = self._optimized_cache
         else:
             try:
@@ -415,6 +470,8 @@ class QueryRenderer(BaseExpressionRenderer):
             except ParsetimeAdapterCallError:
                 return None
 
+            expressions = [e for e in expressions if not isinstance(e, exp.Semicolon)]
+
             if not expressions:
                 raise ConfigError(f"Failed to render query at '{self._path}':\n{self._expression}")
 
@@ -426,10 +483,12 @@ class QueryRenderer(BaseExpressionRenderer):
             if not query:
                 return None
             if not isinstance(query, exp.Query):
-                raise_config_error(f"Query needs to be a SELECT or a UNION {query}.", self._path)
+                raise_config_error(
+                    f"Model query needs to be a SELECT or a UNION, got {query}.", self._path
+                )
                 raise
 
-            if optimize:
+            if needs_optimization:
                 deps = d.find_tables(
                     query, default_catalog=self._default_catalog, dialect=self._dialect
                 )
@@ -439,7 +498,7 @@ class QueryRenderer(BaseExpressionRenderer):
                 if should_cache:
                     self._optimized_cache = query
 
-        if optimize:
+        if needs_optimization:
             query = self._resolve_tables(
                 query,
                 snapshots=snapshots,
@@ -476,14 +535,21 @@ class QueryRenderer(BaseExpressionRenderer):
                 missing_deps.add(dep)
 
         if self._model_fqn and not should_optimize and any(s.is_star for s in query.selects):
+            from sqlmesh.core.console import get_console
+
             deps = ", ".join(f"'{dep}'" for dep in sorted(missing_deps))
 
-            logger.warning(
+            warning = (
                 f"SELECT * cannot be expanded due to missing schema(s) for model(s): {deps}. "
                 "Run `sqlmesh create_external_models` and / or make sure that the model "
-                f"'{self._model_fqn}' can be rendered at parse time.",
+                f"'{self._model_fqn}' can be rendered at parse time."
             )
 
+            if self._validate_query:
+                raise_config_error(warning, self._path)
+
+            get_console().log_warning(warning)
+
         try:
             if should_optimize:
                 query = query.copy()
@@ -501,10 +567,22 @@ class QueryRenderer(BaseExpressionRenderer):
                     )
                 )
         except SqlglotError as ex:
+            from sqlmesh.core.console import get_console
+
+            warning = (
+                f"{ex} for model '{self._model_fqn}', the column may not exist or is ambiguous."
+            )
+
+            if self._validate_query:
+                raise_config_error(warning, self._path)
+
             query = original
 
-            logger.warning(
-                "%s for model '%s', the column may not exist or is ambiguous", ex, self._model_fqn
+            get_console().log_warning(warning)
+        except Exception as ex:
+            raise_config_error(
+                f"Failed to optimize query, please file an issue at https://github.com/TobikoData/sqlmesh/issues/new. {ex}",
+                self._path,
             )
 
         if not query.type:
