@@ -1,13 +1,12 @@
 from __future__ import annotations
-
+from enum import Enum
 import logging
-import traceback
 import typing as t
-from datetime import datetime
-
+from sqlglot import exp
 from sqlmesh.core import constants as c
 from sqlmesh.core.console import Console, get_console
 from sqlmesh.core.environment import EnvironmentNamingInfo
+from sqlmesh.core.node import IntervalUnit
 from sqlmesh.core.notification_target import (
     NotificationEvent,
     NotificationTargetManager,
@@ -15,34 +14,55 @@ from sqlmesh.core.notification_target import (
 from sqlmesh.core.snapshot import (
     DeployabilityIndex,
     Snapshot,
+    SnapshotId,
     SnapshotEvaluator,
+    apply_auto_restatements,
     earliest_start_date,
     missing_intervals,
+    merge_intervals,
+    Intervals,
+)
+from sqlmesh.core.snapshot.definition import (
+    Interval,
+    expand_range,
+    parent_snapshots_by_name,
 )
-from sqlmesh.core.snapshot.definition import Interval as SnapshotInterval
-from sqlmesh.core.snapshot.definition import SnapshotId
 from sqlmesh.core.state_sync import StateSync
-from sqlmesh.utils import format_exception
-from sqlmesh.utils.concurrency import concurrent_apply_to_dag
+from sqlmesh.utils.concurrency import concurrent_apply_to_dag, NodeExecutionFailedError
 from sqlmesh.utils.dag import DAG
 from sqlmesh.utils.date import (
     TimeLike,
-    now,
     now_timestamp,
-    to_datetime,
+    to_timestamp,
     validate_date_range,
 )
-from sqlmesh.utils.errors import AuditError, CircuitBreakerError, SQLMeshError
+from sqlmesh.utils.errors import AuditError, NodeAuditsErrors, CircuitBreakerError, SQLMeshError
 
 logger = logging.getLogger(__name__)
-Interval = t.Tuple[datetime, datetime]
-Batch = t.List[Interval]
-SnapshotToBatches = t.Dict[Snapshot, Batch]
+SnapshotToIntervals = t.Dict[Snapshot, Intervals]
 # we store snapshot name instead of snapshots/snapshotids because pydantic
 # is extremely slow to hash. snapshot names should be unique within a dag run
 SchedulingUnit = t.Tuple[str, t.Tuple[Interval, int]]
 
 
+class CompletionStatus(Enum):
+    SUCCESS = "success"
+    FAILURE = "failure"
+    NOTHING_TO_DO = "nothing_to_do"
+
+    @property
+    def is_success(self) -> bool:
+        return self == CompletionStatus.SUCCESS
+
+    @property
+    def is_failure(self) -> bool:
+        return self == CompletionStatus.FAILURE
+
+    @property
+    def is_nothing_to_do(self) -> bool:
+        return self == CompletionStatus.NOTHING_TO_DO
+
+
 class Scheduler:
     """Schedules and manages the evaluation of snapshots.
 
@@ -81,33 +101,32 @@ class Scheduler:
             notification_target_manager or NotificationTargetManager()
         )
 
-    def batches(
+    def merged_missing_intervals(
         self,
         start: t.Optional[TimeLike] = None,
         end: t.Optional[TimeLike] = None,
         execution_time: t.Optional[TimeLike] = None,
         deployability_index: t.Optional[DeployabilityIndex] = None,
-        restatements: t.Optional[t.Dict[SnapshotId, SnapshotInterval]] = None,
+        restatements: t.Optional[t.Dict[SnapshotId, Interval]] = None,
+        interval_end_per_model: t.Optional[t.Dict[str, int]] = None,
         ignore_cron: bool = False,
         end_bounded: bool = False,
         selected_snapshots: t.Optional[t.Set[str]] = None,
-    ) -> SnapshotToBatches:
-        """Find the optimal date interval paramaters based on what needs processing and maximal batch size.
+    ) -> SnapshotToIntervals:
+        """Find the largest contiguous date interval parameters based only on what is missing.
 
         For each node name, find all dependencies and look for a stored snapshot from the metastore. If a snapshot is found,
         calculate the missing intervals that need to be processed given the passed in start and end intervals.
 
-        If a snapshot's node specifies a batch size, consecutive intervals are merged into batches of a size that is less than
-        or equal to the configured one. If no batch size is specified, then it uses the intervals that correspond to the node's cron expression.
-        For example, if a node is supposed to run daily and has 70 days to backfill with a batch size set to 30, there would be 2 jobs
-        with 30 days and 1 job with 10.
+        This is a superset of what may actually get processed at runtime based on things like batch size, signal readiness, etc.
 
         Args:
             start: The start of the run. Defaults to the min node start date.
             end: The end of the run. Defaults to now.
-            execution_time: The date/time time reference to use for execution time. Defaults to now.
+            execution_time: The date/time reference to use for execution time. Defaults to now.
             deployability_index: Determines snapshots that are deployable in the context of this evaluation.
             restatements: A set of snapshot names being restated.
+            interval_end_per_model: The mapping from model FQNs to target end dates.
             ignore_cron: Whether to ignore the node's cron schedule.
             end_bounded: If set to true, the returned intervals will be bounded by the target end date, disregarding lookback,
                 allow_partials, and other attributes that could cause the intervals to exceed the target end date.
@@ -117,21 +136,24 @@ class Scheduler:
         validate_date_range(start, end)
 
         snapshots: t.Collection[Snapshot] = self.snapshot_per_version.values()
-        if selected_snapshots is not None:
-            snapshots = [s for s in snapshots if s.name in selected_snapshots]
-
-        self.state_sync.refresh_snapshot_intervals(snapshots)
-
-        return compute_interval_params(
+        snapshots_to_intervals = compute_interval_params(
             snapshots,
             start=start or earliest_start_date(snapshots),
-            end=end or now(),
+            end=end or now_timestamp(),
             deployability_index=deployability_index,
-            execution_time=execution_time or now(),
+            execution_time=execution_time or now_timestamp(),
             restatements=restatements,
+            interval_end_per_model=interval_end_per_model,
             ignore_cron=ignore_cron,
             end_bounded=end_bounded,
         )
+        # Filtering snapshots after computing missing intervals because we need all snapshots in order
+        # to correctly infer start dates.
+        if selected_snapshots is not None:
+            snapshots_to_intervals = {
+                s: i for s, i in snapshots_to_intervals.items() if s.name in selected_snapshots
+            }
+        return snapshots_to_intervals
 
     def evaluate(
         self,
@@ -152,14 +174,12 @@ class Scheduler:
             execution_time: The date/time time reference to use for execution time. Defaults to now.
             deployability_index: Determines snapshots that are deployable in the context of this evaluation.
             batch_index: If the snapshot is part of a batch of related snapshots; which index in the batch is it
+            auto_restatement_enabled: Whether to enable auto restatements.
             kwargs: Additional kwargs to pass to the renderer.
         """
         validate_date_range(start, end)
 
-        snapshots = {
-            self.snapshots[p_sid].name: self.snapshots[p_sid] for p_sid in snapshot.parents
-        }
-        snapshots[snapshot.name] = snapshot
+        snapshots = parent_snapshots_by_name(snapshot, self.snapshots)
 
         is_deployable = deployability_index.is_deployable(snapshot)
 
@@ -173,25 +193,42 @@ class Scheduler:
             batch_index=batch_index,
             **kwargs,
         )
-        try:
-            self.snapshot_evaluator.audit(
-                snapshot=snapshot,
-                start=start,
-                end=end,
-                execution_time=execution_time,
-                snapshots=snapshots,
-                deployability_index=deployability_index,
-                wap_id=wap_id,
-                **kwargs,
+        audit_results = self.snapshot_evaluator.audit(
+            snapshot=snapshot,
+            start=start,
+            end=end,
+            execution_time=execution_time,
+            snapshots=snapshots,
+            deployability_index=deployability_index,
+            wap_id=wap_id,
+            **kwargs,
+        )
+
+        audit_errors_to_raise: t.List[AuditError] = []
+        for audit_result in (result for result in audit_results if result.count):
+            error = AuditError(
+                audit_name=audit_result.audit.name,
+                audit_args=audit_result.audit_args,
+                model=snapshot.model_or_none,
+                count=t.cast(int, audit_result.count),
+                query=t.cast(exp.Query, audit_result.query),
+                adapter_dialect=self.snapshot_evaluator.adapter.dialect,
             )
-        except AuditError as e:
-            self.notification_target_manager.notify(NotificationEvent.AUDIT_FAILURE, e)
+            self.notification_target_manager.notify(NotificationEvent.AUDIT_FAILURE, error)
             if is_deployable and snapshot.node.owner:
                 self.notification_target_manager.notify_user(
-                    NotificationEvent.AUDIT_FAILURE, snapshot.node.owner, e
+                    NotificationEvent.AUDIT_FAILURE, snapshot.node.owner, error
                 )
-            logger.error(f"Audit Failure: {traceback.format_exc()}")
-            raise e
+            if audit_result.blocking:
+                audit_errors_to_raise.append(error)
+            else:
+                get_console().log_warning(
+                    f"\n{error}.",
+                    long_message=f"{error}. Audit query:\n{error.query.sql(error.adapter_dialect)}",
+                )
+
+        if audit_errors_to_raise:
+            raise NodeAuditsErrors(audit_errors_to_raise)
 
         self.state_sync.add_interval(snapshot, start, end, is_dev=not is_deployable)
 
@@ -201,13 +238,15 @@ class Scheduler:
         start: t.Optional[TimeLike] = None,
         end: t.Optional[TimeLike] = None,
         execution_time: t.Optional[TimeLike] = None,
-        restatements: t.Optional[t.Dict[SnapshotId, SnapshotInterval]] = None,
+        restatements: t.Optional[t.Dict[SnapshotId, Interval]] = None,
+        interval_end_per_model: t.Optional[t.Dict[str, int]] = None,
         ignore_cron: bool = False,
         end_bounded: bool = False,
         selected_snapshots: t.Optional[t.Set[str]] = None,
         circuit_breaker: t.Optional[t.Callable[[], bool]] = None,
         deployability_index: t.Optional[DeployabilityIndex] = None,
-    ) -> bool:
+        auto_restatement_enabled: bool = False,
+    ) -> CompletionStatus:
         """Concurrently runs all snapshots in topological order.
 
         Args:
@@ -218,12 +257,14 @@ class Scheduler:
             end: The end of the run. Defaults to now.
             execution_time: The date/time time reference to use for execution time. Defaults to now.
             restatements: A dict of snapshots to restate and their intervals.
+            interval_end_per_model: The mapping from model FQNs to target end dates.
             ignore_cron: Whether to ignore the node's cron schedule.
             end_bounded: If set to true, the evaluated intervals will be bounded by the target end date, disregarding lookback,
                 allow_partials, and other attributes that could cause the intervals to exceed the target end date.
             selected_snapshots: A set of snapshot names to run. If not provided, all snapshots will be run.
             circuit_breaker: An optional handler which checks if the run should be aborted.
             deployability_index: Determines snapshots that are deployable in the context of this render.
+            auto_restatement_enabled: Whether to enable auto restatements.
 
         Returns:
             True if the execution was successful and False otherwise.
@@ -242,32 +283,158 @@ class Scheduler:
             environment_naming_info = environment
 
         deployability_index = deployability_index or (
-            DeployabilityIndex.create(self.snapshots.values())
+            DeployabilityIndex.create(self.snapshots.values(), start=start)
             if environment_naming_info.name != c.PROD
             else DeployabilityIndex.all_deployable()
         )
-        execution_time = execution_time or now()
-        batches = self.batches(
+        execution_time = execution_time or now_timestamp()
+
+        self.state_sync.refresh_snapshot_intervals(self.snapshots.values())
+        for s_id, interval in (restatements or {}).items():
+            self.snapshots[s_id].remove_interval(interval)
+
+        if auto_restatement_enabled:
+            auto_restated_intervals = apply_auto_restatements(self.snapshots, execution_time)
+            self.state_sync.add_snapshots_intervals(auto_restated_intervals)
+            self.state_sync.update_auto_restatements(
+                {s.name_version: s.next_auto_restatement_ts for s in self.snapshots.values()}
+            )
+
+        merged_intervals = self.merged_missing_intervals(
             start,
             end,
             execution_time,
             deployability_index=deployability_index,
             restatements=restatements,
+            interval_end_per_model=interval_end_per_model,
             ignore_cron=ignore_cron,
             end_bounded=end_bounded,
             selected_snapshots=selected_snapshots,
         )
-        if not batches:
-            return True
 
-        dag = self._dag(batches)
+        if not merged_intervals:
+            return CompletionStatus.NOTHING_TO_DO
+
+        errors, skipped_intervals = self.run_merged_intervals(
+            merged_intervals=merged_intervals,
+            deployability_index=deployability_index,
+            environment_naming_info=environment_naming_info,
+            execution_time=execution_time,
+            circuit_breaker=circuit_breaker,
+            start=start,
+            end=end,
+        )
+
+        self.console.stop_evaluation_progress(success=not errors)
+
+        skipped_snapshots = {i[0] for i in skipped_intervals}
+        self.console.log_skipped_models(skipped_snapshots)
+        for skipped in skipped_snapshots:
+            logger.info(f"SKIPPED snapshot {skipped}\n")
+
+        for error in errors:
+            if isinstance(error.__cause__, CircuitBreakerError):
+                raise error.__cause__
+            logger.info(str(error), exc_info=error)
+
+        self.console.log_failed_models(errors)
+
+        return CompletionStatus.FAILURE if errors else CompletionStatus.SUCCESS
+
+    def batch_intervals(
+        self,
+        merged_intervals: SnapshotToIntervals,
+        start: t.Optional[TimeLike] = None,
+        end: t.Optional[TimeLike] = None,
+        execution_time: t.Optional[TimeLike] = None,
+    ) -> t.Dict[Snapshot, Intervals]:
+        def expand_range_as_interval(
+            start_ts: int, end_ts: int, interval_unit: IntervalUnit
+        ) -> t.List[Interval]:
+            values = expand_range(start_ts, end_ts, interval_unit)
+            return [(values[i], values[i + 1]) for i in range(len(values) - 1)]
+
+        dag = DAG[str]()
+
+        for snapshot in merged_intervals:
+            dag.add(snapshot.name, [p.name for p in snapshot.parents])
+
+        snapshot_intervals = {
+            snapshot: [
+                i
+                for interval in intervals
+                for i in expand_range_as_interval(*interval, snapshot.node.interval_unit)
+            ]
+            for snapshot, intervals in merged_intervals.items()
+        }
+        snapshot_batches = {}
+        all_unready_intervals: t.Dict[str, set[Interval]] = {}
+        for snapshot, intervals in snapshot_intervals.items():
+            unready = set(intervals)
+            intervals = snapshot.check_ready_intervals(intervals)
+            unready -= set(intervals)
+
+            for parent in snapshot.parents:
+                if parent.name in all_unready_intervals:
+                    unready.update(all_unready_intervals[parent.name])
+
+            all_unready_intervals[snapshot.name] = unready
+
+            batches = []
+            batch_size = snapshot.node.batch_size
+            next_batch: t.List[t.Tuple[int, int]] = []
+
+            for interval in interval_diff(
+                intervals, merge_intervals(unready), uninterrupted=snapshot.depends_on_past
+            ):
+                if (batch_size and len(next_batch) >= batch_size) or (
+                    next_batch and interval[0] != next_batch[-1][-1]
+                ):
+                    batches.append((next_batch[0][0], next_batch[-1][-1]))
+                    next_batch = []
+                next_batch.append(interval)
+            if next_batch:
+                batches.append((next_batch[0][0], next_batch[-1][-1]))
+            snapshot_batches[snapshot] = batches
+        return snapshot_batches
+
+    def run_merged_intervals(
+        self,
+        *,
+        merged_intervals: SnapshotToIntervals,
+        deployability_index: DeployabilityIndex,
+        environment_naming_info: EnvironmentNamingInfo,
+        execution_time: t.Optional[TimeLike] = None,
+        circuit_breaker: t.Optional[t.Callable[[], bool]] = None,
+        start: t.Optional[TimeLike] = None,
+        end: t.Optional[TimeLike] = None,
+    ) -> t.Tuple[t.List[NodeExecutionFailedError[SchedulingUnit]], t.List[SchedulingUnit]]:
+        """Runs precomputed batches of missing intervals.
+
+        Args:
+            merged_intervals: The snapshots and contiguous interval ranges to evaluate.
+            deployability_index: Determines snapshots that are deployable in the context of this evaluation.
+            environment_naming_info: The environment naming info the user is targeting when applying their change.
+            execution_time: The date/time reference to use for execution time.
+            circuit_breaker: An optional handler which checks if the run should be aborted.
+            start: The start of the run.
+            end: The end of the run.
+
+        Returns:
+            A tuple of errors and skipped intervals.
+        """
+        execution_time = execution_time or now_timestamp()
+
+        batched_intervals = self.batch_intervals(merged_intervals, start, end, execution_time)
 
         self.console.start_evaluation_progress(
-            {snapshot: len(intervals) for snapshot, intervals in batches.items()},
+            {snapshot: len(intervals) for snapshot, intervals in batched_intervals.items()},
             environment_naming_info,
             self.default_catalog,
         )
 
+        dag = self._dag(batched_intervals)
+
         snapshots_by_name = {snapshot.name: snapshot for snapshot in self.snapshots.values()}
 
         def evaluate_node(node: SchedulingUnit) -> None:
@@ -296,7 +463,7 @@ class Scheduler:
 
         try:
             with self.snapshot_evaluator.concurrent_context():
-                errors, skipped_intervals = concurrent_apply_to_dag(
+                return concurrent_apply_to_dag(
                     dag,
                     evaluate_node,
                     self.max_workers,
@@ -305,27 +472,7 @@ class Scheduler:
         finally:
             self.state_sync.recycle()
 
-        self.console.stop_evaluation_progress(success=not errors)
-
-        skipped_snapshots = {i[0] for i in skipped_intervals}
-        for skipped in skipped_snapshots:
-            log_message = f"SKIPPED snapshot {skipped}\n"
-            self.console.log_status_update(log_message)
-            logger.info(log_message)
-
-        for error in errors:
-            if isinstance(error.__cause__, CircuitBreakerError):
-                raise error.__cause__
-            sid = error.node[0]
-            formatted_exception = "".join(format_exception(error.__cause__ or error))
-            log_message = f"FAILED processing snapshot {sid}\n{formatted_exception}"
-            self.console.log_error(log_message)
-            # Log with INFO level to prevent duplicate messages in the console.
-            logger.info(log_message)
-
-        return not errors
-
-    def _dag(self, batches: SnapshotToBatches) -> DAG[SchedulingUnit]:
+    def _dag(self, batches: SnapshotToIntervals) -> DAG[SchedulingUnit]:
         """Builds a DAG of snapshot intervals to be evaluated.
 
         Args:
@@ -340,7 +487,7 @@ class Scheduler:
         }
 
         dag = DAG[SchedulingUnit]()
-        terminal_node = ((to_datetime(0), to_datetime(0)), -1)
+        terminal_node = ((to_timestamp(0), to_timestamp(0)), -1)
 
         for snapshot, intervals in batches.items():
             if not intervals:
@@ -390,28 +537,26 @@ def compute_interval_params(
     end: TimeLike,
     deployability_index: t.Optional[DeployabilityIndex] = None,
     execution_time: t.Optional[TimeLike] = None,
-    restatements: t.Optional[t.Dict[SnapshotId, SnapshotInterval]] = None,
+    restatements: t.Optional[t.Dict[SnapshotId, Interval]] = None,
+    interval_end_per_model: t.Optional[t.Dict[str, int]] = None,
     ignore_cron: bool = False,
     end_bounded: bool = False,
-) -> SnapshotToBatches:
-    """Find the optimal date interval paramaters based on what needs processing and maximal batch size.
+) -> SnapshotToIntervals:
+    """Find the largest contiguous date interval parameters based only on what is missing.
 
     For each node name, find all dependencies and look for a stored snapshot from the metastore. If a snapshot is found,
     calculate the missing intervals that need to be processed given the passed in start and end intervals.
 
-    If a snapshot's node specifies a batch size, consecutive intervals are merged into batches of a size that is less than
-    or equal to the configured one. If no batch size is specified, then it uses the intervals that correspond to the node's cron expression.
-    For example, if a node is supposed to run daily and has 70 days to backfill with a batch size set to 30, there would be 2 jobs
-    with 30 days and 1 job with 10.
+    This is a superset of what may actually get processed at runtime based on things like batch size, signal readiness, etc.
 
     Args:
         snapshots: A set of target snapshots for which intervals should be computed.
-        intervals: A list of all snapshot intervals that should be considered.
         start: Start of the interval.
         end: End of the interval.
         deployability_index: Determines snapshots that are deployable in the context of this evaluation.
-        execution_time: The date/time time reference to use for execution time.
+        execution_time: The date/time reference to use for execution time.
         restatements: A dict of snapshot names being restated and their intervals.
+        interval_end_per_model: The mapping from model FQNs to target end dates.
         ignore_cron: Whether to ignore the node's cron schedule.
         end_bounded: If set to true, the returned intervals will be bounded by the target end date, disregarding lookback,
             allow_partials, and other attributes that could cause the intervals to exceed the target end date.
@@ -419,7 +564,7 @@ def compute_interval_params(
     Returns:
         A dict containing all snapshots needing to be run with their associated interval params.
     """
-    snapshot_batches = {}
+    snapshot_merged_intervals = {}
 
     for snapshot, intervals in missing_intervals(
         snapshots,
@@ -428,25 +573,61 @@ def compute_interval_params(
         execution_time=execution_time,
         restatements=restatements,
         deployability_index=deployability_index,
+        interval_end_per_model=interval_end_per_model,
         ignore_cron=ignore_cron,
         end_bounded=end_bounded,
     ).items():
-        batches = []
-        batch_size = snapshot.node.batch_size
-        next_batch: t.List[t.Tuple[int, int]] = []
+        contiguous_batch = []
+        next_batch: Intervals = []
 
         for interval in intervals:
-            if (batch_size and len(next_batch) >= batch_size) or (
-                next_batch and interval[0] != next_batch[-1][-1]
-            ):
-                batches.append((next_batch[0][0], next_batch[-1][-1]))
+            if next_batch and interval[0] != next_batch[-1][-1]:
+                contiguous_batch.append((next_batch[0][0], next_batch[-1][-1]))
                 next_batch = []
             next_batch.append(interval)
         if next_batch:
-            batches.append((next_batch[0][0], next_batch[-1][-1]))
-        snapshot_batches[snapshot] = [(to_datetime(s), to_datetime(e)) for s, e in batches]
+            contiguous_batch.append((next_batch[0][0], next_batch[-1][-1]))
+        snapshot_merged_intervals[snapshot] = contiguous_batch
 
-    return snapshot_batches
+    return snapshot_merged_intervals
+
+
+def interval_diff(
+    intervals_a: Intervals, intervals_b: Intervals, uninterrupted: bool = False
+) -> Intervals:
+    if not intervals_a or not intervals_b:
+        return intervals_a
+
+    index_a, index_b = 0, 0
+    len_a = len(intervals_a)
+    len_b = len(intervals_b)
+
+    results = []
+
+    while index_a < len_a and index_b < len_b:
+        interval_a = intervals_a[index_a]
+        interval_b = intervals_b[index_b]
+
+        if interval_a[0] >= interval_b[1]:
+            index_b += 1
+        elif interval_b[0] >= interval_a[1]:
+            results.append(interval_a)
+            index_a += 1
+        else:
+            if uninterrupted:
+                return results
+
+            if interval_a[0] >= interval_b[0]:
+                index_a += 1
+            else:
+                index_b += 1
+
+    if index_a < len_a:
+        interval_a = intervals_a[index_a]
+        if interval_a[0] >= interval_b[1] or interval_b[0] >= interval_a[1]:
+            results.extend(intervals_a[index_a:])
+
+    return results
 
 
 def _resolve_one_snapshot_per_version(
