@@ -17,16 +17,13 @@ from sqlmesh.core.snapshot import (
     SnapshotInfoLike,
     SnapshotTableCleanupTask,
     SnapshotTableInfo,
+    SnapshotNameVersion,
 )
-from sqlmesh.core.snapshot.definition import Interval
+from sqlmesh.core.snapshot.definition import Interval, SnapshotIntervals
 from sqlmesh.utils import major_minor
 from sqlmesh.utils.date import TimeLike
 from sqlmesh.utils.errors import SQLMeshError
-from sqlmesh.utils.pydantic import (
-    PydanticModel,
-    field_validator,
-    field_validator_v1_args,
-)
+from sqlmesh.utils.pydantic import PydanticModel, ValidationInfo, field_validator
 
 logger = logging.getLogger(__name__)
 
@@ -70,11 +67,10 @@ class PromotionResult(PydanticModel):
     removed_environment_naming_info: t.Optional[EnvironmentNamingInfo]
 
     @field_validator("removed_environment_naming_info")
-    @field_validator_v1_args
     def _validate_removed_environment_naming_info(
-        cls, v: t.Optional[EnvironmentNamingInfo], values: t.Any
+        cls, v: t.Optional[EnvironmentNamingInfo], info: ValidationInfo
     ) -> t.Optional[EnvironmentNamingInfo]:
-        if v and not values["removed"]:
+        if v and not info.data.get("removed"):
             raise ValueError("removed_environment_naming_info must be None if removed is empty")
         return v
 
@@ -86,14 +82,12 @@ class StateReader(abc.ABC):
     def get_snapshots(
         self,
         snapshot_ids: t.Optional[t.Iterable[SnapshotIdLike]],
-        hydrate_seeds: bool = False,
     ) -> t.Dict[SnapshotId, Snapshot]:
         """Bulk fetch snapshots given the corresponding snapshot ids.
 
         Args:
             snapshot_ids: Iterable of snapshot ids to get. If not provided all
                 available snapshots will be returned.
-            hydrate_seeds: Whether to hydrate seed snapshots with the content.
 
         Returns:
             A dictionary of snapshot ids to snapshots for ones that could be found.
@@ -142,34 +136,30 @@ class StateReader(abc.ABC):
         """
 
     @abc.abstractmethod
-    def max_interval_end_for_environment(
-        self, environment: str, ensure_finalized_snapshots: bool = False
-    ) -> t.Optional[int]:
-        """Returns the max interval end for the given environment.
-
-        Args:
-            environment: The environment.
-            ensure_finalized_snapshots: Whether to use snapshots from the latest finalized environment state,
-                or to use whatever snapshots are in the current environment state even if the environment is not finalized.
+    def get_environments_summary(self) -> t.Dict[str, int]:
+        """Fetches all environment names along with expiry datetime.
 
         Returns:
-            A timestamp or None if no interval or environment exists.
+            A dict of all environment names along with expiry datetime.
         """
 
     @abc.abstractmethod
-    def greatest_common_interval_end(
-        self, environment: str, models: t.Set[str], ensure_finalized_snapshots: bool = False
-    ) -> t.Optional[int]:
-        """Returns the greatest common interval end for given models in the target environment.
+    def max_interval_end_per_model(
+        self,
+        environment: str,
+        models: t.Optional[t.Set[str]] = None,
+        ensure_finalized_snapshots: bool = False,
+    ) -> t.Dict[str, int]:
+        """Returns the max interval end per model for the given environment.
 
         Args:
-            environment: The environment.
-            models: The model FQNs to select intervals from.
+            environment: The target environment.
+            models: The models to get the max interval end for. If None, all models are considered.
             ensure_finalized_snapshots: Whether to use snapshots from the latest finalized environment state,
                 or to use whatever snapshots are in the current environment state even if the environment is not finalized.
 
         Returns:
-            A timestamp or None if no interval or environment exists.
+            A dictionary of model FQNs to their respective interval ends in milliseconds since epoch.
         """
 
     @abc.abstractmethod
@@ -181,6 +171,20 @@ class StateReader(abc.ABC):
     def close(self) -> None:
         """Closes all open connections and releases all allocated resources."""
 
+    @abc.abstractmethod
+    def state_type(self) -> str:
+        """Returns the type of state sync."""
+
+    @abc.abstractmethod
+    def update_auto_restatements(
+        self, next_auto_restatement_ts: t.Dict[SnapshotNameVersion, t.Optional[int]]
+    ) -> None:
+        """Updates the next auto restatement timestamp for the snapshots.
+
+        Args:
+            next_auto_restatement_ts: A dictionary of snapshot name / version pairs to the next auto restatement timestamp.
+        """
+
     def get_versions(self, validate: bool = True) -> Versions:
         """Get the current versions of the SQLMesh schema and libraries.
 
@@ -248,12 +252,9 @@ class StateReader(abc.ABC):
         return versions
 
     @abc.abstractmethod
-    def _get_versions(self, lock_for_update: bool = False) -> Versions:
+    def _get_versions(self) -> Versions:
         """Queries the store to get the current versions of SQLMesh and deps.
 
-        Args:
-            lock_for_update: Whether or not the usage of this method plans to update the row.
-
         Returns:
             The versions object.
         """
@@ -285,12 +286,18 @@ class StateSync(StateReader, abc.ABC):
         """
 
     @abc.abstractmethod
-    def delete_expired_snapshots(self) -> t.List[SnapshotTableCleanupTask]:
+    def delete_expired_snapshots(
+        self, ignore_ttl: bool = False
+    ) -> t.List[SnapshotTableCleanupTask]:
         """Removes expired snapshots.
 
         Expired snapshots are snapshots that have exceeded their time-to-live
         and are no longer in use within an environment.
 
+        Args:
+            ignore_ttl: Ignore the TTL on the snapshot when considering it expired. This has the effect of deleting
+                all snapshots that are not referenced in any environment
+
         Returns:
             The list of table cleanup tasks.
         """
@@ -304,30 +311,9 @@ class StateSync(StateReader, abc.ABC):
         """
 
     @abc.abstractmethod
-    def add_interval(
-        self,
-        snapshot: Snapshot,
-        start: TimeLike,
-        end: TimeLike,
-        is_dev: bool = False,
-    ) -> None:
-        """Add an interval to a snapshot and sync it to the store.
-
-        Snapshots must be pushed before adding intervals to them.
-
-        Args:
-            snapshot: The snapshot like object to add an interval to.
-            start: The start of the interval to add.
-            end: The end of the interval to add.
-            is_dev: Indicates whether the given interval is being added while in
-                development mode.is_dev.
-        """
-
-    @abc.abstractmethod
-    def remove_interval(
+    def remove_intervals(
         self,
         snapshot_intervals: t.Sequence[t.Tuple[SnapshotInfoLike, Interval]],
-        execution_time: t.Optional[TimeLike] = None,
         remove_shared_versions: bool = False,
     ) -> None:
         """Remove an interval from a list of snapshots and sync it to the store.
@@ -336,10 +322,8 @@ class StateSync(StateReader, abc.ABC):
         can also grab all snapshots tied to the passed in version.
 
         Args:
-            snapshots: The snapshot info like object to remove intervals from.
-            start: The start of the interval to add.
-            end: The end of the interval to add.
-            all_snapshots: All snapshots can be passed in to skip fetching matching snapshot versions.
+            snapshot_intervals: The snapshot intervals to remove.
+            remove_shared_versions: Whether to remove intervals for snapshots that share the same version with the target snapshots.
         """
 
     @abc.abstractmethod
@@ -429,6 +413,43 @@ class StateSync(StateReader, abc.ABC):
     def rollback(self) -> None:
         """Rollback to previous backed up state."""
 
+    @abc.abstractmethod
+    def add_snapshots_intervals(self, snapshots_intervals: t.Sequence[SnapshotIntervals]) -> None:
+        """Add snapshot intervals to state
+
+        Args:
+            snapshots_intervals: The intervals to add.
+        """
+
+    def add_interval(
+        self,
+        snapshot: Snapshot,
+        start: TimeLike,
+        end: TimeLike,
+        is_dev: bool = False,
+    ) -> None:
+        """Add an interval to a snapshot and sync it to the store.
+
+        Args:
+            snapshot: The snapshot like object to add an interval to.
+            start: The start of the interval to add.
+            end: The end of the interval to add.
+            is_dev: Indicates whether the given interval is being added while in development mode
+        """
+        start_ts, end_ts = snapshot.inclusive_exclusive(start, end, strict=False, expand=False)
+        if not snapshot.version:
+            raise SQLMeshError("Snapshot version must be set to add an interval.")
+        intervals = [(start_ts, end_ts)]
+        snapshot_intervals = SnapshotIntervals(
+            name=snapshot.name,
+            identifier=snapshot.identifier,
+            version=snapshot.version,
+            dev_version=snapshot.dev_version,
+            intervals=intervals if not is_dev else [],
+            dev_intervals=intervals if is_dev else [],
+        )
+        self.add_snapshots_intervals([snapshot_intervals])
+
 
 class DelegatingStateSync(StateSync):
     def __init__(self, state_sync: StateSync) -> None:
