@@ -1,14 +1,21 @@
+# ruff: noqa: E402
 from __future__ import annotations
 
 import json
 import logging
+import os
 import re
 import typing as t
 from argparse import Namespace
 from collections import defaultdict
 from pathlib import Path
 
-from dbt import flags
+from dbt import constants as dbt_constants, flags
+
+# Override the file name to prevent dbt commands from invalidating the cache.
+dbt_constants.PARTIAL_PARSE_FILE_NAME = "sqlmesh_partial_parse.msgpack"
+
+import jinja2
 from dbt.adapters.factory import register_adapter, reset_adapters
 from dbt.config import Profile, Project, RuntimeConfig
 from dbt.config.profile import read_profile
@@ -16,6 +23,7 @@ from dbt.config.renderer import DbtProjectYamlRenderer, ProfileRenderer
 from dbt.parser.manifest import ManifestLoader
 from dbt.tracking import do_not_track
 
+from sqlmesh.core import constants as c
 from sqlmesh.dbt.basemodel import Dependencies
 from sqlmesh.dbt.builtin import BUILTIN_FILTERS, BUILTIN_GLOBALS, OVERRIDDEN_MACROS
 from sqlmesh.dbt.model import ModelConfig
@@ -25,6 +33,7 @@ from sqlmesh.dbt.source import SourceConfig
 from sqlmesh.dbt.target import TargetConfig
 from sqlmesh.dbt.test import TestConfig
 from sqlmesh.dbt.util import DBT_VERSION
+from sqlmesh.utils.cache import FileCache
 from sqlmesh.utils.errors import ConfigError
 from sqlmesh.utils.jinja import (
     MacroInfo,
@@ -36,6 +45,7 @@ from sqlmesh.utils.jinja import (
 if t.TYPE_CHECKING:
     from dbt.contracts.graph.manifest import Macro, Manifest
     from dbt.contracts.graph.nodes import ManifestNode, SourceDefinition
+    from sqlmesh.utils.jinja import CallNames
 
 logger = logging.getLogger(__name__)
 
@@ -80,6 +90,9 @@ class ManifestHelper:
         self._tests_by_owner: t.Dict[str, t.List[TestConfig]] = defaultdict(list)
         self._disabled_refs: t.Optional[t.Set[str]] = None
         self._disabled_sources: t.Optional[t.Set[str]] = None
+        self._call_cache: FileCache[t.Dict[str, t.List[CallNames]]] = FileCache(
+            self.project_path / c.CACHE, "jinja_calls"
+        )
 
     def tests(self, package_name: t.Optional[str] = None) -> TestConfigs:
         self._load_all()
@@ -113,12 +126,17 @@ class ManifestHelper:
     def _load_all(self) -> None:
         if self._is_loaded:
             return
+
+        self._calls = {k: (v, False) for k, v in (self._call_cache.get("") or {}).items()}
+
         self._load_macros()
         self._load_sources()
         self._load_tests()
         self._load_models_and_seeds()
         self._is_loaded = True
 
+        self._call_cache.put("", value={k: v for k, (v, used) in self._calls.items() if used})
+
     def _load_sources(self) -> None:
         for source in self._manifest.sources.values():
             source_config = SourceConfig(
@@ -149,6 +167,19 @@ class ManifestHelper:
                 path=Path(macro.original_file_path),
             )
 
+        # This is a workaround for dbt adapter macros (eg. "spark__dateadd") whcih are expected to be
+        # available in the global scope regardless of the package they came from.
+        adapter_macro_names = {
+            name[name.find("__") + 2 :]
+            for name in self._macros_per_package.get("dbt", {})
+            if "__" in name
+        }
+        for macros in self._macros_per_package.values():
+            for name, macro_config in macros.items():
+                pos = name.find("__")
+                if pos > 0 and name[pos + 2 :] in adapter_macro_names:
+                    macro_config.info.is_top_level = True
+
     def _load_tests(self) -> None:
         for node in self._manifest.nodes.values():
             if node.resource_type != "test":
@@ -212,6 +243,11 @@ class ManifestHelper:
             )
             node_config = _node_base_config(node)
 
+            node_name = node.name
+            node_version = getattr(node, "version", None)
+            if node_version:
+                node_name = f"{node_name}_v{node_version}"
+
             if node.resource_type in {"model", "snapshot"}:
                 sql = node.raw_code if DBT_VERSION >= (1, 3) else node.raw_sql  # type: ignore
                 dependencies = Dependencies(
@@ -222,14 +258,14 @@ class ManifestHelper:
                     self._flatten_dependencies_from_macros(dependencies.macros, node.package_name)
                 )
 
-                self._models_per_package[node.package_name][node.name] = ModelConfig(
+                self._models_per_package[node.package_name][node_name] = ModelConfig(
                     sql=sql,
                     dependencies=dependencies,
                     tests=tests,
                     **node_config,
                 )
             else:
-                self._seeds_per_package[node.package_name][node.name] = SeedConfig(
+                self._seeds_per_package[node.package_name][node_name] = SeedConfig(
                     dependencies=Dependencies(macros=macro_references),
                     tests=tests,
                     **node_config,
@@ -253,6 +289,7 @@ class ManifestHelper:
         args: Namespace = Namespace(
             vars=variables,
             profile=self.profile_name,
+            project_dir=str(self.project_path),
             profiles_dir=str(self.profiles_path),
             target=self.target.name,
             macro_debugging=False,
@@ -260,6 +297,11 @@ class ManifestHelper:
         )
         flags.set_from_args(args, None)
 
+        if DBT_VERSION >= (1, 8):
+            from dbt_common.context import set_invocation_context  # type: ignore
+
+            set_invocation_context(os.environ)
+
         profile = self._load_profile()
         project = self._load_project(profile)
 
@@ -274,10 +316,8 @@ class ManifestHelper:
 
         if DBT_VERSION >= (1, 8):
             from dbt.mp_context import get_mp_context  # type: ignore
-            from dbt_common.context import set_invocation_context  # type: ignore
 
             register_adapter(runtime_config, get_mp_context())  # type: ignore
-            set_invocation_context({})
         else:
             register_adapter(runtime_config)  # type: ignore
 
@@ -370,9 +410,12 @@ class ManifestHelper:
         # This behavior has been observed with macros like dbt.current_timestamp(), dbt_utils.slugify(), and source().
         # Here we apply our custom extractor to make a best effort to supplement references captured in the manifest.
         dependencies = Dependencies()
-        for call_name, node in extract_call_names(target):
+        for call_name, node in extract_call_names(target, cache=self._calls):
             if call_name[0] == "config":
                 continue
+            elif isinstance(node, jinja2.nodes.Getattr):
+                if call_name[0] == "model":
+                    dependencies.model_attrs.add(call_name[1])
             elif call_name[0] == "source":
                 args = [jinja_call_arg_name(arg) for arg in node.args]
                 if args and all(arg for arg in args):
@@ -447,7 +490,13 @@ def _macro_references(
 
 def _refs(node: ManifestNode) -> t.Set[str]:
     if DBT_VERSION >= (1, 5):
-        return {f"{r.package}.{r.name}" if r.package else r.name for r in node.refs}  # type: ignore
+        result = set()
+        for r in node.refs:
+            ref_name = f"{r.package}.{r.name}" if r.package else r.name
+            if getattr(r, "version", None):
+                ref_name = f"{ref_name}_v{r.version}"
+            result.add(ref_name)
+        return result
     else:
         return {".".join(r) for r in node.refs}  # type: ignore
 
