@@ -1,19 +1,19 @@
 from __future__ import annotations
 
 import abc
-import logging
-import sys
 import typing as t
 from pathlib import Path
 
 from dbt.adapters.base import BaseRelation, Column
-from dbt.contracts.relation import Policy
 from pydantic import Field
 
+from sqlmesh.core.console import get_console
 from sqlmesh.core.config.connection import (
+    AthenaConnectionConfig,
     BigQueryConnectionConfig,
     BigQueryConnectionMethod,
     BigQueryPriority,
+    ClickhouseConnectionConfig,
     ConnectionConfig,
     DatabricksConnectionConfig,
     DuckDBConnectionConfig,
@@ -30,21 +30,11 @@ from sqlmesh.core.model import (
     IncrementalUnmanagedKind,
 )
 from sqlmesh.dbt.common import DbtConfig
+from sqlmesh.dbt.relation import Policy
 from sqlmesh.dbt.util import DBT_VERSION
 from sqlmesh.utils import AttributeDict, classproperty
 from sqlmesh.utils.errors import ConfigError
-from sqlmesh.utils.pydantic import (
-    field_validator,
-    model_validator,
-    model_validator_v1_args,
-)
-
-if sys.version_info >= (3, 9):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
-
-logger = logging.getLogger(__name__)
+from sqlmesh.utils.pydantic import field_validator, model_validator
 
 IncrementalKind = t.Union[
     t.Type[IncrementalByUniqueKeyKind],
@@ -109,6 +99,10 @@ class TargetConfig(abc.ABC, DbtConfig):
             return MSSQLConfig(**data)
         elif db_type == "trino":
             return TrinoConfig(**data)
+        elif db_type == "clickhouse":
+            return ClickhouseConfig(**data)
+        elif db_type == "athena":
+            return AthenaConfig(**data)
 
         raise ConfigError(f"{db_type} not supported.")
 
@@ -159,7 +153,7 @@ class DuckDbConfig(TargetConfig):
         settings: A dictionary of settings to pass into the duckdb connector.
     """
 
-    type: Literal["duckdb"] = "duckdb"
+    type: t.Literal["duckdb"] = "duckdb"
     database: str = "main"
     schema_: str = Field(default="main", alias="schema")
     path: str = DUCKDB_IN_MEMORY
@@ -167,20 +161,22 @@ class DuckDbConfig(TargetConfig):
     settings: t.Optional[t.Dict[str, t.Any]] = None
 
     @model_validator(mode="before")
-    @model_validator_v1_args
-    def validate_authentication(
-        cls, values: t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]
-    ) -> t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]:
-        if "database" not in values and DBT_VERSION >= (1, 5):
-            path = values.get("path")
-            values["database"] = (
+    def validate_authentication(cls, data: t.Any) -> t.Any:
+        if not isinstance(data, dict):
+            return data
+
+        if "database" not in data and DBT_VERSION >= (1, 5):
+            path = data.get("path")
+            data["database"] = (
                 "memory"
                 if path is None or path == DUCKDB_IN_MEMORY
                 else Path(t.cast(str, path)).stem
             )
-        if "threads" in values and t.cast(int, values["threads"]) > 1:
-            logger.warning("DuckDB does not support concurrency - setting threads to 1.")
-        return values
+
+        if "threads" in data and t.cast(int, data["threads"]) > 1:
+            get_console().log_warning("DuckDB does not support concurrency - setting threads to 1.")
+
+        return data
 
     def default_incremental_strategy(self, kind: IncrementalKind) -> str:
         return "delete+insert"
@@ -226,7 +222,7 @@ class SnowflakeConfig(TargetConfig):
         token: OAuth authentication: The Snowflake OAuth 2.0 access token
     """
 
-    type: Literal["snowflake"] = "snowflake"
+    type: t.Literal["snowflake"] = "snowflake"
     account: str
     user: str
 
@@ -257,17 +253,16 @@ class SnowflakeConfig(TargetConfig):
     retry_all: bool = False
 
     @model_validator(mode="before")
-    @model_validator_v1_args
-    def validate_authentication(
-        cls, values: t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]
-    ) -> t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]:
-        if (
-            values.get("password")
-            or values.get("authenticator")
-            or values.get("private_key")
-            or values.get("private_key_path")
+    @classmethod
+    def validate_authentication(cls, data: t.Any) -> t.Any:
+        if not isinstance(data, dict) or (
+            data.get("password")
+            or data.get("authenticator")
+            or data.get("private_key")
+            or data.get("private_key_path")
         ):
-            return values
+            return data
+
         raise ConfigError("No supported Snowflake authentication method found in target profile.")
 
     def default_incremental_strategy(self, kind: IncrementalKind) -> str:
@@ -325,7 +320,7 @@ class PostgresConfig(TargetConfig):
         sslmode: SSL Mode used to connect to the database
     """
 
-    type: Literal["postgres"] = "postgres"
+    type: t.Literal["postgres"] = "postgres"
     host: str
     user: str
     password: str
@@ -339,14 +334,16 @@ class PostgresConfig(TargetConfig):
     sslmode: t.Optional[str] = None
 
     @model_validator(mode="before")
-    @model_validator_v1_args
-    def validate_database(
-        cls, values: t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]
-    ) -> t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]:
-        values["database"] = values.get("database") or values.get("dbname")
-        if not values["database"]:
+    @classmethod
+    def validate_database(cls, data: t.Any) -> t.Any:
+        if not isinstance(data, dict):
+            return data
+
+        data["database"] = data.get("database") or data.get("dbname")
+        if not data["database"]:
             raise ConfigError("Either database or dbname must be set")
-        return values
+
+        return data
 
     @field_validator("port")
     @classmethod
@@ -389,7 +386,7 @@ class RedshiftConfig(TargetConfig):
     """
 
     # TODO add other forms of authentication
-    type: Literal["redshift"] = "redshift"
+    type: t.Literal["redshift"] = "redshift"
     host: str
     user: str
     password: str
@@ -401,14 +398,16 @@ class RedshiftConfig(TargetConfig):
     sslmode: t.Optional[str] = None
 
     @model_validator(mode="before")
-    @model_validator_v1_args
-    def validate_database(
-        cls, values: t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]
-    ) -> t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]:
-        values["database"] = values.get("database") or values.get("dbname")
-        if not values["database"]:
+    @classmethod
+    def validate_database(cls, data: t.Any) -> t.Any:
+        if not isinstance(data, dict):
+            return data
+
+        data["database"] = data.get("database") or data.get("dbname")
+        if not data["database"]:
             raise ConfigError("Either database or dbname must be set")
-        return values
+
+        return data
 
     def default_incremental_strategy(self, kind: IncrementalKind) -> str:
         return "append"
@@ -454,11 +453,14 @@ class DatabricksConfig(TargetConfig):
         database: Name of the database. Not applicable for Databricks and ignored
     """
 
-    type: Literal["databricks"] = "databricks"
+    type: t.Literal["databricks"] = "databricks"
     host: str
     http_path: str
-    token: str
+    token: t.Optional[str] = None  # only required if auth_type is not set to 'oauth'
     database: t.Optional[str] = Field(alias="catalog")  # type: ignore
+    auth_type: t.Optional[str] = None
+    client_id: t.Optional[str] = None
+    client_secret: t.Optional[str] = None
 
     def default_incremental_strategy(self, kind: IncrementalKind) -> str:
         return "merge"
@@ -482,6 +484,9 @@ class DatabricksConfig(TargetConfig):
             access_token=self.token,
             concurrent_tasks=self.threads,
             catalog=self.database,
+            auth_type="databricks-oauth" if self.auth_type == "oauth" else self.auth_type,
+            oauth_client_id=self.client_id,
+            oauth_client_secret=self.client_secret,
             **kwargs,
         )
 
@@ -512,11 +517,12 @@ class BigQueryConfig(TargetConfig):
         maximum_bytes_billed: The maximum number of bytes to be billed for the underlying job
     """
 
-    type: Literal["bigquery"] = "bigquery"
+    type: t.Literal["bigquery"] = "bigquery"
     method: t.Optional[str] = BigQueryConnectionMethod.OAUTH
     dataset: t.Optional[str] = None
     project: t.Optional[str] = None
     execution_project: t.Optional[str] = None
+    quota_project: t.Optional[str] = None
     location: t.Optional[str] = None
     keyfile: t.Optional[str] = None
     keyfile_json: t.Optional[t.Dict[str, t.Any]] = None
@@ -539,17 +545,19 @@ class BigQueryConfig(TargetConfig):
     maximum_bytes_billed: t.Optional[int] = None
 
     @model_validator(mode="before")
-    @model_validator_v1_args
-    def validate_fields(
-        cls, values: t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]
-    ) -> t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]:
-        values["schema"] = values.get("schema") or values.get("dataset")
-        if not values["schema"]:
+    @classmethod
+    def validate_fields(cls, data: t.Any) -> t.Any:
+        if not isinstance(data, dict):
+            return data
+
+        data["schema"] = data.get("schema") or data.get("dataset")
+        if not data["schema"]:
             raise ConfigError("Either schema or dataset must be set")
-        values["database"] = values.get("database") or values.get("project")
-        if not values["database"]:
+        data["database"] = data.get("database") or data.get("project")
+        if not data["database"]:
             raise ConfigError("Either database or project must be set")
-        return values
+
+        return data
 
     def default_incremental_strategy(self, kind: IncrementalKind) -> str:
         return "merge"
@@ -577,6 +585,7 @@ class BigQueryConfig(TargetConfig):
             method=self.method,
             project=self.database,
             execution_project=self.execution_project,
+            quota_project=self.quota_project,
             location=self.location,
             concurrent_tasks=self.threads,
             keyfile=self.keyfile,
@@ -623,7 +632,7 @@ class MSSQLConfig(TargetConfig):
         client_secret: The client secret of the Azure Active Directory service principal, not used by SQLMesh
     """
 
-    type: Literal["sqlserver"] = "sqlserver"
+    type: t.Literal["sqlserver"] = "sqlserver"
     host: t.Optional[str] = None
     server: t.Optional[str] = None
     port: int = 1433
@@ -653,23 +662,24 @@ class MSSQLConfig(TargetConfig):
     client_secret: t.Optional[str] = None  # Azure Active Directory auth
 
     @model_validator(mode="before")
-    @model_validator_v1_args
-    def validate_alias_fields(
-        cls, values: t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]
-    ) -> t.Dict[str, t.Union[t.Tuple[str, ...], t.Optional[str], t.Dict[str, t.Any]]]:
-        values["host"] = values.get("host") or values.get("server")
-        if not values["host"]:
+    @classmethod
+    def validate_alias_fields(cls, data: t.Any) -> t.Any:
+        if not isinstance(data, dict):
+            return data
+
+        data["host"] = data.get("host") or data.get("server")
+        if not data["host"]:
             raise ConfigError("Either host or server must be set")
 
-        values["user"] = values.get("user") or values.get("username") or values.get("UID")
-        if not values["user"]:
+        data["user"] = data.get("user") or data.get("username") or data.get("UID")
+        if not data["user"]:
             raise ConfigError("One of user, username, or UID must be set")
 
-        values["password"] = values.get("password") or values.get("PWD")
-        if not values["password"]:
+        data["password"] = data.get("password") or data.get("PWD")
+        if not data["password"]:
             raise ConfigError("Either password or PWD must be set")
 
-        return values
+        return data
 
     @field_validator("authentication")
     @classmethod
@@ -689,7 +699,12 @@ class MSSQLConfig(TargetConfig):
 
     @classproperty
     def column_class(cls) -> t.Type[Column]:
-        from dbt.adapters.sqlserver.sql_server_column import SQLServerColumn
+        try:
+            # 1.8.0+
+            from dbt.adapters.sqlserver.sqlserver_column import SQLServerColumn
+        except ImportError:
+            # <1.8.0
+            from dbt.adapters.sqlserver.sql_server_column import SQLServerColumn  # type: ignore
 
         return SQLServerColumn
 
@@ -756,7 +771,7 @@ class TrinoConfig(TargetConfig):
         "oauth_console": TrinoAuthenticationMethod.OAUTH,
     }
 
-    type: Literal["trino"] = "trino"
+    type: t.Literal["trino"] = "trino"
     host: str
     database: str
     schema_: str = Field(alias="schema")
@@ -844,6 +859,182 @@ class TrinoConfig(TargetConfig):
         )
 
 
+class ClickhouseConfig(TargetConfig):
+    """
+    Project connection and operational configuration for the Clickhouse target
+
+    Args:
+      host: [localhost]
+      user: [default] # User for all database operations
+      password: [<empty string>] # Password for the user
+      secure: [False] # Use TLS (native protocol) or HTTPS (http protocol)
+      port: [8123]  # If not set, defaults to 8123, 8443 depending on the secure and driver settings
+      connect_timeout: [10] # Timeout in seconds to establish a connection to ClickHouse
+      send_receive_timeout: [300] # Timeout in seconds to receive data from the ClickHouse server
+      verify: [True] # Validate TLS certificate if using TLS/SSL
+      cluster: [<empty string>] # If set, certain DDL/table operations will be executed with the `ON CLUSTER` clause using this cluster.
+      custom_settings: [{}] # A dictionary/mapping of custom ClickHouse settings for the connection - default is empty.
+      schema: [default] # ClickHouse database for dbt models, not used by SQLMesh
+      driver: [http] # http or native.  If not set this will be autodetermined based on port setting, not used by SQLMesh
+      retries: [1] # Number of times to retry a "retriable" database exception (such as a 503 'Service Unavailable' error), not used by SQLMesh
+      compression: [<empty string>] # Use gzip compression if truthy (http), or compression type for a native connection, not used by SQLMesh
+      cluster_mode: [False] # Use specific settings designed to improve operation on Replicated databases (recommended for ClickHouse Cloud), not used by SQLMesh
+      use_lw_deletes: [False] # Use the strategy `delete+insert` as the default incremental strategy, not used by SQLMesh
+      check_exchange: [True] # Validate that clickhouse support the atomic EXCHANGE TABLES command. Not used by SQLMesh.
+      local_suffix: [_local] # Table suffix of local tables on shards for distributed materializations, not used by SQLMesh
+      local_db_prefix: [<empty string>] # Database prefix of local tables on shards for distributed materializations, not used by SQLMesh
+      allow_automatic_deduplication: [False] # Enable ClickHouse automatic deduplication for Replicated tables, not used by SQLMesh
+      tcp_keepalive: [False] # Native client only, specify TCP keepalive configuration. Specify custom keepalive settings as [idle_time_sec, interval_sec, probes], not used by SQLMesh
+      sync_request_timeout: [5] # Timeout for server ping, not used by SQLMesh
+      compress_block_size: [1048576] # Compression block size if compression is enabled, not used by SQLMesh
+    """
+
+    host: str = "localhost"
+    user: str = Field(default="default", alias="username")
+    password: str = ""
+    port: t.Optional[int] = None
+    cluster: t.Optional[str] = None
+    schema_: str = Field(default="default", alias="schema")
+    connect_timeout: int = 10
+    send_receive_timeout: int = 300
+    verify: bool = True
+    compression: str = ""
+    custom_settings: t.Optional[t.Dict[str, t.Any]] = None
+
+    # Not used by SQLMesh
+    driver: t.Optional[str] = None
+    secure: bool = False
+    retries: int = 1
+    database_engine: t.Optional[str] = None
+    cluster_mode: bool = False
+    sync_request_timeout: int = 5
+    compress_block_size: int = 1048576
+    check_exchange: bool = True
+    use_lw_deletes: bool = False
+    allow_automatic_deduplication: bool = False
+    tcp_keepalive: t.Union[bool, t.Tuple[int, ...], t.List[int]] = False
+    database: str = ""
+    local_suffix: str = "local"
+    local_db_prefix: str = ""
+
+    type: t.Literal["clickhouse"] = "clickhouse"
+
+    def default_incremental_strategy(self, kind: IncrementalKind) -> str:
+        # dbt-clickhouse name for temp table swap. That is sqlmesh's default
+        #   strategy so doesn't require special handling during conversion.
+        return "legacy"
+
+    @classproperty
+    def relation_class(cls) -> t.Type[BaseRelation]:
+        from dbt.adapters.clickhouse.relation import ClickHouseRelation
+
+        return ClickHouseRelation
+
+    @classproperty
+    def column_class(cls) -> t.Type[Column]:
+        from dbt.adapters.clickhouse.column import ClickHouseColumn
+
+        return ClickHouseColumn
+
+    def to_sqlmesh(self, **kwargs: t.Any) -> ConnectionConfig:
+        return ClickhouseConnectionConfig(
+            host=self.host,
+            username=self.user,
+            password=self.password,
+            port=self.port,
+            cluster=self.cluster,
+            connect_timeout=self.connect_timeout,
+            send_receive_timeout=self.send_receive_timeout,
+            verify=self.verify,
+            compression_method=self.compression,
+            connection_settings=self.custom_settings,
+            **kwargs,
+        )
+
+
+class AthenaConfig(TargetConfig):
+    """
+    Project connection and operational configuration for the Athena target.
+
+    Args:
+        s3_staging_dir: S3 location to store Athena query results and metadata
+        s3_data_dir: Prefix for storing tables, if different from the connection's s3_staging_dir
+        s3_data_naming: How to generate table paths in s3_data_dir
+        s3_tmp_table_dir: Prefix for storing temporary tables, if different from the connection's s3_data_dir
+        region_name: AWS region of your Athena instance
+        schema: Specify the schema (Athena database) to build models into (lowercase only)
+        database: Specify the database (Data catalog) to build models into (lowercase only)
+        poll_interval: Interval in seconds to use for polling the status of query results in Athena
+        debug_query_state: Flag if debug message with Athena query state is needed
+        aws_access_key_id: Access key ID of the user performing requests
+        aws_secret_access_key: Secret access key of the user performing requests
+        aws_profile_name: Profile to use from your AWS shared credentials file
+        work_group: Identifier of Athena workgroup
+        skip_workgroup_check: Indicates if the WorkGroup check (additional AWS call) can be skipped
+        num_retries: Number of times to retry a failing query
+        num_boto3_retries: Number of times to retry boto3 requests (e.g. deleting S3 files for materialized tables)
+        num_iceberg_retries: Number of times to retry iceberg commit queries to fix ICEBERG_COMMIT_ERROR
+        spark_work_group: Identifier of Athena Spark workgroup for running Python models
+        seed_s3_upload_args: Dictionary containing boto3 ExtraArgs when uploading to S3
+        lf_tags_database: Default LF tags for new database if it's created by dbt
+    """
+
+    type: t.Literal["athena"] = "athena"
+    threads: int = 4
+
+    s3_staging_dir: t.Optional[str] = None
+    s3_data_dir: t.Optional[str] = None
+    s3_data_naming: t.Optional[str] = None
+    s3_tmp_table_dir: t.Optional[str] = None
+    poll_interval: t.Optional[int] = None
+    debug_query_state: bool = False
+    work_group: t.Optional[str] = None
+    skip_workgroup_check: t.Optional[bool] = None
+    spark_work_group: t.Optional[str] = None
+
+    aws_access_key_id: t.Optional[str] = None
+    aws_secret_access_key: t.Optional[str] = None
+    aws_profile_name: t.Optional[str] = None
+    region_name: t.Optional[str] = None
+
+    num_retries: t.Optional[int] = None
+    num_boto3_retries: t.Optional[int] = None
+    num_iceberg_retries: t.Optional[int] = None
+
+    seed_s3_upload_args: t.Dict[str, str] = {}
+    lf_tags_database: t.Dict[str, str] = {}
+
+    @classproperty
+    def relation_class(cls) -> t.Type[BaseRelation]:
+        from dbt.adapters.athena.relation import AthenaRelation
+
+        return AthenaRelation
+
+    @classproperty
+    def column_class(cls) -> t.Type[Column]:
+        from dbt.adapters.athena.column import AthenaColumn
+
+        return AthenaColumn
+
+    def default_incremental_strategy(self, kind: IncrementalKind) -> str:
+        return "insert_overwrite"
+
+    def to_sqlmesh(self, **kwargs: t.Any) -> ConnectionConfig:
+        return AthenaConnectionConfig(
+            type="athena",
+            aws_access_key_id=self.aws_access_key_id,
+            aws_secret_access_key=self.aws_secret_access_key,
+            region_name=self.region_name,
+            work_group=self.work_group,
+            s3_staging_dir=self.s3_staging_dir,
+            s3_warehouse_location=self.s3_data_dir,
+            schema_name=self.schema_,
+            catalog_name=self.database,
+            concurrent_tasks=self.threads,
+            **kwargs,
+        )
+
+
 TARGET_TYPE_TO_CONFIG_CLASS: t.Dict[str, t.Type[TargetConfig]] = {
     "databricks": DatabricksConfig,
     "duckdb": DuckDbConfig,
@@ -852,5 +1043,8 @@ TARGET_TYPE_TO_CONFIG_CLASS: t.Dict[str, t.Type[TargetConfig]] = {
     "snowflake": SnowflakeConfig,
     "bigquery": BigQueryConfig,
     "sqlserver": MSSQLConfig,
+    "tsql": MSSQLConfig,
     "trino": TrinoConfig,
+    "athena": AthenaConfig,
+    "clickhouse": ClickhouseConfig,
 }
