@@ -3,11 +3,17 @@ from __future__ import annotations
 import functools
 import logging
 import typing as t
-from argparse import Namespace
+from argparse import Namespace, SUPPRESS
 from collections import defaultdict
+from copy import deepcopy
 
 from hyperscript import h
-from IPython.core.display import display
+
+try:
+    from IPython.core.display import display  # type: ignore
+except ImportError:
+    from IPython.display import display
+
 from IPython.core.magic import (
     Magics,
     cell_magic,
@@ -23,7 +29,7 @@ from sqlmesh.cli.example_project import ProjectTemplate, init_example_project
 from sqlmesh.core import analytics
 from sqlmesh.core import constants as c
 from sqlmesh.core.config import load_configs
-from sqlmesh.core.console import get_console
+from sqlmesh.core.console import create_console, set_console, configure_console
 from sqlmesh.core.context import Context
 from sqlmesh.core.dialect import format_model_expressions, parse
 from sqlmesh.core.model import load_sql_based_model
@@ -52,7 +58,9 @@ def pass_sqlmesh_context(func: t.Callable) -> t.Callable:
                 f"Context must be defined and initialized with one of these names: {', '.join(CONTEXT_VARIABLE_NAMES)}"
             )
         old_console = context.console
-        context.console = get_console(display=self.display)
+        new_console = create_console(display=self.display)
+        context.console = new_console
+        set_console(new_console)
         context.refresh()
 
         magic_name = func.__name__
@@ -60,8 +68,19 @@ def pass_sqlmesh_context(func: t.Callable) -> t.Callable:
         if bound_method:
             args_split = arg_split(args[0])
             parser = bound_method.parser
-            # Calling the private method to bypass setting of defaults.
-            parsed_args, _ = parser._parse_known_args(args_split, Namespace())
+
+            original_parser_actions = deepcopy(parser._actions)
+            original_parser_defaults = parser._defaults
+
+            # Temporarily supress default values, otherwise any missing arg would be set and affect analytics
+            parser._defaults = {}
+            for action in parser._actions:
+                action.default = SUPPRESS
+
+            parsed_args, _ = parser.parse_known_args(args_split, Namespace())
+
+            parser._actions = original_parser_actions
+            parser._defaults = original_parser_defaults
 
             command_args = {k for k, v in parsed_args.__dict__.items() if v is not None}
             analytics.collector.on_magic_command(command_name=magic_name, command_args=command_args)
@@ -69,6 +88,7 @@ def pass_sqlmesh_context(func: t.Callable) -> t.Callable:
         func(self, context, *args, **kwargs)
 
         context.console = old_console
+        set_console(old_console)
 
     return wrapper
 
@@ -116,9 +136,8 @@ class SQLMeshMagics(Magics):
         args = parse_argstring(self.context, line)
         configs = load_configs(args.config, Context.CONFIG_TYPE, args.paths)
         log_limit = list(configs.values())[0].log_limit
-        configure_logging(
-            args.debug, args.ignore_warnings, log_limit=log_limit, log_file_dir=args.log_file_dir
-        )
+        configure_logging(args.debug, log_limit=log_limit, log_file_dir=args.log_file_dir)
+        configure_console(ignore_warnings=args.ignore_warnings)
         try:
             context = Context(paths=args.paths, config=configs, gateway=args.gateway)
             self._shell.user_ns["context"] = context
@@ -141,6 +160,11 @@ class SQLMeshMagics(Magics):
         type=str,
         help="Project template. Supported values: airflow, dbt, default, empty.",
     )
+    @argument(
+        "--dlt-pipeline",
+        type=str,
+        help="DLT pipeline for which to generate a SQLMesh project. Use alongside template: dlt",
+    )
     @line_magic
     def init(self, line: str) -> None:
         """Creates a SQLMesh project scaffold with a default SQL dialect."""
@@ -151,7 +175,7 @@ class SQLMeshMagics(Magics):
             )
         except ValueError:
             raise MagicError(f"Invalid project template '{args.template}'")
-        init_example_project(args.path, args.sql_dialect, project_template)
+        init_example_project(args.path, args.sql_dialect, project_template, args.dlt_pipeline)
         html = str(
             h(
                 "div",
@@ -188,7 +212,7 @@ class SQLMeshMagics(Magics):
                 path=model._path,
                 dialect=config.dialect,
                 time_column_format=config.time_column_format,
-                physical_schema_override=context.config.physical_schema_override,
+                physical_schema_mapping=context.config.physical_schema_mapping,
                 default_catalog=context.default_catalog,
             )
 
@@ -199,7 +223,10 @@ class SQLMeshMagics(Magics):
                 expressions = parse(file.read(), default_dialect=config.dialect)
 
         formatted = format_model_expressions(
-            expressions, model.dialect, **config.format.generator_options
+            expressions,
+            model.dialect,
+            rewrite_casts=not config.format.no_rewrite_casts,
+            **config.format.generator_options,
         )
 
         self._shell.set_next_input(
@@ -325,8 +352,14 @@ class SQLMeshMagics(Magics):
     )
     @argument(
         "--skip-backfill",
+        "--dry-run",
         action="store_true",
-        help="Skip the backfill step.",
+        help="Skip the backfill step and only create a virtual update for the plan.",
+    )
+    @argument(
+        "--empty-backfill",
+        action="store_true",
+        help="Produce empty backfill. Like --skip-backfill no models will be backfilled, unlike --skip-backfill missing intervals will be recorded as if they were backfilled.",
     )
     @argument(
         "--forward-only",
@@ -373,7 +406,7 @@ class SQLMeshMagics(Magics):
         "--backfill-model",
         type=str,
         nargs="*",
-        help="Backfill only the models whose names match the expression. This is supported only when targeting a development environment.",
+        help="Backfill only the models whose names match the expression.",
     )
     @argument(
         "--no-diff",
@@ -392,6 +425,11 @@ class SQLMeshMagics(Magics):
         help="Enable preview for forward-only models when targeting a development environment.",
         default=None,
     )
+    @argument(
+        "--diff-rendered",
+        action="store_true",
+        help="Output text differences for the rendered versions of the models and standalone audits",
+    )
     @line_magic
     @pass_sqlmesh_context
     def plan(self, context: Context, line: str) -> None:
@@ -409,6 +447,7 @@ class SQLMeshMagics(Magics):
             backfill_models=args.backfill_model,
             no_gaps=args.no_gaps,
             skip_backfill=args.skip_backfill,
+            empty_backfill=args.empty_backfill,
             forward_only=args.forward_only,
             no_prompts=args.no_prompts,
             auto_apply=args.auto_apply,
@@ -419,6 +458,7 @@ class SQLMeshMagics(Magics):
             no_diff=args.no_diff,
             run=args.run,
             enable_preview=args.enable_preview,
+            diff_rendered=args.diff_rendered,
         )
 
     @magic_arguments()
@@ -436,20 +476,39 @@ class SQLMeshMagics(Magics):
         action="store_true",
         help="Run for all missing intervals, ignoring individual cron schedules.",
     )
+    @argument(
+        "--select-model",
+        type=str,
+        nargs="*",
+        help="Select specific models to run. Note: this always includes upstream dependencies.",
+    )
+    @argument(
+        "--exit-on-env-update",
+        type=int,
+        help="If set, the command will exit with the specified code if the run is interrupted by an update to the target environment.",
+    )
+    @argument(
+        "--no-auto-upstream",
+        action="store_true",
+        help="Do not automatically include upstream models. Only applicable when --select-model is used. Note: this may result in missing / invalid data for the selected models.",
+    )
     @line_magic
     @pass_sqlmesh_context
     def run_dag(self, context: Context, line: str) -> None:
         """Evaluate the DAG of models using the built-in scheduler."""
         args = parse_argstring(self.run_dag, line)
 
-        success = context.run(
+        completion_status = context.run(
             args.environment,
             start=args.start,
             end=args.end,
             skip_janitor=args.skip_janitor,
             ignore_cron=args.ignore_cron,
+            select_models=args.select_model,
+            exit_on_env_update=args.exit_on_env_update,
+            no_auto_upstream=args.no_auto_upstream,
         )
-        if not success:
+        if completion_status.is_failure:
             raise SQLMeshError("Error Running DAG. Check logs for details.")
 
     @magic_arguments()
@@ -558,11 +617,17 @@ class SQLMeshMagics(Magics):
         context.console.log_success("Migration complete")
 
     @magic_arguments()
+    @argument(
+        "--strict",
+        action="store_true",
+        help="Raise an error if the external model is missing in the database",
+    )
     @line_magic
     @pass_sqlmesh_context
     def create_external_models(self, context: Context, line: str) -> None:
         """Create a schema file containing external model schemas."""
-        context.create_external_models()
+        args = parse_argstring(self.create_external_models, line)
+        context.create_external_models(strict=args.strict)
 
     @magic_arguments()
     @argument(
@@ -577,6 +642,12 @@ class SQLMeshMagics(Magics):
         nargs="*",
         help="The column to join on. Can be specified multiple times. The model grain will be used if not specified.",
     )
+    @argument(
+        "--skip-columns",
+        type=str,
+        nargs="*",
+        help="The column(s) to skip when comparing the source and target table.",
+    )
     @argument(
         "--model",
         type=str,
@@ -604,6 +675,11 @@ class SQLMeshMagics(Magics):
         default=3,
         help="The number of decimal places to keep when comparing floating point columns. Default: 3",
     )
+    @argument(
+        "--skip-grain-check",
+        action="store_true",
+        help="Disable the check for a primary key (grain) that is missing or is not unique.",
+    )
     @line_magic
     @pass_sqlmesh_context
     def table_diff(self, context: Context, line: str) -> None:
@@ -617,11 +693,13 @@ class SQLMeshMagics(Magics):
             source=source,
             target=target,
             on=args.on,
+            skip_columns=args.skip_columns,
             model_or_snapshot=args.model,
             where=args.where,
             limit=args.limit,
             show_sample=args.show_sample,
             decimals=args.decimals,
+            skip_grain_check=args.skip_grain_check,
         )
 
     @magic_arguments()
@@ -643,6 +721,42 @@ class SQLMeshMagics(Magics):
         args = parse_argstring(self.table_name, line)
         context.console.log_status_update(context.table_name(args.model_name, args.dev))
 
+    @magic_arguments()
+    @argument(
+        "pipeline",
+        nargs="?",
+        type=str,
+        help="The dlt pipeline to attach for this SQLMesh project.",
+    )
+    @argument(
+        "--table",
+        "-t",
+        type=str,
+        nargs="*",
+        help="The specific dlt tables to refresh in the SQLMesh models.",
+    )
+    @argument(
+        "--force",
+        "-f",
+        action="store_true",
+        help="If set, existing models are overwritten with the new DLT tables.",
+    )
+    @line_magic
+    @pass_sqlmesh_context
+    def dlt_refresh(self, context: Context, line: str) -> None:
+        """Attaches to a DLT pipeline with the option to update specific or all missing tables in the SQLMesh project."""
+        from sqlmesh.integrations.dlt import generate_dlt_models
+
+        args = parse_argstring(self.dlt_refresh, line)
+        sqlmesh_models = generate_dlt_models(
+            context, args.pipeline, list(args.table or []), args.force
+        )
+        if sqlmesh_models:
+            model_names = "\n".join([f"- {model_name}" for model_name in sqlmesh_models])
+            context.console.log_success(f"Updated SQLMesh project with models:\n{model_names}")
+        else:
+            context.console.log_success("All SQLMesh models are up to date.")
+
     @magic_arguments()
     @argument(
         "--read",
@@ -683,6 +797,12 @@ class SQLMeshMagics(Magics):
         help="Whether or not to append a newline to the end of the file.",
         default=None,
     )
+    @argument(
+        "--no-rewrite-casts",
+        action="store_true",
+        help="Whether or not to preserve the existing casts, without rewriting them to use the :: syntax.",
+        default=None,
+    )
     @argument(
         "--normalize",
         action="store_true",
@@ -715,12 +835,21 @@ class SQLMeshMagics(Magics):
         type=int,
         help="The max number of characters in a segment before creating new lines in pretty mode.",
     )
+    @argument(
+        "--check",
+        action="store_true",
+        help="Whether or not to check formatting (but not actually format anything).",
+        default=None,
+    )
     @line_magic
     @pass_sqlmesh_context
-    def format(self, context: Context, line: str) -> None:
-        """Format all SQL models."""
-        args = parse_argstring(self.format, line)
-        context.format(**{k: v for k, v in vars(args).items() if v is not None})
+    def format(self, context: Context, line: str) -> bool:
+        """Format all SQL models and audits."""
+        format_opts = vars(parse_argstring(self.format, line))
+        if format_opts.pop("no_rewrite_casts", None):
+            format_opts["rewrite_casts"] = False
+
+        return context.format(**{k: v for k, v in format_opts.items() if v is not None})
 
     @magic_arguments()
     @argument("environment", type=str, help="The environment to diff local state against.")
@@ -740,6 +869,19 @@ class SQLMeshMagics(Magics):
         args = parse_argstring(self.invalidate, line)
         context.invalidate_environment(args.environment)
 
+    @magic_arguments()
+    @argument(
+        "--ignore-ttl",
+        action="store_true",
+        help="Cleanup snapshots that are not referenced in any environment, regardless of when they're set to expire",
+    )
+    @line_magic
+    @pass_sqlmesh_context
+    def janitor(self, context: Context, line: str) -> None:
+        """Run the janitor process to clean up old environments and expired snapshots."""
+        args = parse_argstring(self.janitor, line)
+        context.run_janitor(ignore_ttl=args.ignore_ttl)
+
     @magic_arguments()
     @argument("model", type=str)
     @argument(
@@ -747,7 +889,7 @@ class SQLMeshMagics(Magics):
         "-q",
         type=str,
         nargs="+",
-        required=True,
+        default=[],
         help="Queries that will be used to generate data for the model's dependencies.",
     )
     @argument(
@@ -849,12 +991,13 @@ class SQLMeshMagics(Magics):
         help="Skip the connection test.",
         default=False,
     )
+    @argument("--verbose", "-v", action="store_true", help="Verbose output.")
     @line_magic
     @pass_sqlmesh_context
     def info(self, context: Context, line: str) -> None:
         """Display SQLMesh project information."""
         args = parse_argstring(self.info, line)
-        context.print_info(skip_connection=args.skip_connection)
+        context.print_info(skip_connection=args.skip_connection, verbose=args.verbose)
 
     @magic_arguments()
     @line_magic
@@ -871,6 +1014,13 @@ class SQLMeshMagics(Magics):
         context.clear_caches()
         context.console.log_success("SQLMesh cache and build artifacts cleared")
 
+    @magic_arguments()
+    @line_magic
+    @pass_sqlmesh_context
+    def environments(self, context: Context, line: str) -> None:
+        """Prints the list of SQLMesh environments with its expiry datetime."""
+        context.print_environment_names()
+
 
 def register_magics() -> None:
     try:
