@@ -11,6 +11,7 @@ from sqlmesh.core.snapshot import (
     SnapshotIdLike,
     SnapshotInfoLike,
     SnapshotTableCleanupTask,
+    SnapshotNameVersion,
 )
 from sqlmesh.core.snapshot.definition import Interval, SnapshotIntervals
 from sqlmesh.core.state_sync import StateSync, Versions
@@ -67,39 +68,31 @@ class HttpStateSync(StateSync):
         """
         return self._client.get_environments()
 
-    def max_interval_end_for_environment(
-        self, environment: str, ensure_finalized_snapshots: bool = False
-    ) -> t.Optional[int]:
-        """Returns the max interval end for the given environment.
-
-        Args:
-            environment: The environment.
-            ensure_finalized_snapshots: Whether to use snapshots from the latest finalized environment state,
-                or to use whatever snapshots are in the current environment state even if the environment is not finalized.
-
-        Returns:
-            A timestamp or None if no interval or environment exists.
-        """
-        return self._client.max_interval_end_for_environment(
-            environment, ensure_finalized_snapshots
+    def get_environments_summary(self) -> t.Dict[str, int]:
+        """Fetches all environment names along with expiry datetime."""
+        raise NotImplementedError(
+            "get_environments_summary method is not implemented for the Airflow state sync."
         )
 
-    def greatest_common_interval_end(
-        self, environment: str, models: t.Set[str], ensure_finalized_snapshots: bool = False
-    ) -> t.Optional[int]:
-        """Returns the greatest common interval end for given models in the target environment.
+    def max_interval_end_per_model(
+        self,
+        environment: str,
+        models: t.Optional[t.Set[str]] = None,
+        ensure_finalized_snapshots: bool = False,
+    ) -> t.Dict[str, int]:
+        """Returns the max interval end per model for the given environment.
 
         Args:
-            environment: The environment.
-            models: The model FQNs to select intervals from.
+            environment: The target environment.
+            models: The models to get the max interval end for. If None, all models are considered.
             ensure_finalized_snapshots: Whether to use snapshots from the latest finalized environment state,
                 or to use whatever snapshots are in the current environment state even if the environment is not finalized.
 
         Returns:
-            A timestamp or None if no interval or environment exists.
+            A dictionary of model FQNs to their respective interval ends in milliseconds since epoch.
         """
-        return self._client.greatest_common_interval_end(
-            environment, models, ensure_finalized_snapshots
+        return self._client.max_interval_end_per_model(
+            environment, models=models, ensure_finalized_snapshots=ensure_finalized_snapshots
         )
 
     def get_snapshots(
@@ -176,7 +169,9 @@ class HttpStateSync(StateSync):
         """
         raise NotImplementedError("Deleting snapshots is not supported by the Airflow state sync.")
 
-    def delete_expired_snapshots(self) -> t.List[SnapshotTableCleanupTask]:
+    def delete_expired_snapshots(
+        self, ignore_ttl: bool = False
+    ) -> t.List[SnapshotTableCleanupTask]:
         """Removes expired snapshots.
 
         Expired snapshots are snapshots that have exceeded their time-to-live
@@ -217,10 +212,10 @@ class HttpStateSync(StateSync):
         """
         raise NotImplementedError("Adding intervals is not supported by the Airflow state sync.")
 
-    def _add_snapshot_intervals(self, snapshot_intervals: SnapshotIntervals) -> None:
+    def add_snapshots_intervals(self, snapshots_intervals: t.Sequence[SnapshotIntervals]) -> None:
         raise NotImplementedError("Adding intervals is not supported by the Airflow state sync.")
 
-    def remove_interval(
+    def remove_intervals(
         self,
         snapshot_intervals: t.Sequence[t.Tuple[SnapshotInfoLike, Interval]],
         remove_shared_versions: bool = False,
@@ -247,9 +242,8 @@ class HttpStateSync(StateSync):
         Returns:
             The updated snapshots.
         """
-        raise NotImplementedError(
-            "Refreshing snapshot intervals is not supported by the Airflow state sync."
-        )
+        logger.warning("Refreshing snapshot intervals is not supported by the Airflow state sync")
+        return list(snapshots)
 
     def promote(
         self,
@@ -322,6 +316,13 @@ class HttpStateSync(StateSync):
             "Compacting intervals is not supported by the Airflow state sync."
         )
 
+    def update_auto_restatements(
+        self, next_auto_restatement_ts: t.Dict[SnapshotNameVersion, t.Optional[int]]
+    ) -> None:
+        raise NotImplementedError(
+            "Updating auto restatements is not supported by the Airflow state sync."
+        )
+
     def migrate(
         self,
         default_catalog: t.Optional[str],
