@@ -65,7 +65,7 @@ def random_id(short: bool = False) -> str:
     return uuid.uuid4().hex
 
 
-class UniqueKeyDict(dict, t.Mapping[KEY, VALUE]):
+class UniqueKeyDict(t.Dict[KEY, VALUE]):
     """Dict that raises when a duplicate key is set."""
 
     def __init__(self, name: str, *args: t.Dict[KEY, VALUE], **kwargs: VALUE) -> None:
@@ -81,7 +81,10 @@ class UniqueKeyDict(dict, t.Mapping[KEY, VALUE]):
 
 
 class AttributeDict(dict, t.Mapping[KEY, VALUE]):
-    __getattr__ = dict.get
+    def __getattr__(self, key: t.Any) -> t.Optional[VALUE]:
+        if key.startswith("__") and not hasattr(self, key):
+            raise AttributeError
+        return self.get(key)
 
     def set(self, field: str, value: t.Any) -> str:
         self[field] = value
@@ -100,6 +103,9 @@ class AttributeDict(dict, t.Mapping[KEY, VALUE]):
         # Return an empty string, so that this method can be used within Jinja
         return ""
 
+    def __getstate__(self) -> t.Optional[t.Dict[t.Any, t.Any]]:
+        return None
+
 
 class registry_decorator:
     """A decorator that registers itself."""
@@ -169,9 +175,7 @@ def sys_path(*paths: Path) -> t.Iterator[None]:
 
 def format_exception(exception: BaseException) -> t.List[str]:
     if sys.version_info < (3, 10):
-        return traceback.format_exception(
-            type(exception), exception, exception.__traceback__
-        )  # type: ignore
+        return traceback.format_exception(type(exception), exception, exception.__traceback__)  # type: ignore
     else:
         return traceback.format_exception(exception)  # type: ignore
 
@@ -302,7 +306,7 @@ def groupby(
 
 
 def columns_to_types_to_struct(
-    columns_to_types: t.Union[t.Dict[str, exp.DataType], t.Dict[str, str]]
+    columns_to_types: t.Union[t.Dict[str, exp.DataType], t.Dict[str, str]],
 ) -> exp.DataType:
     """
     Converts a dict of column names to types to a struct.
