@@ -17,11 +17,14 @@ from pytest_mock.plugin import MockerFixture
 from sqlglot import exp, maybe_parse, parse_one
 from sqlglot.dialects.dialect import DialectType
 from sqlglot.helper import ensure_list
+from sqlglot.optimizer.normalize_identifiers import normalize_identifiers
 
-from sqlmesh.core.config import DuckDBConnectionConfig
+from sqlmesh.core.config import BaseDuckDBConnectionConfig
 from sqlmesh.core.context import Context
-from sqlmesh.core.engine_adapter import SparkEngineAdapter
+from sqlmesh.core.engine_adapter import MSSQLEngineAdapter, SparkEngineAdapter
 from sqlmesh.core.engine_adapter.base import EngineAdapter
+from sqlmesh.core.environment import EnvironmentNamingInfo
+from sqlmesh.core import lineage
 from sqlmesh.core.macros import macro
 from sqlmesh.core.model import IncrementalByTimeRangeKind, SqlModel, model
 from sqlmesh.core.model.kind import OnDestructiveChange
@@ -32,9 +35,11 @@ from sqlmesh.core.snapshot import (
     SnapshotChangeCategory,
     SnapshotDataVersion,
     SnapshotFingerprint,
+    DeployabilityIndex,
 )
 from sqlmesh.utils import random_id
 from sqlmesh.utils.date import TimeLike, to_date
+from sqlmesh.core.engine_adapter.shared import CatalogSupport
 
 pytest_plugins = ["tests.common_fixtures"]
 
@@ -111,12 +116,13 @@ class DuckDBMetadata:
 
 
 class SushiDataValidator:
-    def __init__(self, engine_adapter: EngineAdapter):
+    def __init__(self, engine_adapter: EngineAdapter, sushi_schema_name: str):
         self.engine_adapter = engine_adapter
+        self.sushi_schema_name = sushi_schema_name
 
     @classmethod
-    def from_context(cls, context: Context):
-        return cls(engine_adapter=context.engine_adapter)
+    def from_context(cls, context: Context, sushi_schema_name: str = "sushi"):
+        return cls(engine_adapter=context.engine_adapter, sushi_schema_name=sushi_schema_name)
 
     def validate(
         self,
@@ -126,13 +132,37 @@ class SushiDataValidator:
         *,
         env_name: t.Optional[str] = None,
         dialect: t.Optional[str] = None,
+        environment_naming_info: t.Optional[EnvironmentNamingInfo] = None,
     ) -> t.Dict[t.Any, t.Any]:
+        if (
+            env_name
+            and dialect
+            and environment_naming_info
+            and environment_naming_info.normalize_name
+        ):
+            # if the environment_naming_info was configured to normalize names, then Snapshot.qualified_view_name.table_for_enviromnent()
+            # returns schemas that contain the environment_name normalised for that engine
+            #
+            # in practice, this means "test_prod" becomes "TEST_PROD" on some engines so the final views are named like:
+            #
+            # "sushi__TEST_PROD"."waiter_as_customer_by_day"
+            #
+            # instead of:
+            #
+            # "sushi__test_prod"."waiter_as_customer_by_day"
+            #
+            # this matters for the reading the data back below to validate it
+            env_name = normalize_identifiers(env_name, dialect=dialect).name
+
         """
         Both start and end are inclusive.
         """
-        if model_name == "sushi.customer_revenue_lifetime":
+        if model_name in (
+            f"{self.sushi_schema_name}.customer_revenue_lifetime",
+            "sushi.customer_revenue_lifetime",
+        ):
             env_name = f"__{env_name}" if env_name else ""
-            full_table_path = f"sushi{env_name}.customer_revenue_lifetime"
+            full_table_path = f"{self.sushi_schema_name}{env_name}.customer_revenue_lifetime"
             query = f"SELECT event_date, count(*) AS the_count FROM {full_table_path} group by event_date order by 2 desc, 1 desc"
             results = self.engine_adapter.fetchdf(
                 parse_one(query), quote_identifiers=True
@@ -145,8 +175,8 @@ class SushiDataValidator:
             expected_dates = [
                 pd.to_datetime(end_date - datetime.timedelta(days=x)) for x in range(num_days_diff)
             ]
-            # all engines but duckdb fetch dates as datetime.date objects
-            if dialect and dialect != "duckdb":
+            # all engines but duckdb and clickhouse fetch dates as datetime.date objects
+            if dialect and dialect not in ("duckdb", "clickhouse"):
                 expected_dates = [x.date() for x in expected_dates]  # type: ignore
             assert list(results["event_date"].values()) == expected_dates
 
@@ -156,7 +186,7 @@ class SushiDataValidator:
 
 
 def pytest_collection_modifyitems(items, *args, **kwargs):
-    test_type_markers = {"fast", "slow", "docker", "remote"}
+    test_type_markers = {"fast", "slow", "docker", "remote", "isolated"}
     for item in items:
         for marker in item.iter_markers():
             if marker.name in test_type_markers:
@@ -189,7 +219,7 @@ def rescope_global_models(request):
 
 @pytest.fixture(scope="function", autouse=True)
 def rescope_duckdb_classvar(request):
-    DuckDBConnectionConfig._data_file_to_adapter = {}
+    BaseDuckDBConnectionConfig._data_file_to_adapter = {}
     yield
 
 
@@ -199,6 +229,19 @@ def rescope_log_handlers():
     yield
 
 
+@pytest.fixture(scope="function", autouse=True)
+def rescope_lineage_cache(request):
+    lineage.CACHE.clear()
+    yield
+
+
+@pytest.fixture(autouse=True)
+def reset_console():
+    from sqlmesh.core.console import set_console, NoopConsole
+
+    set_console(NoopConsole())
+
+
 @pytest.fixture
 def duck_conn() -> duckdb.DuckDBPyConnection:
     return duckdb.connect()
@@ -206,11 +249,17 @@ def duck_conn() -> duckdb.DuckDBPyConnection:
 
 def push_plan(context: Context, plan: Plan) -> None:
     plan_evaluator = BuiltInPlanEvaluator(
-        context.state_sync, context.snapshot_evaluator, context.default_catalog
+        context.state_sync,
+        context.snapshot_evaluator,
+        context.create_scheduler,
+        context.default_catalog,
+    )
+    deployability_index = DeployabilityIndex.create(context.snapshots.values())
+    plan_evaluator._push(plan.to_evaluatable(), plan.snapshots, deployability_index)
+    promotion_result = plan_evaluator._promote(plan.to_evaluatable(), plan.snapshots)
+    plan_evaluator._update_views(
+        plan.to_evaluatable(), plan.snapshots, promotion_result, deployability_index
     )
-    plan_evaluator._push(plan)
-    promotion_result = plan_evaluator._promote(plan)
-    plan_evaluator._update_views(plan, promotion_result)
 
 
 @pytest.fixture()
@@ -329,8 +378,8 @@ def make_snapshot() -> t.Callable:
 def make_snapshot_on_destructive_change(make_snapshot: t.Callable) -> t.Callable:
     def _make_function(
         name: str = "a",
-        old_query: str = "select '1' as one, '2022-01-01' ds",
-        new_query: str = "select 1 as one, '2022-01-01' ds",
+        old_query: str = "select '1' as one, '2' as two, '2022-01-01' ds",
+        new_query: str = "select 1 as one, 2 as two, '2022-01-01' ds",
         on_destructive_change: OnDestructiveChange = OnDestructiveChange.ERROR,
     ) -> t.Tuple[Snapshot, Snapshot]:
         snapshot_old = make_snapshot(
@@ -362,6 +411,7 @@ def make_snapshot_on_destructive_change(make_snapshot: t.Callable) -> t.Callable
                 ),
                 version="test_version",
                 change_category=SnapshotChangeCategory.FORWARD_ONLY,
+                dev_table_suffix="dev",
             ),
         )
 
@@ -388,7 +438,11 @@ def sushi_fixed_date_data_validator(sushi_context_fixed_date: Context) -> SushiD
 @pytest.fixture
 def make_mocked_engine_adapter(mocker: MockerFixture) -> t.Callable:
     def _make_function(
-        klass: t.Type[T], dialect: t.Optional[str] = None, register_comments: bool = True
+        klass: t.Type[T],
+        dialect: t.Optional[str] = None,
+        register_comments: bool = True,
+        default_catalog: t.Optional[str] = None,
+        **kwargs: t.Any,
     ) -> T:
         connection_mock = mocker.NonCallableMock()
         cursor_mock = mocker.Mock()
@@ -398,12 +452,19 @@ def make_mocked_engine_adapter(mocker: MockerFixture) -> t.Callable:
             lambda: connection_mock,
             dialect=dialect or klass.DIALECT,
             register_comments=register_comments,
+            default_catalog=default_catalog,
+            **kwargs,
         )
         if isinstance(adapter, SparkEngineAdapter):
             mocker.patch(
                 "sqlmesh.engines.spark.db_api.spark_session.SparkSessionConnection._spark_major_minor",
                 new_callable=PropertyMock(return_value=(3, 5)),
             )
+        if isinstance(adapter, MSSQLEngineAdapter):
+            mocker.patch(
+                "sqlmesh.core.engine_adapter.mssql.MSSQLEngineAdapter.catalog_support",
+                new_callable=PropertyMock(return_value=CatalogSupport.REQUIRES_SET_CATALOG),
+            )
         return adapter
 
     return _make_function
@@ -412,9 +473,11 @@ def make_mocked_engine_adapter(mocker: MockerFixture) -> t.Callable:
 @pytest.fixture
 def copy_to_temp_path(tmp_path: Path) -> t.Callable:
     def ignore(src, names):
+        # do not copy any sub-dirs if current dir is named one of these
         if Path(src).name in {".cache", "__pycache__", "logs", "data", "target"}:
             return names
-        return []
+        # do not copy sub-dirs named ".cache" for any current dir
+        return [name for name in names if name == ".cache"]
 
     def _make_function(
         paths: t.Union[t.Union[str, Path], t.Collection[t.Union[str, Path]]],
