@@ -10,6 +10,9 @@ from sqlglot import expressions as exp
 from sqlglot import parse_one
 
 from sqlmesh.core.engine_adapter.mssql import MSSQLEngineAdapter
+from sqlmesh.core.snapshot import SnapshotEvaluator, SnapshotChangeCategory
+from sqlmesh.core.model import load_sql_based_model
+from sqlmesh.core import dialect as d
 from sqlmesh.core.engine_adapter.shared import (
     DataObject,
     DataObjectType,
@@ -77,6 +80,65 @@ def test_columns(make_mocked_engine_adapter: t.Callable):
     )
 
 
+def test_varchar_workaround_to_max(make_mocked_engine_adapter: t.Callable, mocker: MockerFixture):
+    adapter = make_mocked_engine_adapter(MSSQLEngineAdapter)
+
+    columns = {
+        "binary1": exp.DataType.build("BINARY(1)", dialect=adapter.dialect),
+        "varbinary": exp.DataType.build("VARBINARY", dialect=adapter.dialect),
+        "varbinary1": exp.DataType.build("VARBINARY(1)", dialect=adapter.dialect),
+        "varbinary2": exp.DataType.build("VARBINARY(2)", dialect=adapter.dialect),
+        "varchar": exp.DataType.build("VARCHAR", dialect=adapter.dialect),
+        "varchar1": exp.DataType.build("VARCHAR(1)", dialect=adapter.dialect),
+        "varchar2": exp.DataType.build("VARCHAR(2)", dialect=adapter.dialect),
+        "nvarchar": exp.DataType.build("NVARCHAR", dialect=adapter.dialect),
+        "nvarchar1": exp.DataType.build("NVARCHAR(1)", dialect=adapter.dialect),
+        "nvarchar2": exp.DataType.build("NVARCHAR(2)", dialect=adapter.dialect),
+    }
+
+    assert adapter._default_precision_to_max(columns) == {
+        "binary1": exp.DataType.build("BINARY(1)", dialect=adapter.dialect),
+        "varbinary": exp.DataType.build("VARBINARY", dialect=adapter.dialect),
+        "varbinary1": exp.DataType.build("VARBINARY(max)", dialect=adapter.dialect),
+        "varbinary2": exp.DataType.build("VARBINARY(2)", dialect=adapter.dialect),
+        "varchar": exp.DataType.build("VARCHAR", dialect=adapter.dialect),
+        "varchar1": exp.DataType.build("VARCHAR(max)", dialect=adapter.dialect),
+        "varchar2": exp.DataType.build("VARCHAR(2)", dialect=adapter.dialect),
+        "nvarchar": exp.DataType.build("NVARCHAR", dialect=adapter.dialect),
+        "nvarchar1": exp.DataType.build("NVARCHAR(max)", dialect=adapter.dialect),
+        "nvarchar2": exp.DataType.build("NVARCHAR(2)", dialect=adapter.dialect),
+    }
+
+    mocker.patch(
+        "sqlmesh.core.engine_adapter.base.random_id",
+        return_value="test_random_id",
+    )
+
+    mocker.patch(
+        "sqlmesh.core.engine_adapter.mssql.MSSQLEngineAdapter.table_exists",
+        return_value=True,
+    )
+
+    mocker.patch(
+        "sqlmesh.core.engine_adapter.mssql.MSSQLEngineAdapter.columns",
+        return_value=columns,
+    )
+
+    adapter.ctas(
+        table_name="test_schema.test_table",
+        query_or_df=parse_one(
+            "SELECT binary, varbinary1 + 1 AS varbinary1, varbinary2 AS varbinary2, varchar, varchar1, varchar2, nvarchar, nvarchar1, nvarchar2 FROM (SELECT * FROM table WHERE FALSE LIMIT 0) WHERE d > 0 AND FALSE LIMIT 0"
+        ),
+        exists=False,
+    )
+
+    assert to_sql_calls(adapter) == [
+        "CREATE VIEW [__temp_ctas_test_random_id] AS SELECT [binary], [varbinary1] + 1 AS [varbinary1], [varbinary2] AS [varbinary2], [varchar], [varchar1], [varchar2], [nvarchar], [nvarchar1], [nvarchar2] FROM (SELECT * FROM [table]);",
+        "DROP VIEW IF EXISTS [__temp_ctas_test_random_id];",
+        "CREATE TABLE [test_schema].[test_table] ([binary1] BINARY(1), [varbinary] VARBINARY, [varbinary1] VARBINARY(max), [varbinary2] VARBINARY(2), [varchar] VARCHAR, [varchar1] VARCHAR(max), [varchar2] VARCHAR(2), [nvarchar] NVARCHAR, [nvarchar1] NVARCHAR(max), [nvarchar2] NVARCHAR(2));",
+    ]
+
+
 def test_table_exists(make_mocked_engine_adapter: t.Callable):
     adapter = make_mocked_engine_adapter(MSSQLEngineAdapter)
     adapter.cursor.fetchone.return_value = (1,)
@@ -93,6 +155,113 @@ def test_table_exists(make_mocked_engine_adapter: t.Callable):
     assert not resp
 
 
+@pytest.mark.parametrize(
+    "select_expr, input_time, expected_sql",
+    [
+        # Respect the user's precision for datetimeoffset, time, datetime2
+        (
+            "SELECT ds::datetime2",
+            pd.Timestamp("2022-01-01 00:00:00.1234567"),
+            "CAST('2022-01-01 00:00:00.123456700' AS DATETIME2)",
+        ),
+        (
+            "SELECT ds::datetimeoffset(4)",
+            pd.Timestamp("2022-01-01 00:00:00.1234567"),
+            "CAST('2022-01-01 00:00:00.123456700+00:00' AS DATETIMEOFFSET(4))",
+        ),
+        (
+            "SELECT ds::time",
+            pd.Timestamp("2022-01-01 00:00:00.1234567"),
+            "CAST('2022-01-01 00:00:00.123456700' AS TIME)",
+        ),
+        # Respecting precision in datetimeoffset with time zone offsets
+        (
+            "SELECT ds::time(7)",
+            pd.Timestamp("2022-01-01 00:00:00.1234567+00:00"),
+            "CAST('2022-01-01 00:00:00.123456700' AS TIME(7))",
+        ),
+        (
+            "SELECT ds::datetimeoffset(6)",
+            pd.Timestamp("2022-01-01 00:00:00.1234567+02:00"),
+            "CAST('2021-12-31 22:00:00.123456700+00:00' AS DATETIMEOFFSET(6))",
+        ),
+        # For date types without nano-second precision, truncate as usual
+        (
+            "SELECT ds::datetime",
+            "2022-01-01 00:00:00.1234567+01:00",
+            "CAST('2021-12-31 23:00:00.123456' AS DATETIME)",
+        ),
+        (
+            "SELECT ds::smalldatetime",
+            "2022-01-01 00:00:00.1234567+00:00",
+            "CAST('2022-01-01 00:00:00.123456' AS SMALLDATETIME)",
+        ),
+        ("SELECT ds::date", "2022-01-01 00:00:00.001", "CAST('2022-01-01' AS DATE)"),
+    ],
+)
+def test_to_time_column(select_expr, input_time, expected_sql):
+    expressions = d.parse(
+        f"""
+        MODEL (
+            name db.table,
+            kind INCREMENTAL_BY_TIME_RANGE(
+                time_column (ds)
+            ),
+            dialect tsql
+        );
+
+        {select_expr}
+        """
+    )
+    model = load_sql_based_model(expressions)
+    assert model.convert_to_time_column(input_time).sql("tsql") == expected_sql
+
+
+def test_incremental_by_time_datetimeoffset_precision(
+    make_mocked_engine_adapter: t.Callable, mocker: MockerFixture, make_snapshot
+):
+    adapter = make_mocked_engine_adapter(MSSQLEngineAdapter)
+    adapter.get_current_catalog = mocker.MagicMock(return_value="other_catalog")
+
+    evaluator = SnapshotEvaluator(adapter)
+    parsed = d.parse(  # type: ignore
+        """
+            MODEL (
+                name test_schema.test_model,
+                kind INCREMENTAL_BY_TIME_RANGE (time_column ds),
+            );
+
+            SELECT a::int, ds::datetimeoffset FROM tbl as t WHERE t.ds BETWEEN @start_dt and @end_dt;
+            """,
+    )
+
+    model = load_sql_based_model(parsed, dialect="tsql")
+
+    snapshot = make_snapshot(model)
+    snapshot.categorize_as(SnapshotChangeCategory.BREAKING)
+
+    evaluator.evaluate(
+        snapshot,
+        start="2020-01-01",
+        end="2020-01-02",
+        execution_time="2020-01-02",
+        snapshots={},
+    )
+
+    assert adapter.cursor.execute.call_args_list[0][0][0] == (
+        f"MERGE INTO [sqlmesh__test_schema].[test_schema__test_model__{snapshot.version}] AS [__MERGE_TARGET__] USING "
+        "(SELECT [a] AS [a], [ds] AS [ds] FROM (SELECT CAST([a] AS INTEGER) AS [a], "
+        "CAST([ds] AS DATETIMEOFFSET) AS [ds] FROM [tbl] AS [t] WHERE [t].[ds] BETWEEN "
+        "CAST('2020-01-01 00:00:00+00:00' AS DATETIMEOFFSET) AT TIME ZONE 'UTC' AND "
+        "CAST('2020-01-02 23:59:59.999999999+00:00' AS DATETIMEOFFSET) AT TIME ZONE 'UTC') AS [_subquery] "
+        "WHERE [ds] BETWEEN CAST('2020-01-01 00:00:00+00:00' AS DATETIMEOFFSET) AND "
+        "CAST('2020-01-02 23:59:59.999999999+00:00' AS DATETIMEOFFSET)) AS [__MERGE_SOURCE__] ON (1 = 0) WHEN NOT "
+        "MATCHED BY SOURCE AND [ds] BETWEEN CAST('2020-01-01 00:00:00+00:00' AS DATETIMEOFFSET) AND "
+        "CAST('2020-01-02 23:59:59.999999999+00:00' AS DATETIMEOFFSET) THEN DELETE WHEN NOT MATCHED THEN INSERT "
+        "([a], [ds]) VALUES ([a], [ds]);"
+    )
+
+
 def test_insert_overwrite_by_time_partition_supports_insert_overwrite_pandas_not_exists(
     make_mocked_engine_adapter: t.Callable, mocker: MockerFixture, make_temp_table_name: t.Callable
 ):
