@@ -3,7 +3,7 @@ from __future__ import annotations
 import datetime
 import typing as t
 from pathlib import Path
-from unittest.mock import call
+from unittest.mock import call, patch
 
 import pandas as pd
 import pytest
@@ -20,12 +20,13 @@ from sqlmesh.core.config import (
     ModelDefaultsConfig,
 )
 from sqlmesh.core.context import Context
+from sqlmesh.core.console import get_console
 from sqlmesh.core.dialect import parse
 from sqlmesh.core.engine_adapter import EngineAdapter
 from sqlmesh.core.macros import MacroEvaluator, macro
 from sqlmesh.core.model import Model, SqlModel, load_sql_based_model, model
 from sqlmesh.core.test.definition import ModelTest, PythonModelTest, SqlModelTest
-from sqlmesh.utils.errors import ConfigError, TestError
+from sqlmesh.utils.errors import ConfigError, SQLMeshError, TestError
 from sqlmesh.utils.yaml import dump as dump_yaml
 from sqlmesh.utils.yaml import load as load_yaml
 
@@ -108,7 +109,7 @@ def full_model_with_two_ctes(request) -> SqlModel:
         renamed AS (
             SELECT id AS fid FROM source
         )
-        SELECT fid FROM renamed;
+        SELECT fid FROM RENAMED;
         """,
         dialect=getattr(request, "param", None),
         default_catalog="memory",
@@ -345,6 +346,48 @@ test_foo:
         ),
     )
 
+    model_sql = """
+SELECT
+    ARRAY_AGG(DISTINCT id_contact_b ORDER BY id_contact_b) AS aggregated_duplicates
+FROM
+    source
+GROUP BY
+    id_contact_a
+ORDER BY
+    id_contact_a
+    """
+
+    _check_successful_or_raise(
+        _create_test(
+            body=load_yaml(
+                """
+test_array_order:
+  model: test
+  inputs:
+    source:
+    - id_contact_a: a
+      id_contact_b: b
+    - id_contact_a: a
+      id_contact_b: c
+  outputs:
+    query:
+    - aggregated_duplicates:
+      - c
+      - b
+                """
+            ),
+            test_name="test_array_order",
+            model=_create_model(model_sql),
+            context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
+        ).run(),
+        expected_msg=(
+            """AssertionError: Data mismatch (exp: expected, act: actual)\n\n"""
+            "  aggregated_duplicates        \n"
+            "                    exp     act\n"
+            "0                (c, b)  (b, c)\n"
+        ),
+    )
+
 
 @pytest.mark.parametrize(
     "waiter_names_input",
@@ -356,7 +399,7 @@ test_foo:
           - id: 3
             name: 'bob'
         """,
-        """sushi.waiter_names: 
+        """sushi.waiter_names:
       format: csv
       rows: |
         id,name
@@ -410,21 +453,21 @@ test_foo:
     [
         """sushi.waiter_names:
         format: yaml
-        rows: 
+        rows:
         - id: 1
           name: alice
         - id: 2
           name: 'bob'
         """,
-        """sushi.waiter_names: 
+        """sushi.waiter_names:
       format: csv
       rows: |
         id,name
         1,alice
         2,bob""",
-        """sushi.waiter_names: 
+        """sushi.waiter_names:
       format: csv
-      csv_settings: 
+      csv_settings:
         sep: "#"
       rows: |
         id#name
@@ -1124,25 +1167,30 @@ test_foo:
   inputs:
     sushi.raw:
       columns:
-        array: "INT[]"
+        array1: "INT[]"
+        array2: "STRUCT(k VARCHAR, v STRUCT(v_str VARCHAR, v_int INT, v_int_arr INT[]))[]"
         struct: "STRUCT(x INT[], y VARCHAR, z INT, w STRUCT(a INT))"
       rows:
-        - array: [1, 2, 3]
+        - array1: [1, 2, 3]
+          array2: [{'k': 'hello', 'v': {'v_str': 'there', 'v_int': 10, 'v_int_arr': [1, 2]}}]
           struct: {'x': [1, 2, 3], 'y': 'foo', 'z': 1, 'w': {'a': 5}}
-        - array:
+        - array1:
           - 2
           - 3
-        - array: [0, 4, 1]
+        - array1: [0, 4, 1]
   outputs:
     query:
-      - array: [0, 4, 1]
-      - array: [1, 2, 3]
+      - array1: [0, 4, 1]
+      - array1: [1, 2, 3]
+        array2: [{'k': 'hello', 'v': {'v_str': 'there', 'v_int': 10, 'v_int_arr': [1, 2]}}]
         struct: {'x': [1, 2, 3], 'y': 'foo', 'z': 1, 'w': {'a': 5}}
-      - array: [2, 3]
+      - array1: [2, 3]
                 """
             ),
             test_name="test_foo",
-            model=_create_model("SELECT array, struct FROM sushi.raw", default_catalog="memory"),
+            model=_create_model(
+                "SELECT array1, array2, struct FROM sushi.raw", default_catalog="memory"
+            ),
             context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
         ).run()
     )
@@ -1187,14 +1235,38 @@ test_foo:
         ]
     )
 
+    test = _create_test(
+        body=load_yaml(
+            """
+test_foo:
+  model: xyz
+  outputs:
+    query:
+      - cur_timestamp: "2023-01-01 12:05:03+00:00"
+  vars:
+    execution_time: "2023-01-01 12:05:03+00:00"
+            """
+        ),
+        test_name="test_foo",
+        model=_create_model("SELECT CURRENT_TIMESTAMP AS cur_timestamp"),
+        context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="bigquery"))),
+    )
+
+    spy_execute = mocker.spy(test.engine_adapter, "_execute")
+    _check_successful_or_raise(test.run())
+
+    spy_execute.assert_has_calls(
+        [call('''SELECT CAST('2023-01-01 12:05:03+00:00' AS TIMESTAMPTZ) AS "cur_timestamp"''')]
+    )
+
     @model("py_model", columns={"ts1": "timestamptz", "ts2": "timestamptz"})
     def execute(context, start, end, execution_time, **kwargs):
-        datetime_now = datetime.datetime.now()
+        datetime_now_utc = datetime.datetime.now(tz=datetime.timezone.utc)
 
         context.engine_adapter.execute(exp.select("CURRENT_TIMESTAMP"))
         current_timestamp = context.engine_adapter.cursor.fetchone()[0]
 
-        return pd.DataFrame([{"ts1": datetime_now, "ts2": current_timestamp}])
+        return pd.DataFrame([{"ts1": datetime_now_utc, "ts2": current_timestamp}])
 
     _check_successful_or_raise(
         _create_test(
@@ -1340,11 +1412,15 @@ test_example_full_model_alt:
   inputs:
     sqlmesh_example.incremental_model:
       query: |
-        SELECT 1 AS id, 1 AS item_id
+        SELECT 1 AS item_id, 1 AS id
         UNION ALL
-        SELECT 2 AS id, 1 AS item_id
+        SELECT 1 AS item_id, 2 AS id
         UNION ALL
-        SELECT 3 AS id, 2 AS item_id
+        SELECT 2 AS item_id, 3 AS id
+        UNION ALL
+        SELECT 3 AS item_id, 4 AS id
+        UNION ALL
+        SELECT 4 AS item_id, null AS id
   outputs:
     query:
       rows:
@@ -1352,6 +1428,10 @@ test_example_full_model_alt:
         num_orders: 2
       - item_id: 2
         num_orders: 1
+      - item_id: 3
+        num_orders: 1
+      - item_id: 4
+        num_orders: 0
                 """
             ),
             test_name="test_example_full_model_alt",
@@ -1360,6 +1440,32 @@ test_example_full_model_alt:
         ).run()
     )
 
+    _check_successful_or_raise(
+        _create_test(
+            body=load_yaml(
+                """
+test_example_full_model_partial:
+  model: sqlmesh_example.full_model
+  inputs:
+    sqlmesh_example.incremental_model:
+      query: |
+        SELECT 1 as id,
+        UNION ALL
+        SELECT 2 as id,
+  outputs:
+    query:
+      partial: true
+      rows:
+      - item_id: null
+        num_orders: 2
+                """
+            ),
+            test_name="test_example_full_model_partial",
+            model=context.get_model("sqlmesh_example.full_model"),
+            context=context,
+        ).run()
+    )
+
     mocker.patch("sqlmesh.core.test.definition.random_id", return_value="jzngz56a")
     test = _create_test(
         body=load_yaml(
@@ -1411,12 +1517,12 @@ test_foo:
         )
 
 
-def test_pyspark_python_model() -> None:
+def test_pyspark_python_model(tmp_path: Path) -> None:
     spark_connection_config = SparkConnectionConfig(
         config={
             "spark.master": "local",
-            "spark.sql.warehouse.dir": "/tmp/data_dir",
-            "spark.driver.extraJavaOptions": "-Dderby.system.home=/tmp/derby_dir",
+            "spark.sql.warehouse.dir": f"{tmp_path}/data_dir",
+            "spark.driver.extraJavaOptions": f"-Dderby.system.home={tmp_path}/derby_dir",
         },
     )
     config = Config(
@@ -1449,6 +1555,234 @@ test_pyspark_model:
     )
 
 
+def test_variable_usage(tmp_path: Path) -> None:
+    init_example_project(tmp_path, dialect="duckdb")
+
+    config = Config(
+        default_connection=DuckDBConnectionConfig(),
+        model_defaults=ModelDefaultsConfig(dialect="duckdb"),
+        variables={"gold": "gold_db", "silver": "silver_db"},
+    )
+    context = Context(paths=tmp_path, config=config)
+
+    parent = _create_model(
+        "SELECT 1 AS id, '2022-01-02'::DATE AS ds, @start_ts AS start_ts",
+        meta="MODEL (name silver_db.sch.b, kind INCREMENTAL_BY_TIME_RANGE(time_column ds))",
+    )
+    parent = t.cast(SqlModel, context.upsert_model(parent))
+
+    child = _create_model(
+        "SELECT ds, @IF(@VAR('myvar'), id, id + 1) AS id FROM silver_db.sch.b WHERE ds BETWEEN @start_ds and @end_ds",
+        meta="MODEL (name gold_db.sch.a, kind INCREMENTAL_BY_TIME_RANGE(time_column ds))",
+    )
+    child = t.cast(SqlModel, context.upsert_model(child))
+
+    test_file = tmp_path / "tests" / "test_parameterized_model_names.yaml"
+    test_file.write_text(
+        """
+test_parameterized_model_names:
+  model: {{ var('gold') }}.sch.a
+  vars:
+    myvar: True
+    start_ds: 2022-01-01
+    end_ds: 2022-01-03
+  inputs:
+    {{ var('silver') }}.sch.b:
+      - ds: 2022-01-01
+        id: 1
+      - ds: 2022-01-01
+        id: 2
+  outputs:
+    query:
+      - ds: 2022-01-01
+        id: 1
+      - ds: 2022-01-01
+        id: 2
+        """
+    )
+
+    results = context.test()
+
+    assert not results.failures
+    assert not results.errors
+
+    # The example project has one test and we added another one above
+    assert len(results.successes) == 2
+
+
+def test_custom_testing_schema(mocker: MockerFixture) -> None:
+    test = _create_test(
+        body=load_yaml(
+            """
+test_foo:
+  model: xyz
+  schema: my_schema
+  outputs:
+    query:
+      - a: 1
+            """
+        ),
+        test_name="test_foo",
+        model=_create_model("SELECT 1 AS a"),
+        context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
+    )
+
+    spy_execute = mocker.spy(test.engine_adapter, "_execute")
+    _check_successful_or_raise(test.run())
+
+    spy_execute.assert_has_calls(
+        [
+            call('CREATE SCHEMA IF NOT EXISTS "memory"."my_schema"'),
+            call('SELECT 1 AS "a"'),
+            call('DROP SCHEMA IF EXISTS "memory"."my_schema" CASCADE'),
+        ]
+    )
+
+
+def test_pretty_query(mocker: MockerFixture) -> None:
+    test = _create_test(
+        body=load_yaml(
+            """
+test_foo:
+  model: xyz
+  schema: my_schema
+  outputs:
+    query:
+      - a: 1
+            """
+        ),
+        test_name="test_foo",
+        model=_create_model("SELECT 1 AS a"),
+        context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
+    )
+    test.engine_adapter._pretty_sql = True
+    spy_execute = mocker.spy(test.engine_adapter, "_execute")
+    _check_successful_or_raise(test.run())
+    spy_execute.assert_has_calls(
+        [
+            call('CREATE SCHEMA IF NOT EXISTS "memory"."my_schema"'),
+            call('SELECT\n  1 AS "a"'),
+            call('DROP SCHEMA IF EXISTS "memory"."my_schema" CASCADE'),
+        ]
+    )
+
+
+def test_complicated_recursive_cte() -> None:
+    model_sql = """
+WITH
+    RECURSIVE
+    chained_contacts AS (
+        -- Start with the initial set of contacts and their immediate nodes
+        SELECT
+            id_contact_a,
+            id_contact_b
+        FROM
+            source
+
+        UNION ALL
+
+        -- Recursive step to find further connected nodes
+        SELECT
+            chained_contacts.id_contact_a,
+            unfactorized_duplicates.id_contact_b
+        FROM
+            chained_contacts
+                JOIN source AS unfactorized_duplicates
+                     ON chained_contacts.id_contact_b = unfactorized_duplicates.id_contact_a
+    ),
+    id_contact_a_with_aggregated_id_contact_bs AS (
+        SELECT
+            id_contact_a,
+            ARRAY_AGG(DISTINCT id_contact_b ORDER BY id_contact_b) AS aggregated_id_contact_bs
+        FROM
+            chained_contacts
+        GROUP BY
+            id_contact_a
+    )
+SELECT
+    ARRAY_CONCAT([id_contact_a], aggregated_id_contact_bs) AS aggregated_duplicates
+FROM
+    id_contact_a_with_aggregated_id_contact_bs
+WHERE
+    id_contact_a NOT IN (
+        SELECT DISTINCT
+            id_contact_b
+        FROM
+            source
+    )
+ORDER BY
+    id_contact_a
+    """
+
+    _check_successful_or_raise(
+        _create_test(
+            body=load_yaml(
+                """
+test_recursive_ctes:
+  model: test
+  inputs:
+    source:
+      rows:
+        - id_contact_a: "a"
+          id_contact_b: "b"
+        - id_contact_a: "b"
+          id_contact_b: "c"
+        - id_contact_a: "c"
+          id_contact_b: "d"
+        - id_contact_a: "a"
+          id_contact_b: "g"
+        - id_contact_a: "b"
+          id_contact_b: "e"
+        - id_contact_a: "c"
+          id_contact_b: "f"
+        - id_contact_a: "x"
+          id_contact_b: "y"
+  outputs:
+    ctes:
+      id_contact_a_with_aggregated_id_contact_bs:
+        - id_contact_a: a
+          aggregated_id_contact_bs: [b, c, d, e, f, g]
+        - id_contact_a: x
+          aggregated_id_contact_bs: [y]
+        - id_contact_a: b
+          aggregated_id_contact_bs: [c, d, e, f]
+        - id_contact_a: c
+          aggregated_id_contact_bs: [d, f]
+    query:
+      rows:
+        - aggregated_duplicates: [a, b, c, d, e, f, g]
+        - aggregated_duplicates: [x, y]
+                """
+            ),
+            test_name="test_recursive_ctes",
+            model=_create_model(model_sql),
+            context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
+        ).run()
+    )
+
+
+def test_unknown_model_warns(mocker: MockerFixture) -> None:
+    body = load_yaml(
+        """
+model: unknown
+outputs:
+  query:
+  - c: 1
+        """
+    )
+
+    with patch.object(get_console(), "log_warning") as mock_logger:
+        ModelTest.create_test(
+            body=body,
+            test_name="test_unknown_model",
+            models={},  # type: ignore
+            engine_adapter=mocker.Mock(),
+            dialect=None,
+            path=None,
+        )
+        assert mock_logger.mock_calls == [call("Model '\"unknown\"' was not found")]
+
+
 def test_test_generation(tmp_path: Path) -> None:
     init_example_project(tmp_path, dialect="duckdb")
 
@@ -1595,7 +1929,7 @@ def test_test_generation_with_data_structures(tmp_path: Path, column: str, expec
     bar_sql_file.write_text("MODEL (name sqlmesh_example.bar); SELECT col FROM external_table;")
 
     test = create_test(Context(paths=tmp_path, config=config), f"SELECT {column} AS col")
-    assert test["test_foo"]["inputs"] == {"sqlmesh_example.bar": expected}
+    assert test["test_foo"]["inputs"] == {'"memory"."sqlmesh_example"."bar"': expected}
     assert test["test_foo"]["outputs"] == {"query": expected}
 
 
@@ -1618,20 +1952,179 @@ def test_test_generation_with_timestamp(tmp_path: Path) -> None:
     input_queries = {
         "sqlmesh_example.bar": "SELECT TIMESTAMP '2024-09-20 11:30:00.123456789' AS ts_col"
     }
-
-    context.create_test(
-        "sqlmesh_example.foo",
-        input_queries=input_queries,
-        overwrite=True,
-    )
+    context.create_test("sqlmesh_example.foo", input_queries=input_queries, overwrite=True)
 
     test = load_yaml(context.path / c.TESTS / "test_foo.yaml")
 
     assert len(test) == 1
     assert "test_foo" in test
     assert test["test_foo"]["inputs"] == {
-        "sqlmesh_example.bar": [{"ts_col": datetime.datetime(2024, 9, 20, 11, 30, 0, 123456)}]
+        '"memory"."sqlmesh_example"."bar"': [
+            {"ts_col": datetime.datetime(2024, 9, 20, 11, 30, 0, 123456)}
+        ]
     }
     assert test["test_foo"]["outputs"] == {
         "query": [{"ts_col": datetime.datetime(2024, 9, 20, 11, 30, 0, 123456)}]
     }
+
+
+def test_test_generation_with_decimal(tmp_path: Path, mocker: MockerFixture) -> None:
+    from decimal import Decimal
+
+    init_example_project(tmp_path, dialect="duckdb")
+
+    config = Config(
+        default_connection=DuckDBConnectionConfig(),
+        model_defaults=ModelDefaultsConfig(dialect="duckdb"),
+    )
+    foo_sql_file = tmp_path / "models" / "foo.sql"
+    foo_sql_file.write_text(
+        "MODEL (name sqlmesh_example.foo); SELECT dec_col FROM sqlmesh_example.bar;"
+    )
+    bar_sql_file = tmp_path / "models" / "bar.sql"
+    bar_sql_file.write_text("MODEL (name sqlmesh_example.bar); SELECT dec_col FROM external_table;")
+
+    context = Context(paths=tmp_path, config=config)
+    input_queries = {
+        '"memory"."sqlmesh_example"."bar"': "SELECT CAST(1.23 AS DECIMAL(10,2)) AS dec_col"
+    }
+
+    # DuckDB actually returns a numpy.float64, even though the value is cast into a DECIMAL,
+    # but other engines don't behave the same. E.g. BigQuery returns a proper Decimal value.
+    mocker.patch(
+        "sqlmesh.core.engine_adapter.base.EngineAdapter.fetchdf",
+        return_value=pd.DataFrame({"dec_col": [Decimal("1.23")]}),
+    )
+
+    context.create_test("sqlmesh_example.foo", input_queries=input_queries, overwrite=True)
+
+    test = load_yaml(context.path / c.TESTS / "test_foo.yaml")
+
+    assert len(test) == 1
+    assert "test_foo" in test
+    assert test["test_foo"]["inputs"] == {'"memory"."sqlmesh_example"."bar"': [{"dec_col": "1.23"}]}
+    assert test["test_foo"]["outputs"] == {"query": [{"dec_col": "1.23"}]}
+
+
+def test_test_generation_with_recursive_ctes(tmp_path: Path) -> None:
+    init_example_project(tmp_path, dialect="duckdb")
+
+    config = Config(
+        default_connection=DuckDBConnectionConfig(),
+        model_defaults=ModelDefaultsConfig(dialect="duckdb"),
+    )
+    foo_sql_file = tmp_path / "models" / "foo.sql"
+    foo_sql_file.write_text(
+        "MODEL (name sqlmesh_example.foo);"
+        "WITH RECURSIVE t AS (SELECT 1 AS c UNION ALL SELECT c + 1 FROM t WHERE c < 3) SELECT c FROM t"
+    )
+
+    context = Context(paths=tmp_path, config=config)
+    context.plan(auto_apply=True)
+
+    context.create_test("sqlmesh_example.foo", input_queries={}, overwrite=True, include_ctes=True)
+
+    test = load_yaml(context.path / c.TESTS / "test_foo.yaml")
+    assert len(test) == 1
+    assert "test_foo" in test
+    assert test["test_foo"]["inputs"] == {}
+    assert test["test_foo"]["outputs"] == {
+        "query": [{"c": 1}, {"c": 2}, {"c": 3}],
+        "ctes": {
+            "t": [{"c": 1}, {"c": 2}, {"c": 3}],
+        },
+    }
+
+    _check_successful_or_raise(context.test())
+
+
+def test_test_with_gateway_specific_model(tmp_path: Path, mocker: MockerFixture) -> None:
+    init_example_project(tmp_path, dialect="duckdb")
+
+    config = Config(
+        gateways={
+            "main": GatewayConfig(connection=DuckDBConnectionConfig()),
+            "second": GatewayConfig(connection=DuckDBConnectionConfig()),
+        },
+        default_gateway="main",
+        model_defaults=ModelDefaultsConfig(dialect="duckdb"),
+    )
+    gw_model_sql_file = tmp_path / "models" / "gw_model.sql"
+
+    # The model has a gateway specified which isn't the default
+    gw_model_sql_file.write_text(
+        "MODEL (name sqlmesh_example.gw_model, gateway second); SELECT c FROM sqlmesh_example.input_model;"
+    )
+    input_model_sql_file = tmp_path / "models" / "input_model.sql"
+    input_model_sql_file.write_text(
+        "MODEL (name sqlmesh_example.input_model); SELECT c FROM external_table;"
+    )
+
+    context = Context(paths=tmp_path, config=config)
+    input_queries = {'"memory"."sqlmesh_example"."input_model"': "SELECT 5 AS c"}
+    mocker.patch(
+        "sqlmesh.core.engine_adapter.base.EngineAdapter.fetchdf",
+        return_value=pd.DataFrame({"c": [5]}),
+    )
+
+    assert context.engine_adapter == context._engine_adapters["main"]
+    with pytest.raises(
+        SQLMeshError, match=r"Gateway 'wrong' not found in the available engine adapters."
+    ):
+        context._get_engine_adapter("wrong")
+
+    # Create test should use the gateway specific engine adapter
+    context.create_test("sqlmesh_example.gw_model", input_queries=input_queries, overwrite=True)
+    assert context._get_engine_adapter("second") == context._engine_adapters["second"]
+    assert len(context._engine_adapters) == 2
+
+    test = load_yaml(context.path / c.TESTS / "test_gw_model.yaml")
+
+    assert len(test) == 1
+    assert "test_gw_model" in test
+    assert test["test_gw_model"]["inputs"] == {
+        '"memory"."sqlmesh_example"."input_model"': [{"c": 5}]
+    }
+    assert test["test_gw_model"]["outputs"] == {"query": [{"c": 5}]}
+
+
+def test_test_with_resolve_template_macro(tmp_path: Path):
+    config = Config(
+        default_connection=DuckDBConnectionConfig(),
+        model_defaults=ModelDefaultsConfig(dialect="duckdb"),
+    )
+
+    models_dir = tmp_path / "models"
+    models_dir.mkdir()
+    (models_dir / "foo.sql").write_text(
+        """
+      MODEL (
+        name test.foo,
+        kind full,
+        physical_properties (
+          location = @resolve_template('file:///tmp/@{table_name}')
+        )
+      );
+
+      SELECT t.a + 1 as a
+      FROM @resolve_template('@{schema_name}.dev_@{table_name}', mode := 'table') as t
+      """
+    )
+
+    tests_dir = tmp_path / "tests"
+    tests_dir.mkdir()
+    (tests_dir / "test_foo.yaml").write_text(
+        """
+test_resolve_template_macro:
+  model: test.foo
+  inputs:
+    test.dev_foo:
+      - a: 1
+  outputs:
+    query:
+      - a: 2
+    """
+    )
+
+    context = Context(paths=tmp_path, config=config)
+    _check_successful_or_raise(context.test())
