@@ -9,15 +9,17 @@ import pytest
 from dbt.adapters.base import BaseRelation
 from dbt.adapters.base.column import Column
 from pytest_mock.plugin import MockerFixture
-from sqlglot import exp
+from sqlglot import exp, parse_one
 
 from sqlmesh import Context
 from sqlmesh.core.dialect import schema_
 from sqlmesh.core.snapshot import SnapshotId
+from sqlmesh.dbt.adapter import ParsetimeAdapter
 from sqlmesh.dbt.project import Project
 from sqlmesh.dbt.relation import Policy
 from sqlmesh.dbt.target import SnowflakeConfig
 from sqlmesh.utils.errors import ConfigError
+from sqlmesh.utils.jinja import JinjaMacroRegistry
 
 pytestmark = pytest.mark.dbt
 
@@ -73,10 +75,23 @@ def test_normalization(
     context = sushi_test_project.context
     assert context.target
 
+    # bla and bob will be normalized to lowercase since the target is duckdb
     adapter_mock = mocker.MagicMock()
-    adapter_mock.dialect = "snowflake"
     adapter_mock.default_catalog = "test"
+    adapter_mock.dialect = "duckdb"
 
+    duckdb_renderer = runtime_renderer(context, engine_adapter=adapter_mock)
+
+    schema_bla = schema_("bla", "test", quoted=True)
+    relation_bla_bob = exp.table_("bob", db="bla", catalog="test", quoted=True)
+
+    duckdb_renderer("{{ adapter.get_relation(database=None, schema='bla', identifier='bob') }}")
+    adapter_mock.table_exists.assert_has_calls([call(relation_bla_bob)])
+
+    # bla and bob will be normalized to uppercase since the target is Snowflake, even though the default dialect is duckdb
+    adapter_mock = mocker.MagicMock()
+    adapter_mock.default_catalog = "test"
+    adapter_mock.dialect = "snowflake"
     context.target = SnowflakeConfig(
         account="test",
         user="test",
@@ -85,16 +100,19 @@ def test_normalization(
         database="test",
         schema="test",
     )
-
     renderer = runtime_renderer(context, engine_adapter=adapter_mock)
 
-    # bla and bob will be normalized to uppercase since we're dealing with Snowflake
-    schema_bla = schema_("BLA", "TEST", quoted=True)
-    relation_bla_bob = exp.table_("BOB", db="BLA", catalog="TEST", quoted=True)
+    schema_bla = schema_("bla", "test", quoted=True)
+    relation_bla_bob = exp.table_("bob", db="bla", catalog="test", quoted=True)
 
     renderer("{{ adapter.get_relation(database=None, schema='bla', identifier='bob') }}")
     adapter_mock.table_exists.assert_has_calls([call(relation_bla_bob)])
 
+    renderer("{{ adapter.get_relation(database='custom_db', schema='bla', identifier='bob') }}")
+    adapter_mock.table_exists.assert_has_calls(
+        [call(exp.table_("bob", db="bla", catalog="custom_db", quoted=True))]
+    )
+
     renderer(
         "{%- set relation = api.Relation.create(schema='bla') -%}"
         "{{ adapter.create_schema(relation) }}"
@@ -114,7 +132,7 @@ def test_normalization(
     adapter_mock.drop_table.assert_has_calls([call(relation_bla_bob)])
 
     expected_star_query: exp.Select = exp.maybe_parse(
-        'SELECT * FROM "T" as "T"', dialect="snowflake"
+        'SELECT * FROM "t" as "t"', dialect="snowflake"
     )
 
     # The following call to run_query won't return dataframes and so we're expected to
@@ -148,18 +166,24 @@ def test_adapter_dispatch(sushi_test_project: Project, runtime_renderer: t.Calla
     context = sushi_test_project.context
     renderer = runtime_renderer(context)
     assert renderer("{{ adapter.dispatch('current_engine', 'customers')() }}") == "duckdb"
+    assert renderer("{{ adapter.dispatch('current_timestamp')() }}") == "now()"
+    assert renderer("{{ adapter.dispatch('current_timestamp', 'dbt')() }}") == "now()"
 
     with pytest.raises(ConfigError, match=r"Macro 'current_engine'.*was not found."):
         renderer("{{ adapter.dispatch('current_engine')() }}")
 
 
+@pytest.mark.parametrize("project_dialect", ["duckdb", "bigquery"])
 def test_adapter_map_snapshot_tables(
-    sushi_test_project: Project, runtime_renderer: t.Callable, mocker: MockerFixture
+    sushi_test_project: Project,
+    runtime_renderer: t.Callable,
+    mocker: MockerFixture,
+    project_dialect: str,
 ):
     snapshot_mock = mocker.Mock()
     snapshot_mock.name = '"memory"."test_db"."test_model"'
     snapshot_mock.version = "1"
-    snapshot_mock.is_symbolic = False
+    snapshot_mock.is_embedded = False
     snapshot_mock.table_name.return_value = '"memory"."sqlmesh"."test_db__test_model"'
     snapshot_mock.snapshot_id = SnapshotId(
         name='"memory"."test_db"."test_model"', identifier="12345"
@@ -175,6 +199,7 @@ def test_adapter_map_snapshot_tables(
         test_model=BaseRelation.create(schema="test_db", identifier="test_model"),
         foo_bar=BaseRelation.create(schema="foo", identifier="bar"),
         default_catalog="memory",
+        dialect=project_dialect,
     )
 
     engine_adapter.create_schema("foo")
@@ -187,18 +212,24 @@ def test_adapter_map_snapshot_tables(
         table_name="foo.bar", columns_to_types={"col": exp.DataType.build("int")}
     )
 
+    expected_test_model_table_name = parse_one('"memory"."sqlmesh"."test_db__test_model"').sql(
+        dialect=project_dialect
+    )
+
     assert (
         renderer(
             "{{ adapter.get_relation(database=none, schema='test_db', identifier='test_model') }}"
         )
-        == '"memory"."sqlmesh"."test_db__test_model"'
+        == expected_test_model_table_name
     )
 
     assert "baz" in renderer("{{ run_query('SELECT * FROM test_db.test_model') }}")
 
+    expected_foo_bar_table_name = parse_one('"memory"."foo"."bar"').sql(dialect=project_dialect)
+
     assert (
         renderer("{{ adapter.get_relation(database=none, schema='foo', identifier='bar') }}")
-        == '"memory"."foo"."bar"'
+        == expected_foo_bar_table_name
     )
 
     assert renderer("{{ adapter.resolve_schema(test_model) }}") == "sqlmesh"
@@ -228,3 +259,14 @@ def test_feature_flag_scd_type_2(copy_to_temp_path, caplog):
             "Skipping loading Snapshot (SCD Type 2) models due to the feature flag disabling this feature"
             in caplog.text
         )
+
+
+def test_quote_as_configured():
+    adapter = ParsetimeAdapter(
+        JinjaMacroRegistry(),
+        project_dialect="duckdb",
+        quote_policy=Policy(schema=False, identifier=True),
+    )
+    adapter.quote_as_configured("foo", "identifier") == '"foo"'
+    adapter.quote_as_configured("foo", "schema") == "foo"
+    adapter.quote_as_configured("foo", "database") == "foo"
