@@ -1,3 +1,5 @@
+import agate
+from datetime import datetime
 import json
 import logging
 import typing as t
@@ -7,24 +9,25 @@ from unittest.mock import patch
 import pytest
 from dbt.adapters.base import BaseRelation
 from dbt.exceptions import CompilationError
-from freezegun import freeze_time
+import time_machine
 from pytest_mock.plugin import MockerFixture
 from sqlglot import exp, parse_one
-
 from sqlmesh.core import dialect as d
 from sqlmesh.core.audit import StandaloneAudit
 from sqlmesh.core.context import Context
+from sqlmesh.core.console import get_console
 from sqlmesh.core.model import (
     EmbeddedKind,
     FullKind,
     IncrementalByTimeRangeKind,
     IncrementalByUniqueKeyKind,
     IncrementalUnmanagedKind,
+    ManagedKind,
     SqlModel,
     ViewKind,
 )
 from sqlmesh.core.model.kind import SCDType2ByColumnKind, SCDType2ByTimeKind
-from sqlmesh.core.state_sync.engine_adapter import _snapshot_to_json
+from sqlmesh.core.state_sync.db.snapshot import _snapshot_to_json
 from sqlmesh.dbt.builtin import _relation_info_to_relation
 from sqlmesh.dbt.column import (
     ColumnConfig,
@@ -35,8 +38,8 @@ from sqlmesh.dbt.context import DbtContext
 from sqlmesh.dbt.model import Materialization, ModelConfig
 from sqlmesh.dbt.project import Project
 from sqlmesh.dbt.relation import Policy
-from sqlmesh.dbt.seed import SeedConfig
-from sqlmesh.dbt.target import BigQueryConfig, DuckDbConfig, SnowflakeConfig
+from sqlmesh.dbt.seed import SeedConfig, Integer
+from sqlmesh.dbt.target import BigQueryConfig, DuckDbConfig, SnowflakeConfig, ClickhouseConfig
 from sqlmesh.dbt.test import TestConfig
 from sqlmesh.utils.errors import ConfigError, MacroEvalError, SQLMeshError
 
@@ -66,6 +69,27 @@ def test_model_name():
     )
 
 
+def test_materialization():
+    context = DbtContext()
+    context.project_name = "Test"
+    context.target = DuckDbConfig(name="target", schema="foo")
+
+    with patch.object(get_console(), "log_warning") as mock_logger:
+        model_config = ModelConfig(
+            name="model", alias="model", schema="schema", materialized="materialized_view"
+        )
+
+    assert (
+        "SQLMesh does not support the 'materialized_view' model materialization. Falling back to the 'view' materialization."
+        in mock_logger.call_args[0][0]
+    )
+    assert model_config.materialized == "view"
+
+    # clickhouse "dictionary" materialization
+    with pytest.raises(ConfigError):
+        ModelConfig(name="model", alias="model", schema="schema", materialized="dictionary")
+
+
 def test_model_kind():
     context = DbtContext()
     context.project_name = "Test"
@@ -100,6 +124,20 @@ def test_model_kind():
         execution_time_as_valid_from=True,
         dialect="duckdb",
     )
+    assert ModelConfig(
+        materialized=Materialization.SNAPSHOT,
+        unique_key=["id"],
+        strategy="check",
+        check_cols=["foo"],
+        dialect="bigquery",
+    ).model_kind(context) == SCDType2ByColumnKind(
+        unique_key=["id"],
+        valid_from_name="dbt_valid_from",
+        valid_to_name="dbt_valid_to",
+        columns=["foo"],
+        execution_time_as_valid_from=True,
+        dialect="bigquery",
+    )
 
     assert ModelConfig(materialized=Materialization.INCREMENTAL, time_column="foo").model_kind(
         context
@@ -118,9 +156,12 @@ def test_model_kind():
         time_column="foo", dialect="duckdb", forward_only=True
     )
     assert ModelConfig(
-        materialized=Materialization.INCREMENTAL, time_column="foo", unique_key=["bar"]
+        materialized=Materialization.INCREMENTAL,
+        time_column="foo",
+        unique_key=["bar"],
+        dialect="bigquery",
     ).model_kind(context) == IncrementalByTimeRangeKind(
-        time_column="foo", dialect="duckdb", forward_only=True
+        time_column="foo", dialect="bigquery", forward_only=True
     )
 
     assert ModelConfig(
@@ -129,6 +170,24 @@ def test_model_kind():
         unique_key=["bar"], dialect="duckdb", forward_only=True, disable_restatement=False
     )
 
+    dbt_incremental_predicate = "DBT_INTERNAL_DEST.session_start > dateadd(day, -7, current_date)"
+    expected_sqlmesh_predicate = parse_one(
+        "__MERGE_TARGET__.session_start > DATEADD(day, -7, CURRENT_DATE)"
+    )
+    ModelConfig(
+        materialized=Materialization.INCREMENTAL,
+        unique_key=["bar"],
+        incremental_strategy="merge",
+        dialect="postgres",
+        merge_filter=[dbt_incremental_predicate],
+    ).model_kind(context) == IncrementalByUniqueKeyKind(
+        unique_key=["bar"],
+        dialect="postgres",
+        forward_only=True,
+        disable_restatement=False,
+        merge_filter=expected_sqlmesh_predicate,
+    )
+
     assert ModelConfig(materialized=Materialization.INCREMENTAL, unique_key=["bar"]).model_kind(
         context
     ) == IncrementalByUniqueKeyKind(
@@ -167,8 +226,13 @@ def test_model_kind():
         unique_key=["bar"],
         disable_restatement=True,
         full_refresh=False,
+        auto_restatement_cron="0 0 * * *",
     ).model_kind(context) == IncrementalByUniqueKeyKind(
-        unique_key=["bar"], dialect="duckdb", forward_only=True, disable_restatement=True
+        unique_key=["bar"],
+        dialect="duckdb",
+        forward_only=True,
+        disable_restatement=True,
+        auto_restatement_cron="0 0 * * *",
     )
 
     assert ModelConfig(
@@ -194,6 +258,22 @@ def test_model_kind():
         forward_only=False,
     ).model_kind(context) == IncrementalByTimeRangeKind(time_column="foo", dialect="duckdb")
 
+    assert ModelConfig(
+        materialized=Materialization.INCREMENTAL,
+        time_column="foo",
+        incremental_strategy="insert_overwrite",
+        partition_by={"field": "bar"},
+        forward_only=False,
+        auto_restatement_cron="0 0 * * *",
+        auto_restatement_intervals=3,
+    ).model_kind(context) == IncrementalByTimeRangeKind(
+        time_column="foo",
+        dialect="duckdb",
+        forward_only=False,
+        auto_restatement_cron="0 0 * * *",
+        auto_restatement_intervals=3,
+    )
+
     assert ModelConfig(
         materialized=Materialization.INCREMENTAL,
         incremental_strategy="insert_overwrite",
@@ -210,6 +290,10 @@ def test_model_kind():
         materialized=Materialization.INCREMENTAL, incremental_strategy="append"
     ).model_kind(context) == IncrementalUnmanagedKind(disable_restatement=False)
 
+    assert ModelConfig(
+        materialized=Materialization.INCREMENTAL, incremental_strategy="append", full_refresh=None
+    ).model_kind(context) == IncrementalUnmanagedKind(disable_restatement=False)
+
     assert ModelConfig(
         materialized=Materialization.INCREMENTAL,
         incremental_strategy="insert_overwrite",
@@ -246,6 +330,21 @@ def test_model_kind():
         insert_overwrite=True, disable_restatement=True
     )
 
+    assert ModelConfig(
+        materialized=Materialization.INCREMENTAL,
+        incremental_strategy="insert_overwrite",
+        auto_restatement_cron="0 0 * * *",
+    ).model_kind(context) == IncrementalUnmanagedKind(
+        insert_overwrite=True, auto_restatement_cron="0 0 * * *", disable_restatement=False
+    )
+
+    assert (
+        ModelConfig(materialized=Materialization.DYNAMIC_TABLE, target_lag="1 hour").model_kind(
+            context
+        )
+        == ManagedKind()
+    )
+
     with pytest.raises(ConfigError):
         ModelConfig(
             materialized=Materialization.INCREMENTAL,
@@ -286,6 +385,24 @@ def test_model_kind_snapshot_bigquery():
         dialect="bigquery",
     )
 
+    # time_data_type is bigquery version even though model dialect is DuckDB
+    # because model target is BigQuery
+    assert ModelConfig(
+        materialized=Materialization.SNAPSHOT,
+        unique_key=["id"],
+        updated_at="updated_at",
+        strategy="timestamp",
+        dialect="duckdb",
+    ).model_kind(context) == SCDType2ByTimeKind(
+        unique_key=["id"],
+        valid_from_name="dbt_valid_from",
+        valid_to_name="dbt_valid_to",
+        updated_at_as_valid_from=True,
+        updated_at_name="updated_at",
+        time_data_type=exp.DataType.build("TIMESTAMPTZ"),  # bigquery version
+        dialect="duckdb",
+    )
+
 
 def test_model_columns():
     model = ModelConfig(
@@ -307,9 +424,9 @@ def test_model_columns():
     )
 
     expected_column_types = {
-        "ADDRESS": parse_one("text", into=exp.DataType),
-        "ZIPCODE": parse_one("varchar(5)", into=exp.DataType),
-        "DATE": parse_one("timestamp_ntz", into=exp.DataType, dialect="snowflake"),
+        "ADDRESS": exp.DataType.build("text"),
+        "ZIPCODE": exp.DataType.build("varchar(5)"),
+        "DATE": exp.DataType.build("timestamp_ntz", dialect="snowflake"),
     }
     expected_column_descriptions = {
         "ADDRESS": "Business address",
@@ -340,14 +457,14 @@ def test_seed_columns():
                 name="address", data_type="text", description="Business address"
             ),
             "zipcode": ColumnConfig(
-                name="zipcode", data_type="varchar(5)", description="Business zipcode"
+                name="zipcode", data_type="text", description="Business zipcode"
             ),
         },
     )
 
     expected_column_types = {
-        "address": parse_one("text", into=exp.DataType),
-        "zipcode": parse_one("varchar(5)", into=exp.DataType),
+        "address": exp.DataType.build("text"),
+        "zipcode": exp.DataType.build("text"),
     }
     expected_column_descriptions = {
         "address": "Business address",
@@ -362,12 +479,45 @@ def test_seed_columns():
     assert sqlmesh_seed.column_descriptions == expected_column_descriptions
 
 
+def test_seed_column_types():
+    seed = SeedConfig(
+        name="foo",
+        package="package",
+        path=Path("examples/sushi_dbt/seeds/waiter_names.csv"),
+        column_types={
+            "address": "text",
+            "zipcode": "text",
+        },
+        columns={
+            "zipcode": ColumnConfig(name="zipcode", description="Business zipcode"),
+        },
+        quote_columns=True,
+    )
+
+    expected_column_types = {
+        "address": exp.DataType.build("text"),
+        "zipcode": exp.DataType.build("text"),
+    }
+    expected_column_descriptions = {
+        "zipcode": "Business zipcode",
+    }
+
+    context = DbtContext()
+    context.project_name = "Foo"
+    context.target = DuckDbConfig(name="target", schema="test")
+    sqlmesh_seed = seed.to_sqlmesh(context)
+
+    assert sqlmesh_seed.columns_to_types == expected_column_types
+    assert sqlmesh_seed.column_descriptions == expected_column_descriptions
+
+
 def test_seed_column_inference(tmp_path):
     seed_csv = tmp_path / "seed.csv"
     with open(seed_csv, "w", encoding="utf-8") as fd:
         fd.write("int_col,double_col,datetime_col,date_col,boolean_col,text_col\n")
         fd.write("1,1.2,2021-01-01 00:00:00,2021-01-01,true,foo\n")
         fd.write("2,2.3,2021-01-02 00:00:00,2021-01-02,false,bar\n")
+        fd.write("null,,null,,,null\n")
 
     seed = SeedConfig(
         name="test_model",
@@ -389,6 +539,128 @@ def test_seed_column_inference(tmp_path):
     }
 
 
+def test_seed_partial_column_inference(tmp_path):
+    seed_csv = tmp_path / "seed.csv"
+    with open(seed_csv, "w", encoding="utf-8") as fd:
+        fd.write("int_col,double_col,datetime_col,boolean_col\n")
+        fd.write("1,1.2,2021-01-01 00:00:00,true\n")
+        fd.write("2,2.3,2021-01-02 00:00:00,false\n")
+        fd.write("null,,null,\n")
+
+    seed = SeedConfig(
+        name="test_model",
+        package="package",
+        path=Path(seed_csv),
+        column_types={
+            "double_col": "double",
+        },
+        columns={
+            "int_col": ColumnConfig(
+                name="int_col", data_type="int", description="Description with type."
+            ),
+            "datetime_col": ColumnConfig(
+                name="datetime_col", description="Description without type."
+            ),
+            "boolean_col": ColumnConfig(name="boolean_col"),
+        },
+    )
+
+    expected_column_types = {
+        "int_col": exp.DataType.build("int"),
+        "double_col": exp.DataType.build("double"),
+        "datetime_col": exp.DataType.build("datetime"),
+        "boolean_col": exp.DataType.build("boolean"),
+    }
+
+    expected_column_descriptions = {
+        "int_col": "Description with type.",
+        "datetime_col": "Description without type.",
+    }
+
+    context = DbtContext()
+    context.project_name = "Foo"
+    context.target = DuckDbConfig(name="target", schema="test")
+    sqlmesh_seed = seed.to_sqlmesh(context)
+    assert sqlmesh_seed.columns_to_types == expected_column_types
+    assert sqlmesh_seed.column_descriptions == expected_column_descriptions
+
+    # Check that everything still lines up
+    seed_df = next(sqlmesh_seed.render_seed())
+    assert list(seed_df.columns) == list(sqlmesh_seed.columns_to_types.keys())
+
+
+def test_seed_column_order(tmp_path):
+    seed_csv = tmp_path / "seed.csv"
+
+    with open(seed_csv, "w", encoding="utf-8") as fd:
+        fd.writelines("\n".join(["id,name", "0,Toby", "1,Tyson", "2,Ryan"]))
+
+    seed = SeedConfig(
+        name="test_model",
+        package="package",
+        path=Path(seed_csv),
+        columns={
+            "id": ColumnConfig(name="id"),
+            "name": ColumnConfig(name="name", data_type="varchar"),
+        },
+    )
+
+    context = DbtContext()
+    context.project_name = "Foo"
+    context.target = DuckDbConfig(name="target", schema="test")
+    sqlmesh_seed = seed.to_sqlmesh(context)
+
+    # Check that everything still lines up
+    seed_df = next(sqlmesh_seed.render_seed())
+    assert list(seed_df.columns) == list(sqlmesh_seed.columns_to_types.keys())
+
+
+def test_agate_integer_cast():
+    agate_integer = Integer(null_values=("null", ""))
+    assert agate_integer.cast("1") == 1
+    assert agate_integer.cast(1) == 1
+    assert agate_integer.cast("null") is None
+    assert agate_integer.cast("") is None
+
+    with pytest.raises(agate.exceptions.CastError):
+        agate_integer.cast("1.2")
+
+    with pytest.raises(agate.exceptions.CastError):
+        agate_integer.cast(1.2)
+
+    with pytest.raises(agate.exceptions.CastError):
+        agate_integer.cast(datetime.now())
+
+
+@pytest.mark.xdist_group("dbt_manifest")
+def test_model_dialect(sushi_test_project: Project, assert_exp_eq):
+    model_config = ModelConfig(
+        name="model",
+        package_name="package",
+        schema="sushi",
+        alias="table",
+        sql="SELECT 1 AS `one` FROM {{ schema }}",
+    )
+    context = sushi_test_project.context
+
+    # cannot parse model sql without specifying bigquery dialect
+    with pytest.raises(ConfigError):
+        model_config.to_sqlmesh(context).render_query_or_raise().sql()
+
+    model_config = ModelConfig(
+        name="model",
+        package_name="package",
+        schema="sushi",
+        alias="table",
+        sql="SELECT 1 AS `one` FROM {{ schema }}",
+        dialect="bigquery",
+    )
+    assert_exp_eq(
+        model_config.to_sqlmesh(context).render_query_or_raise().sql(),
+        'SELECT 1 AS "one" FROM "sushi" AS "sushi"',
+    )
+
+
 @pytest.mark.xdist_group("dbt_manifest")
 @pytest.mark.parametrize(
     "model_fqn", ['"memory"."sushi"."waiters"', '"memory"."sushi"."waiter_names"']
@@ -495,7 +767,32 @@ def test_test_this(assert_exp_eq, sushi_test_project: Project):
     context = sushi_test_project.context
     audit = t.cast(StandaloneAudit, test_config.to_sqlmesh(context))
     assert_exp_eq(
-        audit.render_query(audit).sql(),
+        audit.render_audit_query().sql(),
+        'SELECT 1 AS "one" FROM "test" AS "test"',
+    )
+
+
+@pytest.mark.xdist_group("dbt_manifest")
+def test_test_dialect(assert_exp_eq, sushi_test_project: Project):
+    test_config = TestConfig(
+        name="test",
+        alias="alias",
+        database="database",
+        schema_="schema",
+        standalone=True,
+        sql="SELECT 1 AS `one` FROM {{ this.identifier }}",
+    )
+    context = sushi_test_project.context
+
+    # can't parse test sql without specifying bigquery as default dialect
+    with pytest.raises(ConfigError):
+        audit = t.cast(StandaloneAudit, test_config.to_sqlmesh(context))
+        audit.render_audit_query().sql()
+
+    test_config.dialect_ = "bigquery"
+    audit = t.cast(StandaloneAudit, test_config.to_sqlmesh(context))
+    assert_exp_eq(
+        audit.render_audit_query().sql(),
         'SELECT 1 AS "one" FROM "test" AS "test"',
     )
 
@@ -828,6 +1125,7 @@ def test_is_incremental(sushi_test_project: Project, assert_exp_eq, mocker):
 
     snapshot = mocker.Mock()
     snapshot.intervals = [1]
+    snapshot.is_incremental = True
 
     assert_exp_eq(
         model_config.to_sqlmesh(context).render_query_or_raise(snapshot=snapshot).sql(),
@@ -835,6 +1133,32 @@ def test_is_incremental(sushi_test_project: Project, assert_exp_eq, mocker):
     )
 
 
+@pytest.mark.xdist_group("dbt_manifest")
+def test_is_incremental_non_incremental_model(sushi_test_project: Project, assert_exp_eq, mocker):
+    model_config = ModelConfig(
+        name="model",
+        package_name="package",
+        schema="sushi",
+        alias="some_table",
+        sql="""
+        SELECT 1 AS one FROM tbl_a
+        {% if is_incremental() %}
+        WHERE ds > (SELECT MAX(ds) FROM model)
+        {% endif %}
+        """,
+    )
+    context = sushi_test_project.context
+
+    snapshot = mocker.Mock()
+    snapshot.intervals = [1]
+    snapshot.is_incremental = False
+
+    assert_exp_eq(
+        model_config.to_sqlmesh(context).render_query_or_raise(snapshot=snapshot).sql(),
+        'SELECT 1 AS "one" FROM "tbl_a" AS "tbl_a"',
+    )
+
+
 @pytest.mark.xdist_group("dbt_manifest")
 def test_dbt_max_partition(sushi_test_project: Project, assert_exp_eq, mocker: MockerFixture):
     model_config = ModelConfig(
@@ -864,7 +1188,7 @@ def test_dbt_max_partition(sushi_test_project: Project, assert_exp_eq, mocker: M
 JINJA_STATEMENT_BEGIN;
 {% if is_incremental() %}
   DECLARE _dbt_max_partition DATETIME DEFAULT (
-    COALESCE((SELECT MAX(PARSE_DATETIME('%Y%m', partition_id)) FROM `{{ target.database }}.{{ adapter.resolve_schema(this) }}.INFORMATION_SCHEMA.PARTITIONS` WHERE table_name = '{{ adapter.resolve_identifier(this) }}' AND NOT partition_id IS NULL AND partition_id <> '__NULL__'), CAST('1970-01-01' AS DATETIME))
+    COALESCE((SELECT MAX(PARSE_DATETIME('%Y%m', partition_id)) FROM `{{ target.database }}`.`{{ adapter.resolve_schema(this) }}`.`INFORMATION_SCHEMA.PARTITIONS` AS PARTITIONS WHERE table_name = '{{ adapter.resolve_identifier(this) }}' AND NOT partition_id IS NULL AND partition_id <> '__NULL__'), CAST('1970-01-01' AS DATETIME))
   );
 {% endif %}
 JINJA_END;""".strip()
@@ -912,6 +1236,60 @@ def test_bigquery_physical_properties(sushi_test_project: Project, mocker: Mocke
     }
 
 
+@pytest.mark.xdist_group("dbt_manifest")
+def test_clickhouse_properties(mocker: MockerFixture):
+    context = DbtContext(target_name="production")
+    context._project_name = "Foo"
+    context._target = ClickhouseConfig(name="production")
+    model_config = ModelConfig(
+        name="model",
+        alias="model",
+        schema="test",
+        package_name="package",
+        materialized="incremental",
+        incremental_strategy="delete+insert",
+        incremental_predicates=["ds > (SELECT MAX(ds) FROM model)"],
+        query_settings={"QUERY_SETTING": "value"},
+        sharding_key="rand()",
+        engine="MergeTree()",
+        partition_by=["toMonday(ds)", "partition_col"],
+        order_by=["toStartOfWeek(ds)", "order_col"],
+        primary_key=["ds", "primary_key_col"],
+        ttl="time + INTERVAL 1 WEEK",
+        settings={"SETTING": "value"},
+        sql="""SELECT 1 AS one, ds FROM foo""",
+    )
+
+    with patch.object(get_console(), "log_warning") as mock_logger:
+        model_to_sqlmesh = model_config.to_sqlmesh(context)
+
+    assert [call[0][0] for call in mock_logger.call_args_list] == [
+        "The 'delete+insert' incremental strategy is not supported - SQLMesh will use the temp table/partition swap strategy.",
+        "SQLMesh does not support 'incremental_predicates' - they will not be applied.",
+        "SQLMesh does not support the 'query_settings' model configuration parameter. Specify the query settings directly in the model query.",
+        "SQLMesh does not support the 'sharding_key' model configuration parameter or distributed materializations.",
+        "Using unmanaged incremental materialization for model '`test`.`model`'. Some features might not be available. Consider adding either a time_column ('delete+insert', 'insert_overwrite') or a unique_key ('merge', 'none') configuration to mitigate this.",
+    ]
+
+    assert [e.sql("clickhouse") for e in model_to_sqlmesh.partitioned_by] == [
+        'toMonday("ds")',
+        '"partition_col"',
+    ]
+    assert model_to_sqlmesh.storage_format == "MergeTree()"
+
+    physical_properties = model_to_sqlmesh.physical_properties
+    assert [e.sql("clickhouse", identify=True) for e in physical_properties["order_by"]] == [
+        'toStartOfWeek("ds")',
+        '"order_col"',
+    ]
+    assert [e.sql("clickhouse", identify=True) for e in physical_properties["primary_key"]] == [
+        '"ds"',
+        '"primary_key_col"',
+    ]
+    assert physical_properties["ttl"].sql("clickhouse") == "time + INTERVAL 1 WEEK"
+    assert physical_properties["SETTING"].sql("clickhouse") == "value"
+
+
 @pytest.mark.xdist_group("dbt_manifest")
 def test_snapshot_json_payload():
     sushi_context = Context(paths=["tests/fixtures/dbt/sushi_test"])
@@ -928,7 +1306,7 @@ def test_snapshot_json_payload():
 
 
 @pytest.mark.xdist_group("dbt_manifest")
-@freeze_time("2023-01-08 00:00:00")
+@time_machine.travel("2023-01-08 00:00:00 UTC")
 def test_dbt_package_macros(sushi_test_project: Project):
     context = sushi_test_project.context
 
@@ -1000,7 +1378,7 @@ def test_model_cluster_by():
         sql="SELECT * FROM baz",
         materialized=Materialization.TABLE.value,
     )
-    assert model.to_sqlmesh(context).clustered_by == ["BAR"]
+    assert model.to_sqlmesh(context).clustered_by == [exp.to_column('"BAR"')]
 
     model = ModelConfig(
         name="model",
@@ -1011,4 +1389,122 @@ def test_model_cluster_by():
         sql="SELECT * FROM baz",
         materialized=Materialization.TABLE.value,
     )
-    assert model.to_sqlmesh(context).clustered_by == ["BAR", "QUX"]
+    assert model.to_sqlmesh(context).clustered_by == [
+        exp.to_column('"BAR"'),
+        exp.to_column('"QUX"'),
+    ]
+
+
+def test_snowflake_dynamic_table():
+    context = DbtContext()
+    context._target = SnowflakeConfig(
+        name="target",
+        schema="test",
+        database="test",
+        account="account",
+        user="user",
+        password="password",
+    )
+
+    model = ModelConfig(
+        name="model",
+        alias="model",
+        package_name="package",
+        target_schema="test",
+        sql="SELECT * FROM baz",
+        materialized=Materialization.DYNAMIC_TABLE.value,
+        target_lag="1 hour",
+        snowflake_warehouse="SMALL",
+    )
+
+    as_sqlmesh = model.to_sqlmesh(context)
+    assert as_sqlmesh.kind == ManagedKind()
+    assert as_sqlmesh.physical_properties == {
+        "target_lag": exp.Literal.string("1 hour"),
+        "warehouse": exp.Literal.string("SMALL"),
+    }
+
+    # both target_lag and snowflake_warehouse are required properties
+    # https://docs.getdbt.com/reference/resource-configs/snowflake-configs#dynamic-tables
+    for required_property in ["target_lag", "snowflake_warehouse"]:
+        with pytest.raises(ConfigError, match=r".*must be set for dynamic tables"):
+            model.copy(update={required_property: None}).to_sqlmesh(context)
+
+
+@pytest.mark.xdist_group("dbt_manifest")
+def test_refs_in_jinja_globals(sushi_test_project: Project, mocker: MockerFixture):
+    context = sushi_test_project.context
+
+    sqlmesh_model = t.cast(
+        SqlModel,
+        sushi_test_project.packages["sushi"].models["simple_model_b"].to_sqlmesh(context),
+    )
+    assert set(sqlmesh_model.jinja_macros.global_objs["refs"].keys()) == {"simple_model_a"}  # type: ignore
+
+    sqlmesh_model = t.cast(
+        SqlModel,
+        sushi_test_project.packages["sushi"].models["top_waiters"].to_sqlmesh(context),
+    )
+    assert set(sqlmesh_model.jinja_macros.global_objs["refs"].keys()) == {  # type: ignore
+        "waiter_revenue_by_day",
+        "sushi.waiter_revenue_by_day",
+    }
+
+
+def test_dbt_incremental_allow_partials_by_default():
+    context = DbtContext()
+    context._target = SnowflakeConfig(
+        name="target",
+        schema="test",
+        database="test",
+        account="account",
+        user="user",
+        password="password",
+    )
+
+    model = ModelConfig(
+        name="model",
+        alias="model",
+        package_name="package",
+        target_schema="test",
+        sql="SELECT * FROM baz",
+        materialized=Materialization.TABLE.value,
+    )
+    assert model.allow_partials is None
+    assert not model.to_sqlmesh(context).allow_partials
+
+    model.materialized = Materialization.INCREMENTAL.value
+    assert model.allow_partials is None
+    assert model.to_sqlmesh(context).allow_partials
+
+    model.allow_partials = True
+    assert model.to_sqlmesh(context).allow_partials
+
+    model.allow_partials = False
+    assert not model.to_sqlmesh(context).allow_partials
+
+
+def test_grain():
+    context = DbtContext()
+    context._target = SnowflakeConfig(
+        name="target",
+        schema="test",
+        database="test",
+        account="account",
+        user="user",
+        password="password",
+    )
+
+    model = ModelConfig(
+        name="model",
+        alias="model",
+        package_name="package",
+        target_schema="test",
+        sql="SELECT * FROM baz",
+        materialized=Materialization.TABLE.value,
+        grain=["id_a", "id_b"],
+    )
+    assert model.to_sqlmesh(context).grains == [exp.to_column("id_a"), exp.to_column("id_b")]
+
+    model.grain = "id_a"
+    assert model.to_sqlmesh(context).grains == [exp.to_column("id_a")]
