@@ -5,7 +5,6 @@
 }}
 
 SELECT DISTINCT
-  waiter_id::INT AS waiter_id,
-  ds::TEXT AS ds
-FROM {{ source('raw', 'orders') }}
+   *
+FROM {{ source('streaming', 'orders') }}
 {{ incremental_by_time('ds', 'ds') }}
