@@ -9,7 +9,7 @@ import typing as t
 from unittest import mock
 
 import pytest
-from freezegun import freeze_time
+import time_machine
 from pytest_mock.plugin import MockerFixture
 from sqlglot import exp
 
@@ -23,6 +23,7 @@ from sqlmesh.integrations.github.cicd.controller import (
     GithubCheckStatus,
     GithubController,
 )
+from sqlmesh.utils.errors import CICDBotError
 from tests.integrations.github.cicd.fixtures import MockIssueComment
 
 pytest_plugins = ["tests.integrations.github.cicd.fixtures"]
@@ -51,7 +52,7 @@ def get_columns(
     return controller._context.engine_adapter.columns(table)
 
 
-@freeze_time("2023-01-01 15:00:00")
+@time_machine.travel("2023-01-01 15:00:00 UTC")
 def test_merge_pr_has_non_breaking_change(
     github_client,
     make_controller,
@@ -164,7 +165,7 @@ def test_merge_pr_has_non_breaking_change(
 
 +++ 
 
-@@ -11,7 +11,8 @@
+@@ -15,7 +15,8 @@
 
  SELECT
    CAST(o.waiter_id AS INT) AS waiter_id,
@@ -227,10 +228,11 @@ def test_merge_pr_has_non_breaking_change(
     assert len(created_comments) == 1
     assert (
         created_comments[0].body
-        == f"""**SQLMesh Bot Info**
-- PR Virtual Data Environment: hello_world_2
+        == f""":robot: **SQLMesh Bot Info** :robot:
+- :eyes: To **review** this PR's changes, use virtual data environment:
+  - `hello_world_2`
 <details>
-  <summary>Prod Plan Being Applied</summary>
+  <summary>:ship: Prod Plan Being Applied</summary>
 
 {expected_prod_plan_summary}
 </details>
@@ -246,7 +248,7 @@ def test_merge_pr_has_non_breaking_change(
         )
 
 
-@freeze_time("2023-01-01 15:00:00")
+@time_machine.travel("2023-01-01 15:00:00 UTC")
 def test_merge_pr_has_non_breaking_change_diff_start(
     github_client,
     make_controller,
@@ -360,7 +362,7 @@ def test_merge_pr_has_non_breaking_change_diff_start(
 
 +++ 
 
-@@ -11,7 +11,8 @@
+@@ -15,7 +15,8 @@
 
  SELECT
    CAST(o.waiter_id AS INT) AS waiter_id,
@@ -377,8 +379,8 @@ def test_merge_pr_has_non_breaking_change_diff_start(
 - `sushi.top_waiters`
 
 
-**Models needing backfill (missing dates):**
-* `sushi.waiter_revenue_by_day`: 2022-12-25 - 2022-12-28
+**Models needing backfill:**
+* `sushi.waiter_revenue_by_day`: [2022-12-25 - 2022-12-28]
 """
     assert prod_plan_preview_checks_runs[2]["output"]["summary"] == expected_prod_plan
 
@@ -425,10 +427,11 @@ def test_merge_pr_has_non_breaking_change_diff_start(
     assert len(created_comments) == 1
     assert (
         created_comments[0].body
-        == f"""**SQLMesh Bot Info**
-- PR Virtual Data Environment: hello_world_2
+        == f""":robot: **SQLMesh Bot Info** :robot:
+- :eyes: To **review** this PR's changes, use virtual data environment:
+  - `hello_world_2`
 <details>
-  <summary>Prod Plan Being Applied</summary>
+  <summary>:ship: Prod Plan Being Applied</summary>
 
 {expected_prod_plan}
 </details>
@@ -444,7 +447,7 @@ def test_merge_pr_has_non_breaking_change_diff_start(
         )
 
 
-@freeze_time("2023-01-01 15:00:00")
+@time_machine.travel("2023-01-01 15:00:00 UTC")
 def test_merge_pr_has_non_breaking_change_no_categorization(
     github_client,
     make_controller,
@@ -507,7 +510,8 @@ def test_merge_pr_has_non_breaking_change_no_categorization(
     github_output_file = tmp_path / "github_output.txt"
 
     with mock.patch.dict(os.environ, {"GITHUB_OUTPUT": str(github_output_file)}):
-        command._run_all(controller)
+        with pytest.raises(CICDBotError):
+            command._run_all(controller)
 
     assert "SQLMesh - Run Unit Tests" in controller._check_run_mapping
     test_checks_runs = controller._check_run_mapping["SQLMesh - Run Unit Tests"].all_kwargs
@@ -735,9 +739,9 @@ def test_merge_pr_has_no_changes(
     assert len(created_comments) == 1
     assert (
         created_comments[0].body
-        == f"""**SQLMesh Bot Info**
+        == f""":robot: **SQLMesh Bot Info** :robot:
 <details>
-  <summary>Prod Plan Being Applied</summary>
+  <summary>:ship: Prod Plan Being Applied</summary>
 
 {expected_prod_plan_summary}
 </details>
@@ -753,7 +757,7 @@ def test_merge_pr_has_no_changes(
         )
 
 
-@freeze_time("2023-01-01 15:00:00")
+@time_machine.travel("2023-01-01 15:00:00 UTC")
 def test_no_merge_since_no_deploy_signal(
     github_client,
     make_controller,
@@ -864,7 +868,7 @@ def test_no_merge_since_no_deploy_signal(
 
 +++ 
 
-@@ -11,7 +11,8 @@
+@@ -15,7 +15,8 @@
 
  SELECT
    CAST(o.waiter_id AS INT) AS waiter_id,
@@ -921,8 +925,9 @@ def test_no_merge_since_no_deploy_signal(
     assert len(created_comments) == 1
     assert (
         created_comments[0].body
-        == """**SQLMesh Bot Info**
-- PR Virtual Data Environment: hello_world_2"""
+        == """:robot: **SQLMesh Bot Info** :robot:
+- :eyes: To **review** this PR's changes, use virtual data environment:
+  - `hello_world_2`"""
     )
 
     with open(github_output_file, "r", encoding="utf-8") as f:
@@ -933,7 +938,7 @@ def test_no_merge_since_no_deploy_signal(
         )
 
 
-@freeze_time("2023-01-01 15:00:00")
+@time_machine.travel("2023-01-01 15:00:00 UTC")
 def test_no_merge_since_no_deploy_signal_no_approvers_defined(
     github_client,
     make_controller,
@@ -1044,7 +1049,7 @@ def test_no_merge_since_no_deploy_signal_no_approvers_defined(
 
 +++ 
 
-@@ -11,7 +11,8 @@
+@@ -15,7 +15,8 @@
 
  SELECT
    CAST(o.waiter_id AS INT) AS waiter_id,
@@ -1061,8 +1066,8 @@ def test_no_merge_since_no_deploy_signal_no_approvers_defined(
 - `sushi.top_waiters`
 
 
-**Models needing backfill (missing dates):**
-* `sushi.waiter_revenue_by_day`: 2022-12-25 - 2022-12-29
+**Models needing backfill:**
+* `sushi.waiter_revenue_by_day`: [2022-12-25 - 2022-12-29]
 """
     assert prod_plan_preview_checks_runs[2]["output"]["title"] == "Prod Plan Preview"
     assert prod_plan_preview_checks_runs[2]["output"]["summary"] == expected_prod_plan
@@ -1080,8 +1085,9 @@ def test_no_merge_since_no_deploy_signal_no_approvers_defined(
     assert len(created_comments) == 1
     assert (
         created_comments[0].body
-        == """**SQLMesh Bot Info**
-- PR Virtual Data Environment: hello_world_2"""
+        == """:robot: **SQLMesh Bot Info** :robot:
+- :eyes: To **review** this PR's changes, use virtual data environment:
+  - `hello_world_2`"""
     )
 
     with open(github_output_file, "r", encoding="utf-8") as f:
@@ -1092,7 +1098,7 @@ def test_no_merge_since_no_deploy_signal_no_approvers_defined(
         )
 
 
-@freeze_time("2023-01-01 15:00:00")
+@time_machine.travel("2023-01-01 15:00:00 UTC")
 def test_deploy_comment_pre_categorized(
     github_client,
     make_controller,
@@ -1213,7 +1219,7 @@ def test_deploy_comment_pre_categorized(
 
 +++ 
 
-@@ -11,7 +11,8 @@
+@@ -15,7 +15,8 @@
 
  SELECT
    CAST(o.waiter_id AS INT) AS waiter_id,
@@ -1257,10 +1263,13 @@ def test_deploy_comment_pre_categorized(
     assert len(created_comments) == 1
     assert (
         created_comments[0].body
-        == f"""**SQLMesh Bot Info**
-- PR Virtual Data Environment: hello_world_2
+        == f""":robot: **SQLMesh Bot Info** :robot:
+- :eyes: To **review** this PR's changes, use virtual data environment:
+  - `hello_world_2`
+- :arrow_forward: To **apply** this PR's plan to prod, comment:
+  - `/deploy`
 <details>
-  <summary>Prod Plan Being Applied</summary>
+  <summary>:ship: Prod Plan Being Applied</summary>
 
 {expected_prod_plan}
 </details>
@@ -1276,7 +1285,7 @@ def test_deploy_comment_pre_categorized(
         )
 
 
-@freeze_time("2023-01-01 15:00:00")
+@time_machine.travel("2023-01-01 15:00:00 UTC")
 def test_error_msg_when_applying_plan_with_bug(
     github_client,
     make_controller,
@@ -1340,7 +1349,8 @@ def test_error_msg_when_applying_plan_with_bug(
     github_output_file = tmp_path / "github_output.txt"
 
     with mock.patch.dict(os.environ, {"GITHUB_OUTPUT": str(github_output_file)}):
-        command._run_all(controller)
+        with pytest.raises(CICDBotError):
+            command._run_all(controller)
 
     assert "SQLMesh - Run Unit Tests" in controller._check_run_mapping
     test_checks_runs = controller._check_run_mapping["SQLMesh - Run Unit Tests"].all_kwargs
@@ -1428,7 +1438,7 @@ def test_error_msg_when_applying_plan_with_bug(
         )
 
 
-@freeze_time("2023-01-01 15:00:00")
+@time_machine.travel("2023-01-01 15:00:00 UTC")
 def test_overlapping_changes_models(
     github_client,
     make_controller,
@@ -1566,6 +1576,7 @@ def test_overlapping_changes_models(
 ```
 
 **Indirectly Modified:**
+- `sushi.active_customers`
 - `sushi.waiter_as_customer_by_day`
 
 """
@@ -1605,7 +1616,7 @@ def test_overlapping_changes_models(
 """
     )
 
-    assert len(get_environment_objects(controller, "hello_world_2")) == 3
+    assert len(get_environment_objects(controller, "hello_world_2")) == 4
     assert "new_col" in get_columns(controller, "hello_world_2", "customers")
 
     assert mock_pull_request.merge.called
@@ -1613,10 +1624,11 @@ def test_overlapping_changes_models(
     assert len(created_comments) == 1
     assert (
         created_comments[0].body
-        == f"""**SQLMesh Bot Info**
-- PR Virtual Data Environment: hello_world_2
+        == f""":robot: **SQLMesh Bot Info** :robot:
+- :eyes: To **review** this PR's changes, use virtual data environment:
+  - `hello_world_2`
 <details>
-  <summary>Prod Plan Being Applied</summary>
+  <summary>:ship: Prod Plan Being Applied</summary>
 
 {expected_prod_plan_summary}
 </details>
@@ -1632,7 +1644,7 @@ def test_overlapping_changes_models(
         )
 
 
-@freeze_time("2023-01-01 15:00:00")
+@time_machine.travel("2023-01-01 15:00:00 UTC")
 def test_pr_delete_model(
     github_client,
     make_controller,
@@ -1791,10 +1803,11 @@ def test_pr_delete_model(
     assert len(created_comments) == 1
     assert (
         created_comments[0].body
-        == f"""**SQLMesh Bot Info**
-- PR Virtual Data Environment: hello_world_2
+        == f""":robot: **SQLMesh Bot Info** :robot:
+- :eyes: To **review** this PR's changes, use virtual data environment:
+  - `hello_world_2`
 <details>
-  <summary>Prod Plan Being Applied</summary>
+  <summary>:ship: Prod Plan Being Applied</summary>
 
 {expected_prod_plan_summary}
 </details>
