@@ -1,6 +1,4 @@
 import json
-from unittest.mock import call
-from urllib.parse import urlencode
 
 import pytest
 import requests
@@ -11,9 +9,10 @@ from sqlmesh.core.config import EnvironmentSuffixTarget
 from sqlmesh.core.environment import Environment
 from sqlmesh.core.model import IncrementalByTimeRangeKind, SqlModel
 from sqlmesh.core.node import NodeType
-from sqlmesh.core.snapshot import Snapshot, SnapshotChangeCategory, SnapshotId
+from sqlmesh.core.plan import EvaluatablePlan
+from sqlmesh.core.snapshot import Snapshot, SnapshotChangeCategory
 from sqlmesh.schedulers.airflow import common
-from sqlmesh.schedulers.airflow.client import AirflowClient, _list_to_json
+from sqlmesh.schedulers.airflow.client import AirflowClient
 from sqlmesh.utils.date import to_timestamp
 
 pytestmark = pytest.mark.airflow
@@ -59,137 +58,165 @@ def test_apply_plan(mocker: MockerFixture, snapshot: Snapshot):
         promoted_snapshot_ids=[snapshot.snapshot_id],
     )
 
-    request_id = "test_request_id"
+    plan = EvaluatablePlan(
+        start=environment.start_at,
+        end=environment.end_at,
+        new_snapshots=[snapshot],
+        environment=environment,
+        no_gaps=False,
+        skip_backfill=False,
+        empty_backfill=False,
+        restatements={snapshot.name: (to_timestamp("2024-01-01"), to_timestamp("2024-01-02"))},
+        is_dev=False,
+        allow_destructive_models=set(),
+        forward_only=False,
+        end_bounded=False,
+        ensure_finalized_snapshots=False,
+        directly_modified_snapshots=[snapshot.snapshot_id],
+        indirectly_modified_snapshots={},
+        removed_snapshots=[],
+        requires_backfill=True,
+        models_to_backfill={'"test_model"'},
+        disabled_restatement_models=set(),
+    )
 
     client = AirflowClient(airflow_url=common.AIRFLOW_LOCAL_URL, session=requests.Session())
-    client.apply_plan(
-        [snapshot],
-        environment,
-        request_id,
-        models_to_backfill={'"test_model"'},
-        directly_modified_snapshots=[snapshot.snapshot_id],
-        restatements={
-            snapshot.snapshot_id: (to_timestamp("2024-01-01"), to_timestamp("2024-01-02"))
-        },
-    )
+    client.apply_plan(plan)
 
     apply_plan_mock.assert_called_once()
     args, data = apply_plan_mock.call_args_list[0]
 
     assert args[0] == "http://localhost:8080/sqlmesh/api/v1/plans"
+    assert data["headers"] == {"Content-Type": "application/json"}
     assert json.loads(data["data"]) == {
-        "new_snapshots": [
-            {
-                "created_ts": 1665014400000,
-                "ttl": "in 1 week",
-                "fingerprint": snapshot.fingerprint.dict(),
-                "intervals": [],
-                "dev_intervals": [],
-                "node": {
-                    "audits": [],
-                    "clustered_by": [],
-                    "cron": "@daily",
-                    "dialect": "spark",
-                    "pre_statements": ["@DEF(key, " "'value')"],
-                    "kind": {
-                        "name": "INCREMENTAL_BY_TIME_RANGE",
-                        "time_column": {"column": "`ds`"},
-                        "forward_only": False,
-                        "on_destructive_change": "ERROR",
-                        "disable_restatement": False,
-                        "dialect": "spark",
-                    },
-                    "mapping_schema": {},
-                    "name": "test_model",
-                    "partitioned_by": ["`a`"],
-                    "query": "SELECT a, ds FROM tbl",
-                    "references": [],
-                    "project": "",
-                    "storage_format": "parquet",
-                    "jinja_macros": {
-                        "create_builtins_module": "sqlmesh.utils.jinja",
-                        "global_objs": {},
-                        "packages": {},
-                        "root_macros": {},
-                        "top_level_packages": [],
-                    },
-                    "source_type": "sql",
-                    "tags": [],
-                    "grains": [],
-                    "allow_partials": False,
-                    "signals": [],
-                },
-                "audits": [],
-                "name": '"test_model"',
-                "parents": [],
-                "previous_versions": [],
-                "updated_ts": 1665014400000,
-                "version": snapshot.version,
-                "change_category": snapshot.change_category,
-                "migrated": False,
-                "unrestorable": False,
-            }
-        ],
-        "environment": {
-            "name": "test_env",
-            "snapshots": [
+        "plan": {
+            "start": "2022-01-01",
+            "end": "2022-01-01",
+            "new_snapshots": [
                 {
+                    "created_ts": 1665014400000,
+                    "ttl": "in 1 week",
                     "fingerprint": snapshot.fingerprint.dict(),
+                    "intervals": [],
+                    "dev_intervals": [],
+                    "dev_table_suffix": "dev",
+                    "pending_restatement_intervals": [],
+                    "node": {
+                        "audits": [],
+                        "audit_definitions": {},
+                        "clustered_by": [],
+                        "cron": "@daily",
+                        "dialect": "spark",
+                        "pre_statements": ["@DEF(key, " "'value')"],
+                        "kind": {
+                            "name": "INCREMENTAL_BY_TIME_RANGE",
+                            "time_column": {"column": "`ds`"},
+                            "forward_only": False,
+                            "on_destructive_change": "ERROR",
+                            "disable_restatement": False,
+                            "dialect": "spark",
+                        },
+                        "mapping_schema": {},
+                        "name": "test_model",
+                        "partitioned_by": ["`a`"],
+                        "query": "SELECT a, ds FROM tbl",
+                        "references": [],
+                        "project": "",
+                        "python_env": {},
+                        "storage_format": "parquet",
+                        "jinja_macros": {
+                            "create_builtins_module": "sqlmesh.utils.jinja",
+                            "global_objs": {},
+                            "packages": {},
+                            "root_macros": {},
+                            "top_level_packages": [],
+                        },
+                        "source_type": "sql",
+                        "tags": [],
+                        "grains": [],
+                        "allow_partials": False,
+                        "signals": [],
+                        "enabled": True,
+                        "extract_dependencies_from_query": True,
+                    },
                     "name": '"test_model"',
-                    "node_type": NodeType.MODEL,
-                    "previous_versions": [],
-                    "version": snapshot.version,
-                    "physical_schema": "sqlmesh__default",
-                    "change_category": snapshot.change_category,
                     "parents": [],
-                    "kind_name": "INCREMENTAL_BY_TIME_RANGE",
+                    "previous_versions": [],
+                    "updated_ts": 1665014400000,
+                    "version": snapshot.version,
+                    "dev_version": snapshot.dev_version,
+                    "change_category": snapshot.change_category,
+                    "migrated": False,
+                    "unrestorable": False,
                 }
             ],
-            "start_at": "2022-01-01",
-            "end_at": "2022-01-01",
-            "plan_id": "test_plan_id",
-            "previous_plan_id": "previous_plan_id",
-            "promoted_snapshot_ids": [
-                {
-                    "name": '"test_model"',
-                    "identifier": snapshot.identifier,
-                }
+            "environment": {
+                "name": "test_env",
+                "snapshots": [
+                    {
+                        "fingerprint": snapshot.fingerprint.dict(),
+                        "name": '"test_model"',
+                        "node_type": NodeType.MODEL,
+                        "previous_versions": [],
+                        "version": snapshot.version,
+                        "dev_version": snapshot.dev_version,
+                        "physical_schema": "sqlmesh__default",
+                        "change_category": snapshot.change_category,
+                        "parents": [],
+                        "kind_name": "INCREMENTAL_BY_TIME_RANGE",
+                        "dev_table_suffix": "dev",
+                    }
+                ],
+                "start_at": "2022-01-01",
+                "end_at": "2022-01-01",
+                "plan_id": "test_plan_id",
+                "previous_plan_id": "previous_plan_id",
+                "promoted_snapshot_ids": [
+                    {
+                        "name": '"test_model"',
+                        "identifier": snapshot.identifier,
+                    }
+                ],
+                "suffix_target": "schema",
+                "normalize_name": True,
+                "requirements": {},
+            },
+            "no_gaps": False,
+            "skip_backfill": False,
+            "empty_backfill": False,
+            "is_dev": False,
+            "forward_only": False,
+            "allow_destructive_models": [],
+            "models_to_backfill": ['"test_model"'],
+            "end_bounded": False,
+            "ensure_finalized_snapshots": False,
+            "directly_modified_snapshots": [
+                {"identifier": snapshot.identifier, "name": '"test_model"'}
             ],
-            "suffix_target": "schema",
+            "indirectly_modified_snapshots": {},
+            "removed_snapshots": [],
+            "restatements": {
+                '"test_model"': [to_timestamp("2024-01-01"), to_timestamp("2024-01-02")]
+            },
+            "requires_backfill": True,
+            "disabled_restatement_models": [],
         },
-        "no_gaps": False,
-        "skip_backfill": False,
         "notification_targets": [],
-        "request_id": request_id,
         "backfill_concurrent_tasks": 1,
         "ddl_concurrent_tasks": 1,
         "users": [],
-        "is_dev": False,
-        "forward_only": False,
-        "allow_destructive_snapshots": [],
-        "models_to_backfill": ['"test_model"'],
-        "end_bounded": False,
-        "ensure_finalized_snapshots": False,
-        "directly_modified_snapshots": [{"identifier": "349602315", "name": '"test_model"'}],
-        "indirectly_modified_snapshots": {},
-        "removed_snapshots": [],
-        "restatements": {'"test_model"': [to_timestamp("2024-01-01"), to_timestamp("2024-01-02")]},
     }
 
     common.PlanApplicationRequest.parse_raw(data["data"])
 
 
-def snapshot_url(snapshot_ids, key="ids") -> str:
-    return urlencode({key: _list_to_json(snapshot_ids)[0]})
-
-
 def test_get_snapshots(mocker: MockerFixture, snapshot: Snapshot):
     snapshots = common.SnapshotsResponse(snapshots=[snapshot])
 
     get_snapshots_response_mock = mocker.Mock()
     get_snapshots_response_mock.status_code = 200
     get_snapshots_response_mock.json.return_value = snapshots.dict()
-    get_snapshots_mock = mocker.patch("requests.Session.get")
+    get_snapshots_mock = mocker.patch("requests.Session.post")
     get_snapshots_mock.return_value = get_snapshots_response_mock
 
     client = AirflowClient(airflow_url=common.AIRFLOW_LOCAL_URL, session=requests.Session())
@@ -197,44 +224,13 @@ def test_get_snapshots(mocker: MockerFixture, snapshot: Snapshot):
 
     assert result == [snapshot]
 
-    get_snapshots_mock.assert_called_once_with(
-        f"http://localhost:8080/sqlmesh/api/v1/snapshots?{snapshot_url([snapshot.snapshot_id])}"
-    )
-
-
-def test_get_snapshots_batching(mocker: MockerFixture, snapshot: Snapshot):
-    snapshots = common.SnapshotsResponse(snapshots=[snapshot])
-
-    get_snapshots_response_mock = mocker.Mock()
-    get_snapshots_response_mock.status_code = 200
-    get_snapshots_response_mock.json.return_value = snapshots.dict()
-    get_snapshots_mock = mocker.patch("requests.Session.get")
-    get_snapshots_mock.return_value = get_snapshots_response_mock
-
-    snapshot_ids_batch_size = 40
-    first_batch_ids = [
-        SnapshotId(name=snapshot.name, identifier=str(i)) for i in range(snapshot_ids_batch_size)
-    ]
-
-    client = AirflowClient(
-        airflow_url=common.AIRFLOW_LOCAL_URL,
-        session=requests.Session(),
-        snapshot_ids_batch_size=snapshot_ids_batch_size,
-    )
-    result = client.get_snapshots([*first_batch_ids, snapshot.snapshot_id])
-
-    assert result == [snapshot] * 2
-
-    get_snapshots_mock.assert_has_calls(
-        [
-            call(f"http://localhost:8080/sqlmesh/api/v1/snapshots?{snapshot_url(first_batch_ids)}"),
-            call().json(),
-            call(
-                f"http://localhost:8080/sqlmesh/api/v1/snapshots?{snapshot_url([snapshot.snapshot_id])}"
-            ),
-            call().json(),
-        ]
-    )
+    args, data = get_snapshots_mock.call_args_list[0]
+    assert args[0] == "http://localhost:8080/sqlmesh/api/v1/snapshots/search"
+    assert data["headers"] == {"Content-Type": "application/json"}
+    assert json.loads(data["data"]) == {
+        "snapshot_ids": [snapshot.snapshot_id.dict()],
+        "check_existence": False,
+    }
 
 
 def test_snapshots_exist(mocker: MockerFixture, snapshot: Snapshot):
@@ -243,7 +239,7 @@ def test_snapshots_exist(mocker: MockerFixture, snapshot: Snapshot):
     snapshots_exist_response_mock = mocker.Mock()
     snapshots_exist_response_mock.status_code = 200
     snapshots_exist_response_mock.json.return_value = snapshot_ids.dict()
-    snapshots_exist_mock = mocker.patch("requests.Session.get")
+    snapshots_exist_mock = mocker.patch("requests.Session.post")
     snapshots_exist_mock.return_value = snapshots_exist_response_mock
 
     client = AirflowClient(airflow_url=common.AIRFLOW_LOCAL_URL, session=requests.Session())
@@ -251,46 +247,13 @@ def test_snapshots_exist(mocker: MockerFixture, snapshot: Snapshot):
 
     assert result == {snapshot.snapshot_id}
 
-    snapshots_exist_mock.assert_called_once_with(
-        f"http://localhost:8080/sqlmesh/api/v1/snapshots?check_existence&{snapshot_url([snapshot.snapshot_id])}"
-    )
-
-
-def test_snapshots_exist_batching(mocker: MockerFixture, snapshot: Snapshot):
-    snapshot_ids = common.SnapshotIdsResponse(snapshot_ids=[snapshot.snapshot_id])
-
-    snapshots_exist_response_mock = mocker.Mock()
-    snapshots_exist_response_mock.status_code = 200
-    snapshots_exist_response_mock.json.return_value = snapshot_ids.dict()
-    snapshots_exist_mock = mocker.patch("requests.Session.get")
-    snapshots_exist_mock.return_value = snapshots_exist_response_mock
-
-    snapshot_ids_batch_size = 40
-    first_batch_ids = [
-        SnapshotId(name=snapshot.name, identifier=str(i)) for i in range(snapshot_ids_batch_size)
-    ]
-
-    client = AirflowClient(
-        airflow_url=common.AIRFLOW_LOCAL_URL,
-        session=requests.Session(),
-        snapshot_ids_batch_size=snapshot_ids_batch_size,
-    )
-    result = client.snapshots_exist([*first_batch_ids, snapshot.snapshot_id])
-
-    assert result == {snapshot.snapshot_id}
-
-    snapshots_exist_mock.assert_has_calls(
-        [
-            call(
-                f"http://localhost:8080/sqlmesh/api/v1/snapshots?check_existence&{snapshot_url(first_batch_ids)}"
-            ),
-            call().json(),
-            call(
-                f"http://localhost:8080/sqlmesh/api/v1/snapshots?check_existence&{snapshot_url([snapshot.snapshot_id])}"
-            ),
-            call().json(),
-        ]
-    )
+    args, data = snapshots_exist_mock.call_args_list[0]
+    assert args[0] == "http://localhost:8080/sqlmesh/api/v1/snapshots/search"
+    assert data["headers"] == {"Content-Type": "application/json"}
+    assert json.loads(data["data"]) == {
+        "snapshot_ids": [snapshot.snapshot_id.dict()],
+        "check_existence": True,
+    }
 
 
 def test_models_exist(mocker: MockerFixture, snapshot: Snapshot):
@@ -334,7 +297,8 @@ def test_get_environment(mocker: MockerFixture, snapshot: Snapshot):
     client = AirflowClient(airflow_url=common.AIRFLOW_LOCAL_URL, session=requests.Session())
     result = client.get_environment("dev")
 
-    assert result == environment
+    assert result is not None
+    assert result.dict() == environment.dict()
 
     get_environment_mock.assert_called_once_with(
         "http://localhost:8080/sqlmesh/api/v1/environments/dev"
@@ -361,7 +325,8 @@ def test_get_environments(mocker: MockerFixture, snapshot: Snapshot):
     client = AirflowClient(airflow_url=common.AIRFLOW_LOCAL_URL, session=requests.Session())
     result = client.get_environments()
 
-    assert result == [environment]
+    assert len(result) == 1
+    assert result[0].dict() == environment.dict()
 
     get_environments_mock.assert_called_once_with(
         "http://localhost:8080/sqlmesh/api/v1/environments"
@@ -369,55 +334,65 @@ def test_get_environments(mocker: MockerFixture, snapshot: Snapshot):
 
 
 @pytest.mark.parametrize("ensure_finalized_snapshots", [True, False])
-def test_max_interval_end_for_environment(
+def test_max_interval_end_per_model(
     mocker: MockerFixture, snapshot: Snapshot, ensure_finalized_snapshots: bool
 ):
     response = common.IntervalEndResponse(
-        environment="test_environment", max_interval_end=to_timestamp("2023-01-01")
+        environment="test_environment",
+        interval_end_per_model={"model_name": to_timestamp("2023-01-01")},
     )
 
     max_interval_end_response_mock = mocker.Mock()
     max_interval_end_response_mock.status_code = 200
     max_interval_end_response_mock.json.return_value = response.dict()
-    max_interval_end_mock = mocker.patch("requests.Session.get")
+    max_interval_end_mock = mocker.patch("requests.Session.post")
     max_interval_end_mock.return_value = max_interval_end_response_mock
 
     client = AirflowClient(airflow_url=common.AIRFLOW_LOCAL_URL, session=requests.Session())
-    result = client.max_interval_end_for_environment("test_environment", ensure_finalized_snapshots)
-
-    assert result == response.max_interval_end
-
-    flags = "?ensure_finalized_snapshots" if ensure_finalized_snapshots else ""
-    max_interval_end_mock.assert_called_once_with(
-        f"http://localhost:8080/sqlmesh/api/v1/environments/test_environment/max_interval_end{flags}"
-    )
-
-
-@pytest.mark.parametrize("ensure_finalized_snapshots", [True, False])
-def test_greatest_common_interval_end(
-    mocker: MockerFixture, snapshot: Snapshot, ensure_finalized_snapshots: bool
-):
-    response = common.IntervalEndResponse(
-        environment="test_environment", max_interval_end=to_timestamp("2023-01-01")
-    )
-
-    max_interval_end_response_mock = mocker.Mock()
-    max_interval_end_response_mock.status_code = 200
-    max_interval_end_response_mock.json.return_value = response.dict()
-    max_interval_end_mock = mocker.patch("requests.Session.get")
-    max_interval_end_mock.return_value = max_interval_end_response_mock
-
-    client = AirflowClient(airflow_url=common.AIRFLOW_LOCAL_URL, session=requests.Session())
-    result = client.greatest_common_interval_end(
+    result = client.max_interval_end_per_model(
         "test_environment", {"a.b.c"}, ensure_finalized_snapshots
     )
 
-    assert result == response.max_interval_end
+    assert result == response.interval_end_per_model
 
-    flags = "ensure_finalized_snapshots&" if ensure_finalized_snapshots else ""
-    max_interval_end_mock.assert_called_once_with(
-        f"http://localhost:8080/sqlmesh/api/v1/environments/test_environment/greatest_common_interval_end?{flags}models=%5B%22a.b.c%22%5D"
+    args, data = max_interval_end_mock.call_args_list[0]
+    assert (
+        args[0]
+        == "http://localhost:8080/sqlmesh/api/v1/environments/test_environment/max_interval_end_per_model"
     )
+    assert data["headers"] == {"Content-Type": "application/json"}
+    assert json.loads(data["data"]) == {
+        "models": ["a.b.c"],
+        "ensure_finalized_snapshots": ensure_finalized_snapshots,
+    }
+
+
+def test_max_interval_end_per_model_no_models(mocker: MockerFixture, snapshot: Snapshot):
+    response = common.IntervalEndResponse(
+        environment="test_environment",
+        interval_end_per_model={"model_name": to_timestamp("2023-01-01")},
+    )
+
+    max_interval_end_response_mock = mocker.Mock()
+    max_interval_end_response_mock.status_code = 200
+    max_interval_end_response_mock.json.return_value = response.dict()
+    max_interval_end_mock = mocker.patch("requests.Session.post")
+    max_interval_end_mock.return_value = max_interval_end_response_mock
+
+    client = AirflowClient(airflow_url=common.AIRFLOW_LOCAL_URL, session=requests.Session())
+    result = client.max_interval_end_per_model("test_environment", None, False)
+
+    assert result == response.interval_end_per_model
+
+    args, data = max_interval_end_mock.call_args_list[0]
+    assert (
+        args[0]
+        == "http://localhost:8080/sqlmesh/api/v1/environments/test_environment/max_interval_end_per_model"
+    )
+    assert data["headers"] == {"Content-Type": "application/json"}
+    assert json.loads(data["data"]) == {
+        "ensure_finalized_snapshots": False,
+    }
 
 
 def test_get_dag_run_state(mocker: MockerFixture):
@@ -463,3 +438,18 @@ def test_get_variable(mocker: MockerFixture):
     assert client.get_variable("test_key") == "test_value"
 
     get_variable_mock.assert_called_once_with("http://localhost:8080/api/v1/variables/test_key")
+
+
+def test_url_no_trailing_slash(mocker: MockerFixture, snapshot: Snapshot):
+    get_variable_response_mock = mocker.Mock()
+    get_variable_response_mock.status_code = 200
+    get_variable_response_mock.json.return_value = {"value": "test_value", "key": "test_key"}
+    get_variable_mock = mocker.patch("requests.Session.get")
+    get_variable_mock.return_value = get_variable_response_mock
+
+    client = AirflowClient(airflow_url="http://localhost:8080/prefix", session=requests.Session())
+    client.get_variable("test_key")
+
+    get_variable_mock.assert_called_once_with(
+        "http://localhost:8080/prefix/api/v1/variables/test_key"
+    )
