@@ -17,6 +17,7 @@ from sqlmesh.core.model import (
     create_sql_model,
 )
 from sqlmesh.core.node import NodeType
+from sqlmesh.core.plan.definition import EvaluatablePlan
 from sqlmesh.core.snapshot import (
     DeployabilityIndex,
     Snapshot,
@@ -26,7 +27,7 @@ from sqlmesh.core.snapshot import (
 )
 from sqlmesh.schedulers.airflow import common
 from sqlmesh.schedulers.airflow.plan import PlanDagState, create_plan_dag_spec
-from sqlmesh.utils.date import to_datetime, to_timestamp
+from sqlmesh.utils.date import to_datetime, to_timestamp, now
 from sqlmesh.utils.errors import SQLMeshError
 
 pytestmark = pytest.mark.airflow
@@ -104,17 +105,15 @@ def test_create_plan_dag_spec(
         catalog_name_override="test_catalog",
     )
 
-    plan_request = common.PlanApplicationRequest(
-        request_id="test_request_id",
+    plan = EvaluatablePlan(
+        start=new_environment.start_at,
+        end=new_environment.end_at,
         new_snapshots=[the_snapshot],
         environment=new_environment,
         no_gaps=True,
         skip_backfill=False,
+        empty_backfill=False,
         restatements={},
-        notification_targets=[],
-        backfill_concurrent_tasks=1,
-        ddl_concurrent_tasks=1,
-        users=[],
         is_dev=False,
         forward_only=True,
         models_to_backfill=None,
@@ -123,6 +122,18 @@ def test_create_plan_dag_spec(
         directly_modified_snapshots=[the_snapshot.snapshot_id],
         indirectly_modified_snapshots={},
         removed_snapshots=[],
+        interval_end_per_model=None,
+        allow_destructive_models=set(),
+        requires_backfill=True,
+        disabled_restatement_models=set(),
+    )
+
+    plan_request = common.PlanApplicationRequest(
+        plan=plan,
+        notification_targets=[],
+        backfill_concurrent_tasks=1,
+        ddl_concurrent_tasks=1,
+        users=[],
     )
 
     deleted_snapshot = SnapshotTableInfo(
@@ -134,6 +145,7 @@ def test_create_plan_dag_spec(
         change_category=SnapshotChangeCategory.BREAKING,
         kind_name=ModelKindName.FULL,
         node_type=NodeType.MODEL,
+        dev_table_suffix="dev",
     )
     old_environment = Environment(
         name=environment_name,
@@ -159,7 +171,7 @@ def test_create_plan_dag_spec(
         plan_spec = create_plan_dag_spec(plan_request, state_sync_mock)
 
     assert plan_spec == common.PlanDagSpec(
-        request_id="test_request_id",
+        request_id="test_plan_id",
         environment=new_environment,
         new_snapshots=[the_snapshot],
         backfill_intervals_per_snapshot=[
@@ -233,22 +245,20 @@ def test_restatement(
 
     the_snapshot.add_interval("2022-01-01", "2022-01-07")
 
-    plan_request = common.PlanApplicationRequest(
-        request_id="test_request_id",
+    plan = EvaluatablePlan(
+        start=new_environment.start_at,
+        end=new_environment.end_at,
         new_snapshots=[],
         environment=new_environment,
         no_gaps=True,
         skip_backfill=False,
+        empty_backfill=False,
         restatements={
             the_snapshot.name: (
                 to_timestamp("2022-01-02"),
                 to_timestamp("2022-01-04"),
             )
         },
-        notification_targets=[],
-        backfill_concurrent_tasks=1,
-        ddl_concurrent_tasks=1,
-        users=[],
         is_dev=False,
         forward_only=True,
         models_to_backfill=None,
@@ -257,6 +267,18 @@ def test_restatement(
         directly_modified_snapshots=[],
         indirectly_modified_snapshots={},
         removed_snapshots=[],
+        interval_end_per_model=None,
+        allow_destructive_models=set(),
+        requires_backfill=True,
+        disabled_restatement_models=set(),
+    )
+
+    plan_request = common.PlanApplicationRequest(
+        plan=plan,
+        notification_targets=[],
+        backfill_concurrent_tasks=1,
+        ddl_concurrent_tasks=1,
+        users=[],
     )
     old_environment = Environment(
         name=environment_name,
@@ -278,7 +300,7 @@ def test_restatement(
         plan_spec = create_plan_dag_spec(plan_request, state_sync_mock)
 
     assert plan_spec == common.PlanDagSpec(
-        request_id="test_request_id",
+        request_id="test_plan_id",
         environment=new_environment,
         new_snapshots=[],
         backfill_intervals_per_snapshot=[
@@ -308,7 +330,7 @@ def test_restatement(
     state_sync_mock.get_environment.assert_called_once()
     state_sync_mock.refresh_snapshot_intervals.assert_called_once()
 
-    state_sync_mock.remove_interval.assert_called_once_with(
+    state_sync_mock.remove_intervals.assert_called_once_with(
         [(the_snapshot, (to_timestamp("2022-01-02"), to_timestamp("2022-01-04")))],
         remove_shared_versions=True,
     )
@@ -350,17 +372,15 @@ def test_select_models_for_backfill(mocker: MockerFixture, random_name, make_sna
         suffix_target=EnvironmentSuffixTarget.TABLE,
     )
 
-    plan_request = common.PlanApplicationRequest(
-        request_id="test_request_id",
+    plan = EvaluatablePlan(
+        start=new_environment.start_at,
+        end=new_environment.end_at,
         new_snapshots=[snapshot_a, snapshot_b],
         environment=new_environment,
         no_gaps=True,
         skip_backfill=False,
+        empty_backfill=False,
         restatements={},
-        notification_targets=[],
-        backfill_concurrent_tasks=1,
-        ddl_concurrent_tasks=1,
-        users=[],
         is_dev=False,
         forward_only=True,
         models_to_backfill={snapshot_b.name},
@@ -369,6 +389,18 @@ def test_select_models_for_backfill(mocker: MockerFixture, random_name, make_sna
         directly_modified_snapshots=[snapshot_a.snapshot_id, snapshot_b.snapshot_id],
         indirectly_modified_snapshots={},
         removed_snapshots=[],
+        interval_end_per_model=None,
+        allow_destructive_models=set(),
+        requires_backfill=True,
+        disabled_restatement_models=set(),
+    )
+
+    plan_request = common.PlanApplicationRequest(
+        plan=plan,
+        notification_targets=[],
+        backfill_concurrent_tasks=1,
+        ddl_concurrent_tasks=1,
+        users=[],
     )
 
     state_sync_mock = mocker.Mock()
@@ -384,7 +416,7 @@ def test_select_models_for_backfill(mocker: MockerFixture, random_name, make_sna
         plan_spec = create_plan_dag_spec(plan_request, state_sync_mock)
 
     assert plan_spec == common.PlanDagSpec(
-        request_id="test_request_id",
+        request_id="test_plan_id",
         environment=new_environment,
         new_snapshots=[snapshot_a, snapshot_b],
         backfill_intervals_per_snapshot=[
@@ -426,17 +458,15 @@ def test_create_plan_dag_spec_duplicated_snapshot(
         plan_id="test_plan_id",
     )
 
-    plan_request = common.PlanApplicationRequest(
-        request_id="test_request_id",
+    plan = EvaluatablePlan(
+        start=new_environment.start_at,
+        end=new_environment.end_at,
         new_snapshots=[snapshot],
         environment=new_environment,
         no_gaps=False,
         skip_backfill=False,
+        empty_backfill=False,
         restatements={},
-        notification_targets=[],
-        backfill_concurrent_tasks=1,
-        ddl_concurrent_tasks=1,
-        users=[],
         is_dev=False,
         forward_only=False,
         models_to_backfill=None,
@@ -445,6 +475,18 @@ def test_create_plan_dag_spec_duplicated_snapshot(
         directly_modified_snapshots=[],
         indirectly_modified_snapshots={},
         removed_snapshots=[],
+        interval_end_per_model=None,
+        allow_destructive_models=set(),
+        requires_backfill=True,
+        disabled_restatement_models=set(),
+    )
+
+    plan_request = common.PlanApplicationRequest(
+        plan=plan,
+        notification_targets=[],
+        backfill_concurrent_tasks=1,
+        ddl_concurrent_tasks=1,
+        users=[],
     )
 
     dag_run_mock = mocker.Mock()
@@ -479,17 +521,15 @@ def test_create_plan_dag_spec_unbounded_end(
         plan_id="test_plan_id",
     )
 
-    plan_request = common.PlanApplicationRequest(
-        request_id="test_request_id",
+    plan = EvaluatablePlan(
+        start=new_environment.start_at,
+        end=now(),
         new_snapshots=[],
         environment=new_environment,
         no_gaps=True,
         skip_backfill=False,
+        empty_backfill=False,
         restatements={},
-        notification_targets=[],
-        backfill_concurrent_tasks=1,
-        ddl_concurrent_tasks=1,
-        users=[],
         is_dev=False,
         forward_only=False,
         models_to_backfill=None,
@@ -498,6 +538,18 @@ def test_create_plan_dag_spec_unbounded_end(
         directly_modified_snapshots=[],
         indirectly_modified_snapshots={},
         removed_snapshots=[],
+        interval_end_per_model=None,
+        allow_destructive_models=set(),
+        requires_backfill=True,
+        disabled_restatement_models=set(),
+    )
+
+    plan_request = common.PlanApplicationRequest(
+        plan=plan,
+        notification_targets=[],
+        backfill_concurrent_tasks=1,
+        ddl_concurrent_tasks=1,
+        users=[],
     )
 
     state_sync_mock = mocker.Mock()
@@ -526,7 +578,7 @@ def test_plan_dag_state(snapshot: Snapshot, sushi_context: Context, random_name)
         plan_id="test_plan_id",
     )
     plan_dag_spec = common.PlanDagSpec(
-        request_id="test_request_id",
+        request_id="test_plan_id",
         environment=environment,
         new_snapshots=[],
         backfill_intervals_per_snapshot=[],
@@ -545,15 +597,21 @@ def test_plan_dag_state(snapshot: Snapshot, sushi_context: Context, random_name)
 
     plan_dag_state = PlanDagState.from_state_sync(sushi_context.state_sync)
 
+    def get_hydrated_dag_specs():
+        state_plan_dag_specs = plan_dag_state.get_dag_specs()
+        for state_plan_dag_spec in state_plan_dag_specs:
+            state_plan_dag_spec.environment.snapshots
+        return state_plan_dag_specs
+
     assert not plan_dag_state.get_dag_specs()
 
     plan_dag_state.add_dag_spec(plan_dag_spec)
-    assert plan_dag_state.get_dag_specs() == [plan_dag_spec]
+    assert get_hydrated_dag_specs() == [plan_dag_spec]
 
     plan_dag_state.delete_dag_specs([])
-    assert plan_dag_state.get_dag_specs() == [plan_dag_spec]
+    assert get_hydrated_dag_specs() == [plan_dag_spec]
 
     plan_dag_state.delete_dag_specs(
-        [common.plan_application_dag_id(environment_name, "test_request_id")]
+        [common.plan_application_dag_id(environment_name, "test_plan_id")]
     )
     assert not plan_dag_state.get_dag_specs()
