@@ -5,24 +5,26 @@ from pathlib import Path
 
 import pyarrow as pa  # type: ignore
 import pytest
+from fastapi import FastAPI
 from fastapi.testclient import TestClient
-from httpx import AsyncClient
+from httpx import ASGITransport, AsyncClient
 from pytest_mock.plugin import MockerFixture
 
 from sqlmesh.core.context import Context
 from sqlmesh.core.environment import Environment
 from sqlmesh.utils.errors import PlanError
 from web.server.api.endpoints.files import _get_file_with_content
-from web.server.main import app
 from web.server.settings import get_settings
 
 pytestmark = pytest.mark.web
 
 
-client = TestClient(app)
+@pytest.fixture
+def client(web_app: FastAPI) -> TestClient:
+    return TestClient(web_app)
 
 
-def test_get_files(project_tmp_path: Path) -> None:
+def test_get_files(client: TestClient, project_tmp_path: Path) -> None:
     models_dir = project_tmp_path / "models"
     models_dir.mkdir()
     sql_file = models_dir / "foo.sql"
@@ -59,7 +61,7 @@ def test_get_files(project_tmp_path: Path) -> None:
     }
 
 
-def test_get_file(project_tmp_path: Path) -> None:
+def test_get_file(client: TestClient, project_tmp_path: Path) -> None:
     txt_file = project_tmp_path / "foo.txt"
     txt_file.write_text("bar")
 
@@ -73,12 +75,12 @@ def test_get_file(project_tmp_path: Path) -> None:
     }
 
 
-def test_get_file_not_found() -> None:
+def test_get_file_not_found(client: TestClient) -> None:
     response = client.get("/api/files/not_found.txt")
     assert response.status_code == 404
 
 
-def test_get_file_invalid_path(project_tmp_path: Path) -> None:
+def test_get_file_invalid_path(client: TestClient, project_tmp_path: Path) -> None:
     config = project_tmp_path / "config.py"
     config.write_text(
         """from sqlmesh.core.config import Config, ModelDefaultsConfig
@@ -92,7 +94,7 @@ config = Config(ignore_patterns=["*.txt"], model_defaults=ModelDefaultsConfig(di
     assert response.status_code == 404
 
 
-def test_write_file(project_tmp_path: Path) -> None:
+def test_write_file(client: TestClient, project_tmp_path: Path) -> None:
     response = client.post("/api/files/foo.txt", json={"content": "bar"})
     file = _get_file_with_content(project_tmp_path / "foo.txt", "foo.txt")
     assert response.status_code == 204
@@ -104,7 +106,7 @@ def test_write_file(project_tmp_path: Path) -> None:
     }
 
 
-def test_update_file(project_tmp_path: Path) -> None:
+def test_update_file(client: TestClient, project_tmp_path: Path) -> None:
     txt_file = project_tmp_path / "foo.txt"
     txt_file.write_text("bar")
 
@@ -119,7 +121,7 @@ def test_update_file(project_tmp_path: Path) -> None:
     }
 
 
-def test_rename_file(project_tmp_path: Path) -> None:
+def test_rename_file(client: TestClient, project_tmp_path: Path) -> None:
     txt_file = project_tmp_path / "foo.txt"
     txt_file.write_text("bar")
 
@@ -135,7 +137,7 @@ def test_rename_file(project_tmp_path: Path) -> None:
     assert not txt_file.exists()
 
 
-def test_rename_file_and_keep_content(project_tmp_path: Path) -> None:
+def test_rename_file_and_keep_content(client: TestClient, project_tmp_path: Path) -> None:
     txt_file = project_tmp_path / "foo.txt"
     txt_file.write_text("bar")
 
@@ -153,12 +155,12 @@ def test_rename_file_and_keep_content(project_tmp_path: Path) -> None:
     assert not txt_file.exists()
 
 
-def test_rename_file_not_found(project_tmp_path: Path) -> None:
+def test_rename_file_not_found(client: TestClient, project_tmp_path: Path) -> None:
     response = client.post("/api/files/foo.txt", json={"new_path": "baz.txt"})
     assert response.status_code == 404
 
 
-def test_rename_file_already_exists(project_tmp_path: Path) -> None:
+def test_rename_file_already_exists(client: TestClient, project_tmp_path: Path) -> None:
     foo_file = project_tmp_path / "foo.txt"
     foo_file.write_text("foo")
     bar_file = project_tmp_path / "bar.txt"
@@ -176,7 +178,7 @@ def test_rename_file_already_exists(project_tmp_path: Path) -> None:
     assert not foo_file.exists()
 
 
-def test_rename_file_to_existing_directory(project_tmp_path: Path) -> None:
+def test_rename_file_to_existing_directory(client: TestClient, project_tmp_path: Path) -> None:
     foo_file = project_tmp_path / "foo.txt"
     foo_file.touch()
     existing_dir = project_tmp_path / "existing_dir"
@@ -187,12 +189,12 @@ def test_rename_file_to_existing_directory(project_tmp_path: Path) -> None:
     assert foo_file.exists()
 
 
-def test_write_file_empty_body(project_tmp_path: Path) -> None:
+def test_write_file_empty_body(client: TestClient, project_tmp_path: Path) -> None:
     response = client.post("/api/files/foo.txt", json={})
     assert response.status_code == 204
 
 
-def test_delete_file(project_tmp_path: Path) -> None:
+def test_delete_file(client: TestClient, project_tmp_path: Path) -> None:
     txt_file = project_tmp_path / "foo.txt"
     txt_file.write_text("bar")
 
@@ -201,19 +203,19 @@ def test_delete_file(project_tmp_path: Path) -> None:
     assert not txt_file.exists()
 
 
-def test_delete_file_not_found(project_tmp_path: Path) -> None:
+def test_delete_file_not_found(client: TestClient, project_tmp_path: Path) -> None:
     response = client.delete("/api/files/not_found.txt")
     assert response.status_code == 404
 
 
-def test_create_directory(project_tmp_path: Path) -> None:
+def test_create_directory(client: TestClient, project_tmp_path: Path) -> None:
     response = client.post("/api/directories/new_dir")
     assert response.status_code == 200
     assert (project_tmp_path / "new_dir").exists()
     assert response.json() == {"directories": [], "files": [], "name": "new_dir", "path": "new_dir"}
 
 
-def test_create_directory_already_exists(project_tmp_path: Path) -> None:
+def test_create_directory_already_exists(client: TestClient, project_tmp_path: Path) -> None:
     new_dir = project_tmp_path / "new_dir"
     new_dir.mkdir()
 
@@ -222,7 +224,7 @@ def test_create_directory_already_exists(project_tmp_path: Path) -> None:
     assert response.json()["message"] == "Directory already exists"
 
 
-def test_rename_directory(project_tmp_path: Path) -> None:
+def test_rename_directory(client: TestClient, project_tmp_path: Path) -> None:
     new_dir = project_tmp_path / "new_dir"
     new_dir.mkdir()
 
@@ -238,7 +240,7 @@ def test_rename_directory(project_tmp_path: Path) -> None:
     }
 
 
-def test_rename_directory_already_exists_empty(project_tmp_path: Path) -> None:
+def test_rename_directory_already_exists_empty(client: TestClient, project_tmp_path: Path) -> None:
     new_dir = project_tmp_path / "new_dir"
     new_dir.mkdir()
     existing_dir = project_tmp_path / "renamed_dir"
@@ -256,7 +258,9 @@ def test_rename_directory_already_exists_empty(project_tmp_path: Path) -> None:
     }
 
 
-def test_rename_directory_already_exists_not_empty(project_tmp_path: Path) -> None:
+def test_rename_directory_already_exists_not_empty(
+    client: TestClient, project_tmp_path: Path
+) -> None:
     new_dir = project_tmp_path / "new_dir"
     new_dir.mkdir()
     existing_dir = project_tmp_path / "renamed_dir"
@@ -270,7 +274,7 @@ def test_rename_directory_already_exists_not_empty(project_tmp_path: Path) -> No
     assert new_dir.exists()
 
 
-def test_rename_directory_to_existing_file(project_tmp_path: Path) -> None:
+def test_rename_directory_to_existing_file(client: TestClient, project_tmp_path: Path) -> None:
     new_dir = project_tmp_path / "new_dir"
     new_dir.mkdir()
     existing_file = project_tmp_path / "foo.txt"
@@ -282,7 +286,7 @@ def test_rename_directory_to_existing_file(project_tmp_path: Path) -> None:
     assert new_dir.exists()
 
 
-def test_delete_directory(project_tmp_path: Path) -> None:
+def test_delete_directory(client: TestClient, project_tmp_path: Path) -> None:
     new_dir = project_tmp_path / "new_dir"
     new_dir.mkdir()
 
@@ -291,12 +295,12 @@ def test_delete_directory(project_tmp_path: Path) -> None:
     assert not new_dir.exists()
 
 
-def test_delete_directory_not_found(project_tmp_path: Path) -> None:
+def test_delete_directory_not_found(client: TestClient, project_tmp_path: Path) -> None:
     response = client.delete("/api/directories/fake_dir")
     assert response.status_code == 404
 
 
-def test_delete_directory_not_a_directory(project_tmp_path: Path) -> None:
+def test_delete_directory_not_a_directory(client: TestClient, project_tmp_path: Path) -> None:
     txt_file = project_tmp_path / "foo.txt"
     txt_file.touch()
 
@@ -305,7 +309,7 @@ def test_delete_directory_not_a_directory(project_tmp_path: Path) -> None:
     assert response.json()["message"] == "Not a directory"
 
 
-def test_delete_directory_not_empty(project_tmp_path: Path) -> None:
+def test_delete_directory_not_empty(client: TestClient, project_tmp_path: Path) -> None:
     new_dir = project_tmp_path / "new_dir"
     new_dir.mkdir()
     (new_dir / "foo.txt").touch()
@@ -315,7 +319,7 @@ def test_delete_directory_not_empty(project_tmp_path: Path) -> None:
     assert not new_dir.exists()
 
 
-def test_apply(project_tmp_path: Path) -> None:
+def test_apply(client: TestClient, project_tmp_path: Path) -> None:
     models_dir = project_tmp_path / "models"
     models_dir.mkdir()
     sql_file = models_dir / "foo.sql"
@@ -329,14 +333,16 @@ def test_apply(project_tmp_path: Path) -> None:
 @pytest.mark.skip(
     reason="needs to be fixed: plan tests are failing inside coroutine and won't throw 422"
 )
-def test_apply_test_failures(web_sushi_context: Context, mocker: MockerFixture) -> None:
+def test_apply_test_failures(
+    client: TestClient, web_sushi_context: Context, mocker: MockerFixture
+) -> None:
     mocker.patch.object(web_sushi_context, "_run_plan_tests", side_effect=PlanError())
     response = client.post("/api/commands/apply", json={"environment": "dev"})
     assert response.status_code == 422
     assert response.json()["message"] == "Unable to run a plan"
 
 
-def test_plan(web_sushi_context: Context) -> None:
+def test_plan(client: TestClient, web_sushi_context: Context) -> None:
     client.app.state.circuit_breaker = threading.Event()  # type: ignore
     response = client.post("/api/plan", json={"environment": "dev"})
     assert response.status_code == 204
@@ -345,7 +351,9 @@ def test_plan(web_sushi_context: Context) -> None:
 @pytest.mark.skip(
     reason="needs to be fixed: plan tests are failing inside coroutine and won't throw 422"
 )
-def test_plan_test_failures(web_sushi_context: Context, mocker: MockerFixture) -> None:
+def test_plan_test_failures(
+    client: TestClient, web_sushi_context: Context, mocker: MockerFixture
+) -> None:
     mocker.patch.object(web_sushi_context, "_run_plan_tests", side_effect=PlanError())
     response = client.post("/api/plan", json={"environment": "dev"})
     assert response.status_code == 422
@@ -353,21 +361,22 @@ def test_plan_test_failures(web_sushi_context: Context, mocker: MockerFixture) -
 
 
 @pytest.mark.asyncio
-async def test_cancel() -> None:
+async def test_cancel(client: TestClient) -> None:
     client.app.state.circuit_breaker = threading.Event()  # type: ignore
-    async with AsyncClient(app=app, base_url="http://testserver") as _client:
+    transport = ASGITransport(client.app)  # type: ignore
+    async with AsyncClient(transport=transport, base_url="http://testserver") as _client:
         await _client.post("/api/plan", json={"environment": "dev"})
         response = await _client.post("/api/plan/cancel")
     assert response.status_code == 204
-    assert app.state.task.cancelled()
+    assert client.app.state.task.cancelled()  # type: ignore
 
 
-def test_cancel_no_task() -> None:
+def test_cancel_no_task(client: TestClient) -> None:
     response = client.post("/api/plan/cancel")
     assert response.status_code == 204
 
 
-def test_evaluate(web_sushi_context: Context) -> None:
+def test_evaluate(client: TestClient, web_sushi_context: Context) -> None:
     response = client.post(
         "/api/commands/evaluate",
         json={
@@ -384,7 +393,7 @@ def test_evaluate(web_sushi_context: Context) -> None:
     assert not df.empty
 
 
-def test_meta() -> None:
+def test_meta(client: TestClient) -> None:
     from sqlmesh.cli.main import _sqlmesh_version
 
     response = client.get("/api/meta")
@@ -392,14 +401,14 @@ def test_meta() -> None:
     assert response.json() == {"version": _sqlmesh_version(), "has_running_task": False}
 
 
-def test_modules() -> None:
+def test_modules(client: TestClient) -> None:
     settings = get_settings()
     response = client.get("/api/modules")
     assert response.status_code == 200
     assert response.json() == list(settings.modules)
 
 
-def test_fetchdf(web_sushi_context: Context) -> None:
+def test_fetchdf(client: TestClient, web_sushi_context: Context) -> None:
     response = client.post("/api/commands/fetchdf", json={"sql": "SELECT * from sushi.top_waiters"})
     assert response.status_code == 200
     with pa.ipc.open_stream(response.content) as reader:
@@ -407,7 +416,7 @@ def test_fetchdf(web_sushi_context: Context) -> None:
     assert not df.empty
 
 
-def test_get_model(web_sushi_context: Context) -> None:
+def test_get_model(client: TestClient, web_sushi_context: Context) -> None:
     response = client.get("/api/models/sushi.customers")
 
     assert response.status_code == 200
@@ -429,7 +438,7 @@ def test_get_model(web_sushi_context: Context) -> None:
 
 
 # TODO: add better tests for this endpoint
-def test_get_models(web_sushi_context: Context) -> None:
+def test_get_models(client: TestClient, web_sushi_context: Context) -> None:
     response = client.get("/api/models")
 
     assert response.status_code == 200
@@ -442,19 +451,19 @@ def test_get_models(web_sushi_context: Context) -> None:
     assert test_model.get("columns")
 
 
-def test_render(web_sushi_context: Context) -> None:
+def test_render(client: TestClient, web_sushi_context: Context) -> None:
     response = client.post("/api/commands/render", json={"model": "sushi.items"})
     assert response.status_code == 200
     assert response.json()["sql"]
 
 
-def test_render_invalid_model(web_sushi_context: Context) -> None:
+def test_render_invalid_model(client: TestClient, web_sushi_context: Context) -> None:
     response = client.post("/api/commands/render", json={"model": "foo.bar"})
     assert response.status_code == 422
     assert response.json()["message"] == "Unable to find a model"
 
 
-def test_get_environments(project_context: Context) -> None:
+def test_get_environments(client: TestClient, project_context: Context) -> None:
     response = client.get("/api/environments")
     assert response.status_code == 200
     response_json = response.json()
@@ -471,13 +480,15 @@ def test_get_environments(project_context: Context) -> None:
     )
 
 
-def test_delete_environment_success(web_sushi_context: Context):
+def test_delete_environment_success(client: TestClient, web_sushi_context: Context):
     response = client.delete("/api/environments/test")
 
     assert response.status_code == 204
 
 
-def test_delete_environment_failure(web_sushi_context: Context, mocker: MockerFixture):
+def test_delete_environment_failure(
+    client: TestClient, web_sushi_context: Context, mocker: MockerFixture
+):
     mocker.patch.object(
         web_sushi_context.state_sync, "invalidate_environment", side_effect=Exception("Some error")
     )
@@ -488,7 +499,7 @@ def test_delete_environment_failure(web_sushi_context: Context, mocker: MockerFi
     assert response.json()["message"] == "Unable to delete environments"
 
 
-def test_table_diff(web_sushi_context: Context) -> None:
+def test_table_diff(client: TestClient, web_sushi_context: Context) -> None:
     web_sushi_context.plan(
         "dev",
         no_prompts=True,
@@ -509,7 +520,7 @@ def test_table_diff(web_sushi_context: Context) -> None:
     assert "row_diff" in response.json()
 
 
-def test_test(web_sushi_context: Context) -> None:
+def test_test(client: TestClient, web_sushi_context: Context) -> None:
     response = client.get("/api/commands/test")
     assert response.status_code == 200
     response_json = response.json()
@@ -524,7 +535,7 @@ def test_test(web_sushi_context: Context) -> None:
     assert response_json["failures"] == []
 
 
-def test_test_failure(project_context: Context) -> None:
+def test_test_failure(client: TestClient, project_context: Context) -> None:
     models_dir = project_context.path / "models"
     models_dir.mkdir()
     sql_file = models_dir / "foo.sql"
