@@ -4,6 +4,7 @@ import asyncio
 import logging
 import os
 import typing as t
+from contextlib import asynccontextmanager
 from functools import lru_cache
 from pathlib import Path
 
@@ -24,18 +25,16 @@ MODE_TO_MODULES = {
     models.Mode.IDE: {
         models.Modules.EDITOR,
         models.Modules.FILES,
-        models.Modules.DOCS,
+        models.Modules.DATA_CATALOG,
         models.Modules.ERRORS,
         models.Modules.PLANS,
     },
-    models.Mode.DOCS: {models.Modules.DOCS, models.Modules.ERRORS},
-    models.Mode.PLAN: {models.Modules.PLANS, models.Modules.LINEAGE, models.Modules.ERRORS},
-    models.Mode.DEFAULT: {
-        models.Modules.DOCS,
+    models.Mode.CATALOG: {models.Modules.DATA_CATALOG},
+    models.Mode.PLAN: {
         models.Modules.PLANS,
+        models.Modules.DATA_CATALOG,
+        models.Modules.LINEAGE,
         models.Modules.ERRORS,
-        models.Modules.EDITOR,
-        models.Modules.FILES,
     },
 }
 
@@ -47,12 +46,14 @@ class Settings(PydanticModel):
     config: str = Field(default_factory=lambda: os.getenv("CONFIG", ""))
     gateway: t.Optional[str] = Field(default_factory=lambda: os.getenv("GATEWAY"))
     ui_mode: Mode = Field(
-        default_factory=lambda: Mode[os.getenv("UI_MODE", Mode.DEFAULT.value).upper()]
+        default_factory=lambda: Mode[os.getenv("UI_MODE", Mode.IDE.value).upper()]
     )
 
     @property
     def modules(self) -> t.Set[models.Modules]:
-        return MODE_TO_MODULES[self.ui_mode]
+        return MODE_TO_MODULES[
+            models.Mode.CATALOG if self.ui_mode == models.Mode.DOCS else self.ui_mode
+        ]
 
 
 @lru_cache()
@@ -62,9 +63,11 @@ def get_settings() -> Settings:
 
 @lru_cache()
 def _get_context(path: str | Path, config: str, gateway: str) -> Context:
+    from sqlmesh.core.console import set_console
     from web.server.main import api_console
 
-    return Context(paths=str(path), config=config, console=api_console, gateway=gateway, load=False)
+    set_console(api_console)
+    return Context(paths=str(path), config=config, gateway=gateway, load=False)
 
 
 @lru_cache()
@@ -83,19 +86,19 @@ async def get_path_to_model_mapping(
     settings: Settings = Depends(get_settings),
 ) -> dict[Path, Model]:
     try:
-        context = await get_loaded_context(settings)
+        async with asynccontextmanager(get_loaded_context)(settings) as context:
+            return _get_path_to_model_mapping(context)
     except Exception:
         logger.exception("Error creating a context")
         return {}
-    return _get_path_to_model_mapping(context)
 
 
-async def get_loaded_context(settings: Settings = Depends(get_settings)) -> Context:
+async def get_loaded_context(settings: Settings = Depends(get_settings)) -> t.AsyncGenerator:
     loop = asyncio.get_running_loop()
 
     try:
         async with get_context_lock:
-            return await loop.run_in_executor(
+            yield await loop.run_in_executor(
                 None,
                 _get_loaded_context,
                 settings.project_path,
